@@ -1,6 +1,7 @@
 #include <config.h>
 
 #include "combo.h"
+#include "debug.h"
 
 void popbrwsr_cb (Fl_Widget *v, long d);
 
@@ -8,7 +9,7 @@ Fl_PopBrowser::Fl_PopBrowser (int X, int Y, int W, int H, retvals R)
  : Fl_Window (X, Y, W, H, "")
 {
   Rvals = R;
-  hRow  = H-9;
+  hRow  = H;
   wRow  = W;
   clear_border();
   box(FL_BORDER_BOX);
@@ -16,11 +17,11 @@ Fl_PopBrowser::Fl_PopBrowser (int X, int Y, int W, int H, retvals R)
   popbrwsr->callback ( (Fl_Callback*)popbrwsr_cb);
   parent = 0;
   end();
+  set_modal();
 }
 
 Fl_PopBrowser::~Fl_PopBrowser ()
 {
-	if (popbrwsr) delete popbrwsr;
 }
 
 int Fl_PopBrowser::handle(int event)
@@ -50,19 +51,22 @@ void Fl_PopBrowser::sort()
 void Fl_PopBrowser::popshow (int x, int y)
 {
   int nRows = popbrwsr->size();
-  int height = (nRows > 10 ? 10 : nRows)*hRow + 4;
+  int height = (nRows > 12 ? 12 : nRows)*hRow;// + 4;
+
   if (nRows == 0) return;
   popbrwsr->resize (0, 0, wRow, height);
   resize (x, y, wRow, height);
 // locate first occurance of Output string value in the list
 // and display that if found
-  int i = 0;
-  while (	i < parent->listsize && 
-			strcasecmp (
-				parent->Output->value(),
-				parent->datalist[i]->s ) != 0 
-		) i++;
-  if (i > parent->listsize) i = 0;
+  int i = parent->index();
+  if (!(i >= 0 && i < parent->listsize)) {
+	  for (i = 0; i < parent->listsize; i++)
+		  if (!strcmp(parent->Output->value(), parent->datalist[i]->s))
+			  break;
+	  if (i == parent->listsize)
+		  i = 0;
+  }
+
   show ();
   popbrwsr->topline (i+1);
 
@@ -81,13 +85,14 @@ void Fl_PopBrowser::popbrwsr_cb_i (Fl_Widget *v, long d)
 //  Fl_Output *tgt = me->Rvals.Inp;
   Fl_Input *tgt = me->Rvals.Inp;
 // update the return values
+  int row = (me->popbrwsr)->value();
+  if (row == 0) return;
+  me->popbrwsr->deselect();
+
   if (tgt) {
-    int row = (me->popbrwsr)->value();
-    if (row > 0) {
-      tgt->value ((me->popbrwsr)->text (row));
-      me->Rvals.retval = (me->popbrwsr)->data (row);
-      *(me->Rvals.idx) = row;
-    }
+    tgt->value ((me->popbrwsr)->text (row));
+    me->Rvals.retval = (me->popbrwsr)->data (row);
+    *(me->Rvals.idx) = row - 1;
   }
   me->pophide();
 // user selected an item from the browser list, so execute the
@@ -103,29 +108,6 @@ void popbrwsr_cb (Fl_Widget *v, long d)
   return;
 }
 
-/* XPM Select_Browser Label */
-static const char *brws_12[] = {
-/* width height num_colors chars_per_pixel */
-"    12    12        3            1",
-/* colors */
-". c #000000",
-"d c none",
-"e c #ffffff",
-/* pixels */
-"dddddddddddd",
-"dddddddddddd",
-"dddddddddddd",
-"............",
-"d..........d",
-"dd........dd",
-"ddd......ddd",
-"dddd....dddd",
-"ddddd..ddddd",
-"dddddddddddd",
-"dddddddddddd",
-"dddddddddddd",
-};
-
 
 void Fl_ComboBox::fl_popbrwsr(Fl_Widget *p)
 {
@@ -166,12 +148,11 @@ void btnComboBox_cb (Fl_Widget *v, void *d)
 Fl_ComboBox::Fl_ComboBox (int X,int Y,int W,int H, const char *L)
  : Fl_Group (X, Y, W, H, L)
 {
-  width = W; height = H;
-  Btn = new Fl_Button (X + W - 18, Y + 1, 18, H - 2);
-  (new Fl_Pixmap (brws_12))->label (Btn);
+  width = W; height = H - 4;
+  Btn = new Fl_Button (X + W - 18, Y + 1, 18, H - 2, "@-12>");
   Btn->callback ((Fl_Callback *)btnComboBox_cb, 0);
-//  Output = new Fl_Output (X, Y, W-18, H);
   Output = new Fl_Input (X, Y, W-18, H);
+
   Brwsr = 0;
   datalist = new datambr *[FL_COMBO_LIST_INCR];
   maxsize = FL_COMBO_LIST_INCR;
@@ -184,6 +165,18 @@ Fl_ComboBox::Fl_ComboBox (int X,int Y,int W,int H, const char *L)
   R.idx = &idx;
 }
 
+Fl_ComboBox::~Fl_ComboBox()
+{
+	if (Brwsr) delete Brwsr;
+	for (int i = 0; i < listsize; i++) {
+		if (datalist[i]) {
+			if (datalist[i]->s) delete [] datalist[i]->s;
+			delete datalist[i];
+		}
+	}
+	delete [] datalist;
+}
+
 void Fl_ComboBox::type (int t)
 {
   listtype = t;
@@ -222,7 +215,7 @@ void Fl_ComboBox::put_value(const char *s)
 void Fl_ComboBox::index(int i)
 {
 	if (i >= 0 && i < listsize)
-		Output->value( datalist[i]->s);
+		Output->value( datalist[idx = i]->s);
 }
 
 
@@ -269,8 +262,8 @@ void Fl_ComboBox::add( const char *s, void * d)
   if (listsize == maxsize) {
     int nusize = maxsize + FL_COMBO_LIST_INCR;
     datambr **temparray = new datambr *[nusize];
-    for (int i = 0; i < listsize; i++) temparray[i] = datalist[i];
-    delete [] datalist;
+    for (int i = 0; i < listsize; i++)	temparray[i] = datalist[i];
+	delete [] datalist;
     datalist = temparray;
     maxsize = nusize;
   }
