@@ -29,11 +29,11 @@
 
 #include <config.h>
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include <cstring>
 
-#include <sys/time.h>
+#include "digiscope.h"
+#include "waterfall.h"
+#include "fl_digi.h"
 
 #include "cw.h"
 #include "misc.h"
@@ -42,12 +42,15 @@
 #include "status.h"
 #include "debug.h"
 
+#define	DEC_RATIO	16
+#define CW_FIRLEN   64
+
 void cw::tx_init(SoundBase *sc)
 {
 	scard = sc;
 	phaseacc = 0;
 	lastsym = 0;
-	qrqphase = 0;
+	qskphase = 0;
 }
 
 void cw::rx_init()
@@ -56,10 +59,10 @@ void cw::rx_init()
 	smpl_ctr = 0;				
 	cw_rr_current = 0;			
 	agc_peak = 0;	
-	set_scope_mode(Digiscope::SCOPE);
+    set_scope_mode(Digiscope::SCOPE);
 	put_MODEstatus(mode);
 	usedefaultWPM = false;
-	set_AFCind(0.0);
+	scope_clear = true;
 }
 
 void cw::init()
@@ -69,19 +72,14 @@ void cw::init()
 	cw_adaptive_receive_threshold = (long int)trackingfilter->run(2 * cw_send_dot_length);
 	put_cwRcvWPM(cw_send_speed);
 	for (int i = 0; i < OUTBUFSIZE; i++)
-		outbuf[i] = qrqbuf[i] = 0.0;
+		outbuf[i] = qskbuf[i] = 0.0;
 	rx_init();
 }
 
 cw::~cw() {
-//	if (KeyLine) {
-//		delete KeyLine;
-//		KeyLine = (modeIO *)0;
-//	}
 	if (cwfilter) delete cwfilter;
 	if (bitfilter) delete bitfilter;
 	if (trackingfilter) delete trackingfilter;
-//	if (keyshape) delete [] keyshape;
 }
 
 
@@ -90,12 +88,15 @@ cw::cw() : modem()
 {
 	double lp;
 
+	cap |= CAP_BW;
+
 	mode = MODE_CW;
 	freqlock = false;
 	usedefaultWPM = false;
 	frequency = progdefaults.CWsweetspot;
 	tx_frequency = get_txfreq_woffset();
 	risetime = progdefaults.CWrisetime;
+	QSKshape = progdefaults.QSKshape;
 	
 	samplerate = CWSampleRate;
 	fragmentsize = CWMaxSymLen;
@@ -109,7 +110,7 @@ cw::cw() : modem()
 	cw_noise_spike_threshold = cw_adaptive_receive_threshold / 4;
 	cw_send_dot_length = DOT_MAGIC / cw_send_speed;
 	cw_send_dash_length = 3 * cw_send_dot_length;
-	symbollen = (int)(1.0 * samplerate * cw_send_dot_length / USECS_PER_SEC);
+	symbollen = (samplerate * 12) / (progdefaults.CWspeed * 10);
 	
 	memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
 
@@ -125,7 +126,7 @@ cw::cw() : modem()
 	cwfilter->init_lowpass (CW_FIRLEN, DEC_RATIO, lp);
 	
 	bitfilter = new Cmovavg(8);
-	bitfilterlen = (int)(samplerate / frequency / 2);
+	bitfilterlen = (int)(samplerate / frequency / 4);
 	bitfilterlen = bitfilterlen < 2 ? 2 : bitfilterlen;
 	bitfilter->setLength(bitfilterlen);
 
@@ -152,14 +153,17 @@ void cw::sync_parameters()
 		cw_send_dot_length = DOT_MAGIC / progdefaults.defCWspeed;
 		
 	cw_send_dash_length = 3 * cw_send_dot_length;
-	nusymbollen = (int)(1.0 * samplerate * cw_send_dot_length / USECS_PER_SEC);
+	nusymbollen = (samplerate * 12) / (progdefaults.CWspeed * 10);
 
-	if (symbollen != nusymbollen || risetime != progdefaults.CWrisetime) {
+	if (symbollen != nusymbollen
+	    || risetime != progdefaults.CWrisetime ||
+	    QSKshape != progdefaults.QSKshape ) {
 		risetime = progdefaults.CWrisetime;
+		QSKshape = progdefaults.QSKshape;
 		symbollen = nusymbollen;
 		makeshape();
 	}
-	int len = (int)(samplerate / frequency / 2);
+	int len = (int)(samplerate / frequency / 4);
 	len = len < 2 ? 2 : len;
 	if (bitfilterlen != len) {
 		bitfilterlen = len;
@@ -198,7 +202,6 @@ void cw::sync_parameters()
 	cw_receive_dash_length = 3 * cw_receive_dot_length;
 
 	cw_noise_spike_threshold = cw_receive_dot_length / 4;
-
 }
 
 
@@ -251,20 +254,33 @@ void cw::update_Status()
 //For CW this is an o scope pattern that shows the cw data stream.
 //=======================================================================
 //
+
+static int clrcount = 16;
 void cw::update_syncscope()
 {
 	int j;
 
 	for (int i = 0; i < pipesize; i++) {
 		j = (i + pipeptr) % pipesize;
-		scopedata[i] = 0.1 + 0.8 * pipe[j] / agc_peak;
+		scopedata[i] = 0.1 + 0.6 * pipe[j] / agc_peak;
 	}
 	set_scope(scopedata, pipesize, false);
 	scopedata.next(); // change buffers
+	clrcount = 16;
 	put_cwRcvWPM(cw_receive_speed);
 	update_Status();
 }
 
+void cw::clear_syncscope()
+{
+    if (--clrcount) return;
+    for (int i = 0; i < pipesize; i++)
+        scopedata[i] = 0;
+    set_scope(scopedata, pipesize, false);
+    scopedata.next();
+    clrcount = 16;
+}
+
 
 
 //=====================================================================
@@ -312,13 +328,10 @@ int cw::rx_process(const double *buf, int len)
 				agc_peak = decayavg(agc_peak, value, 800.0);
 
 			metric = clamp(agc_peak * 1000.0 , 0.0, 100.0);
-			display_metric(metric);
 			
 // save correlation amplitude value for the sync scope
 			pipe[pipeptr] = value;
 			pipeptr = (pipeptr + 1) % pipesize;
-			if (pipeptr == pipesize - 1)
-				update_syncscope();
 
 			if (!progStatus.sqlonoff || metric > progStatus.sldrSquelchValue ) {
 // upward trend means tone starting 
@@ -331,9 +344,12 @@ int cw::rx_process(const double *buf, int len)
 			if (handle_event(CW_QUERY_EVENT, &c) == CW_SUCCESS) {
 				while (*c)
 					put_rx_char(*c++);
-			}
+                update_syncscope();
+    			display_metric(metric);
+            }
 		}
 	}
+    clear_syncscope();
 
 	return 0;
 }
@@ -519,12 +535,23 @@ void cw::makeshape()
 {
 	for (int i = 0; i < KNUM; i++) keyshape[i] = 1.0;
 	knum = (int)(8 * risetime);
-	if (knum > symbollen)
-		knum = symbollen;
+
+	if (knum >= symbollen)
+		knum = symbollen - 1;
+
 	if (knum > KNUM) 
 		knum = KNUM;
-	for (int i = 0; i < knum; i++)
-		keyshape[i] = 0.5 * (1.0 - cos (M_PI * i / knum));
+
+    switch (QSKshape) {
+        case 1: // blackman 
+            for (int i = 0; i < knum; i++)
+                keyshape[i] = (0.42 - 0.50 * cos(M_PI * i/ knum) + 0.08 * cos(2 * M_PI * i / knum));
+            break;
+        case 0: // raised cosine (hanning)
+        default:
+            for (int i = 0; i < knum; i++)
+                keyshape[i] = 0.5 * (1.0 - cos (M_PI * i / knum));
+    }
 }
 
 inline double cw::nco(double freq)
@@ -537,14 +564,14 @@ inline double cw::nco(double freq)
 	return sin(phaseacc);
 }
 
-inline double cw::qrqnco()
+inline double cw::qsknco()
 {
-	qrqphase += 2.0 * M_PI * 1600.0 / samplerate;
+	qskphase += 2.0 * M_PI * 1000 / samplerate;//1600.0 / samplerate;
 
-	if (qrqphase > M_PI)
-		qrqphase -= 2.0 * M_PI;
+	if (qskphase > M_PI)
+		qskphase -= 2.0 * M_PI;
 
-	return sin(qrqphase);
+	return sin(qskphase);
 }
 
 //=====================================================================
@@ -555,17 +582,19 @@ inline double cw::qrqnco()
 //
 // Left channel contains the shaped A2 CW waveform
 // Right channel contains a square wave burst of 1600 Hz that is used
-// to trigger a QRQ switch.  Right channel has pre and post timings for
-// proper switching of the QRQ switch before and after the A2 element.
+// to trigger a qsk switch.  Right channel has pre and post timings for
+// proper switching of the qsk switch before and after the A2 element.
 // If the Pre + Post timing exceeds the interelement spacing then the
 // Pre and / or Post is only applied at the beginning and end of the
 // character.
 //=======================================================================
 
+int q_carryover = 0, carryover = 0;
+
 void cw::send_symbol(int bits)
 {
 	double freq;
-	int sample = 0, i;
+	int sample, qsample, i;
 	int delta = 0;
 	int keydown;
 	int keyup;
@@ -573,103 +602,147 @@ void cw::send_symbol(int bits)
 	int kpost;
 	int duration = 0;
 	int symlen = 0;
-	double dsymlen = 0.0;
+	float dsymlen = 0.0;
 	int currsym = bits & 1;
 	
 	freq = get_txfreq_woffset();
 
-    if ((currsym == 1) && (lastsym == 0)) {
-    	phaseacc = 0.0;
-//    	qrqphase = 0.0;
-    }
-
-	if ((currsym == 1 && lastsym == 0) || (currsym == 0 && lastsym == 1))
-		delta = (int) (symbollen * (progdefaults.CWweight - 50) / 100.0);
+    delta = (int) (symbollen * (progdefaults.CWweight - 50) / 100.0);
 
+    symlen = symbollen;
 	if (currsym == 1) {
-		dsymlen = symbollen * 4.0 / (progdefaults.CWdash2dot + 1.0);
+   		dsymlen = symbollen * (progdefaults.CWdash2dot - 3.0) / (progdefaults.CWdash2dot + 1.0);
 		if (lastsym == 1 && currsym == 1)
-			dsymlen *= ((progdefaults.CWdash2dot - 1.0) / 2.0);
-		symlen = (int) dsymlen;
-		if (symlen < knum) symlen = knum;
-	} else
-		symlen = symbollen;
-//	LOG_DEBUG("%d-%d", symbol & 1, symlen);
+			symlen += (int)(3 * dsymlen);
+        else
+            symlen -= (int)dsymlen;
+	}
 
 	if (delta < -(symlen - knum)) delta = -(symlen - knum);
 	if (delta > (symlen - knum)) delta = symlen - knum;
 
-	keydown = symlen - knum + delta;
-	keyup = symlen - knum - delta;
-	
+	keydown = symlen + delta ;
+	keyup = symlen - delta;
+
 	kpre = (int)(progdefaults.CWpre * 8);
-	kpost = (int)(progdefaults.CWpost * 8);
+	if (kpre > symlen) kpre = symlen;
+
+    if (progdefaults.CWnarrow) {
+        if (keydown - 2*knum < 0)
+            kpost = knum + (int)(progdefaults.CWpost * 8);
+        else
+	        kpost = keydown - knum + (int)(progdefaults.CWpost * 8);
+    } else
+        kpost = keydown + (int)(progdefaults.CWpost * 8);
+
+	if (kpost < 0) kpost = 0;
 	
-	if ( (kpre + kpost) > keyup) {
-		kpre = keyup / 2;
-		kpost = keyup - kpre;
-	}
-	if (bits == 4) {
-		kpost = (int)(progdefaults.CWpost * 8);
-		if (kpost > keyup)
-			kpost = keyup;
-	}
 	if (firstelement) {
-		kpre = (int)(progdefaults.CWpre * 8);
-		kpost = 0;
-		if (kpre > keydown)
-			kpre = keydown;
-	}
+	    firstelement = false;
+	    return;
+    }
 	
-	if (currsym == 1) { // keydown
-		for (i = 0; i < knum; i++, sample++) {
-			if (lastsym == 0)
-				outbuf[sample] = nco(freq) * keyshape[i];
-			else
-				outbuf[sample] = nco(freq);
-			qrqbuf[sample] = qrqnco();
-		}
-		for (i = 0; i < keydown; i++, sample++) {
-			outbuf[sample] = nco(freq);
-			qrqbuf[sample] = qrqnco();
-		}
-		duration = knum + keydown;
-	}
-	else { // keyup
-		for (i = knum - 1; i >= 0; i--, sample++) {
-			if (lastsym == 1) {
-				outbuf[sample] = nco(freq) * keyshape[i];
-			} else {
-				outbuf[sample] = 0.0;
-			}
-		}
-		for (i = 0; i < keyup; i++, sample++) {
-			outbuf[sample] = 0.0;
-		}
+    if (currsym == 1) { // keydown
+        sample = 0;
+        if (lastsym == 1) {
+            for (i = 0; i < keydown; i++, sample++) {
+                outbuf[sample] = nco(freq);
+                qskbuf[sample] = qsknco();
+            }
+            duration = keydown;
+        } else {
+            if (carryover) {
+                for (int i = carryover; i < knum; i++, sample++)
+                    outbuf[sample] = nco(freq) * keyshape[knum - i];
+                while (sample < kpre)
+                    outbuf[sample++] = 0 * nco(freq);
+            } else
+                for (int i = 0; i < kpre; i++, sample++)
+                    outbuf[sample] = 0 * nco(freq);
+            sample = 0;
+            for (int i = 0; i < kpre; i++, sample++) {
+                qskbuf[sample] = qsknco();
+            }
+            for (int i = 0; i < knum; i++, sample++) {
+                outbuf[sample] = nco(freq) * keyshape[i];
+                qskbuf[sample] = qsknco();
+            }
+            duration = kpre + knum;
+        }
+        carryover = 0;
+    }
+    else { // keyup
+        if (lastsym == 0) {
+            duration = keyup;
+            sample = 0;
+            if (carryover) {
+                for (int i = carryover; i < knum; i++, sample++)
+                    outbuf[sample] = nco(freq) * keyshape[knum - i];
+                while (sample < duration)
+                    outbuf[sample++] = 0 * nco(freq);
+            } else
+                while (sample < duration)
+                    outbuf[sample++] = 0 * nco(freq);
+            carryover = 0;
 
-		sample -= (knum + keyup);
-				
-		for (i = 0; i < knum + kpost ; i++, sample++) {
-			if (bits > 2 && lastsym == 1)
-				qrqbuf[sample] = qrqnco();
-			else
-				qrqbuf[sample] = 0.0;
-		}
-		for (i = 0; i < (keyup - kpre - kpost); i++, sample++)
-			qrqbuf[sample] = 0.0;
-		for (i = 0; i < kpre; i++, sample++)
-			if (bits > 4)
-				qrqbuf[sample] = qrqnco();
-			else
-				qrqbuf[sample] = 0.0;
-				
-		duration = knum + keyup;
-	}
+            qsample = 0;
+            if (q_carryover) {
+                for (int i = 0; i < q_carryover; i++, qsample++) {
+                    qskbuf[qsample] = qsknco();
+                }
+                while (qsample < duration)
+                    qskbuf[qsample++] = 0 * qsknco();
+            } else
+                while (qsample < duration)
+                    qskbuf[qsample++] = 0 * qsknco();
+            if (q_carryover > duration)
+                q_carryover = duration - q_carryover;
+            else
+                q_carryover = 0;
 
-	if (progdefaults.QSK)
-		ModulateStereo(outbuf, qrqbuf, duration);
-	else
-		ModulateXmtr(outbuf, duration);
+        } else { // last symbol = 1
+            duration = 2 * symbollen - kpre - knum;
+            carryover = 0;
+            sample = 0;
+
+            int next = keydown - knum;
+            if (progdefaults.CWnarrow)
+                next = keydown - 2*knum;
+
+            for (int i = 0; i < next; i++, sample++)
+                outbuf[sample] = nco(freq);
+
+            for (int i = 0; i < knum; i++, sample++) {
+                if (sample == duration) {
+                    carryover = i;
+                    break;
+                }
+                outbuf[sample] = nco(freq) * keyshape[knum - i];
+            }
+            while (sample < duration)
+                outbuf[sample++] = 0 * nco(freq);
+                
+            q_carryover = 0;
+            qsample = 0;
+
+            for (int i = 0; i < kpost; i++, qsample++) {
+                if (qsample == duration) {
+                    q_carryover = kpost - duration;
+                    break;
+                }
+                qskbuf[qsample] = qsknco();
+            }
+            while (qsample < duration)
+                qskbuf[qsample++] = 0 * qsknco();
+        }
+    }
+
+    if (duration > 0) {
+    	if (progdefaults.QSK)
+	    	ModulateStereo(outbuf, qskbuf, duration);
+	    else
+		    ModulateXmtr(outbuf, duration);
+	}
 	
 	lastsym = currsym;
 	firstelement = false;
@@ -680,28 +753,33 @@ void cw::send_symbol(int bits)
 // sends a morse character and the space afterwards
 //=======================================================================
 
+char cutnumbers[] = "T12345678N";
+
 void cw::send_ch(int ch)
 {
 	int code;
+	int chout = ch;
 
 	sync_parameters();
 // handle word space separately (7 dots spacing) 
-// last char already had 2 elements of inter-character spacing 
+// last char already had 3 elements of inter-character spacing 
 
-	if ((ch == ' ') || (ch == '\n')) {
+	if (progdefaults.cutnbrs && (ch >= '0' && ch <= '9'))
+		chout = cutnumbers[ch - '0'];
+		
+	if ((chout == ' ') || (chout == '\n')) {
 		firstelement = false;
 		send_symbol(0);
 		send_symbol(0);
 		send_symbol(0);
 		send_symbol(0);
-		send_symbol(0);
 		put_echo_char(ch);
 		return;
 	}
 
 // convert character code to a morse representation 
-	if ((ch < 256) && (ch >= 0)) {
-		code = tx_lookup(ch); //cw_tx_lookup(ch);
+	if ((chout < 256) && (chout >= 0)) {
+		code = tx_lookup(chout); //cw_tx_lookup(ch);
 		firstelement = true;
 	} else {
 		code = 0x04; 	// two extra dot spaces
@@ -712,9 +790,11 @@ void cw::send_ch(int ch)
 	while (code > 1) {
 		send_symbol(code);// & 1);
 		code = code >> 1;
-        FL_AWAKE();
 	}
+	send_symbol(0);
 
+    FL_AWAKE();
+    
 	if (ch != -1)
 		put_echo_char(ch);
 }
