@@ -25,14 +25,61 @@
 // ----------------------------------------------------------------------------
 
 #include <config.h>
+#include <iostream>
+using namespace std;
 
 #include "rtty.h"
 #include "waterfall.h"
 #include "confdialog.h"
 #include "configuration.h"
 #include "status.h"
+#include "digiscope.h"
+
+//=====================================================================
+// Baudot support
+//=====================================================================
+
+static unsigned char letters[32] = {
+	'\0',	'E',	'\n',	'A',	' ',	'S',	'I',	'U',
+	'\r',	'D',	'R',	'J',	'N',	'F',	'C',	'K',
+	'T',	'Z',	'L',	'W',	'H',	'Y',	'P',	'Q',
+	'O',	'B',	'G',	'',	'M',	'X',	'V',	''
+};
+
+#if 0
+/*
+ * ITA-2 version of the figures case.
+ */
+static unsigned char figures[32] = {
+	'\0',	'3',	'\n',	'-',	' ',	'\'',	'8',	'7',
+	'\r',	'',	'4',	'\a',	',',	'',	':',	'(',
+	'5',	'+',	')',	'2',	'',	'6',	'0',	'1',
+	'9',	'?',	'',	'',	'.',	'/',	'=',	''
+};
+#endif
+#if 1
+/*
+ * U.S. version of the figures case.
+ */
+static unsigned char figures[32] = {
+	'\0',	'3',	'\n',	'-',	' ',	'\a',	'8',	'7',
+	'\r',	'$',	'4',	'\'',	',',	'!',	':',	'(',
+	'5',	'"',	')',	'2',	'#',	'6',	'0',	'1',
+	'9',	'?',	'&',	'',	'.',	'/',	';',	''
+};
+#endif
+#if 0
+/*
+ * A mix of the two. This is what seems to be what people actually use.
+ */
+static unsigned char figures[32] = {
+	'\0',	'3',	'\n',	'-',	' ',	'\'',	'8',	'7',
+	'\r',	'$',	'4',	'\a',	',',	'!',	':',	'(',
+	'5',	'+',	')',	'2',	'#',	'6',	'0',	'1',
+	'9',	'?',	'&',	'',	'.',	'/',	'=',	''
+};
+#endif
 
-//static char rttymsg[80];
 int dspcnt = 0;
 
 
@@ -63,7 +110,6 @@ void rtty::rx_init()
 	}
 	bitfilt->reset();
 	poserr = negerr = 0.0;
-	set_AFCrange(shift/10.0);
 }
 
 void rtty::init()
@@ -92,8 +138,6 @@ rtty::~rtty()
 
 void rtty::restart()
 {
-	double fhi;
-	double flo;
 	double stl;
 	
 	rtty_shift = shift = _SHIFT[progdefaults.rtty_shift];
@@ -116,21 +160,22 @@ void rtty::restart()
 	rxmode = LETTERS;
 	symbollen = (int) (samplerate / rtty_baud + 0.5);
 	set_bandwidth(shift);
+	
+    rtty_BW = shift + 3*rtty_baud;
+	progdefaults.RTTY_BW = rtty_BW;
+	sldrRTTYbandwidth->value(rtty_BW);
+	
+	wf->redraw_marker();
 
-//	fhi = (shift / 2 + rtty_baud* 1.5) / samplerate;
-	fhi = (shift / 2 + rtty_baud * 2.0) / samplerate;
-	flo = 0.0;
-//	flo = (shift / 2 - rtty_baud* 1.5) / samplerate;
 	if (bpfilt) 
-		bpfilt->create_filter(flo, fhi);
+		bpfilt->create_filter(0, rtty_BW / samplerate / 2.0);
 	else
-		bpfilt = new fftfilt(flo, fhi, 1024);
-
+		bpfilt = new fftfilt(0, rtty_BW / samplerate / 2.0, 1024);
 
 	if (bitfilt)
-		bitfilt->setLength(symbollen / 4);//6);
+		bitfilt->setLength(symbollen / 8);//4);
 	else
-		bitfilt = new Cmovavg(symbollen / 4);//6);
+		bitfilt = new Cmovavg(symbollen / 8);//4);
 
 // stop length = 1, 1.5 or 2 bits
 	rtty_stop = progdefaults.rtty_stop;
@@ -157,6 +202,8 @@ void rtty::restart()
 	freqerrlo = freqerrhi = 0.0;
 	sigsearch = 0;
 	dspcnt = 2*(nbits + 2);//nbits + 2; //2 * symbollen * (nbits + 2);
+	
+	clear_zdata = true;
 }
 
 rtty::rtty(trx_mode tty_mode)
@@ -184,12 +231,12 @@ rtty::rtty(trx_mode tty_mode)
 void rtty::update_syncscope()
 {
 	int j;
-	for (int i = 0; i < 2 * symbollen; i++) {
+	for (int i = 0; i < symbollen; i++) {
 		j = pipeptr - i;
-		if (j < 0) j += (2*symbollen);
+		if (j < 0) j += symbollen;
 		dsppipe[i] = pipe[j];
 	}
-	set_scope(dsppipe, 2*symbollen, false);
+	set_scope(dsppipe, symbollen, false);
 }
 
 void rtty::clear_syncscope()
@@ -360,7 +407,9 @@ void rtty::Metric()
 	 		   wf->powerDensity(frequency + shift * 1.5, delta) + 1e-10;
 	sigpwr = wf->powerDensity(frequency - shift/2, delta) +
 			 wf->powerDensity(frequency + shift/2, delta) + 1e-10;
-	metric = decayavg( metric, 40.0*log10(sigpwr / noisepwr), 8);
+    double snr = sigpwr / noisepwr;
+    metric = decayavg( metric, snr, 8);//16);
+    metric = CLAMP(metric, 0.0, 100.0);
 	display_metric(metric);
 }
 
@@ -407,13 +456,20 @@ int rtty::rx_process(const double *buf, int len)
 	complex z, *zp;
 	double f = 0.0;
 	double fin;
-	static bool bit = false;
-	int n, rxflag;
+	static bool bit = true;//false;
+	int n = 0, rxflag;
 	double deadzone = shift/8;
 	double rotate;
-	double halfshift = rtty_shift/2.0;
 	double ferr = 0;
 
+	if (progdefaults.RTTY_BW != rtty_BW) {
+		rtty_BW = progdefaults.RTTY_BW;
+		bpfilt->create_filter(0, rtty_BW / samplerate / 2.0);
+		wf->redraw_marker();
+	}
+
+	Metric();
+
 	while (len-- > 0) {
 		
 // create analytic signal from sound card input samples
@@ -421,8 +477,6 @@ int rtty::rx_process(const double *buf, int len)
 		z.re = z.im = *buf++;
 		hilbert->run(z, z);
 
-//		MarkSpace(z);
-		
 // mix it with the audio carrier frequency to create a baseband signal
 
 		z = mixer(z);
@@ -445,64 +499,52 @@ int rtty::rx_process(const double *buf, int len)
 				f = bitfilt->run(fin);
 			
 // track the + and - frequency excursions separately to derive an afc signal
-			
+
+                rotate = -4.0 * M_PI * freqerr / rtty_shift;
+                double xmix = fabs(4.0 * freqerr / rtty_shift);
+                if (xmix > 0.5) xmix = 0.5;
+                xmix += 0.05;
 				if (fin > 0.0) {
 					poscnt++;
 					posfreq += fin;
 					QI[i].re = zp[i].re;
-					QI[i].im = 0.1 * zp[i].im;
+					QI[i].im = xmix * zp[i].im;
 				} 
 				if (fin < 0.0) {
 					negcnt++;
 					negfreq += fin;
-					QI[i].re = 0.1 * zp[i].im;
+					QI[i].re = xmix * zp[i].im;
 					QI[i].im = zp[i].re;
 				}
-				
-				if (fin >= 0.0)
-					rotate = (halfshift - freqerrhi) * (M_PI/4.0) / halfshift;
-				else
-					rotate = -(halfshift + freqerrlo) * (M_PI/4.0) / halfshift;
 				QI[i] = QI[i] * complex(cos(rotate), sin(rotate));
-				avgsig = decayavg(avgsig, zp[i].mag(), 32);
+                avgsig = decayavg(avgsig, 1.25 * zp[i].mag(), 64);
 				
-				if (avgsig > 0.002) {
-					QI[i].re = (0.7 / avgsig) * QI[i].re;
-					QI[i].im = (0.7 / avgsig) * QI[i].im;
-				}
+				if (avgsig > 0) 
+				    QI[i] = QI[i] / avgsig;
+
 //	hysterisis dead zone in frequency discriminator bit detector
 
-				if (f > deadzone )  {
-					bit = false;
-				}
-				if (f < -deadzone) {
-					bit = true;
-				}
-			
 				if (dspcnt && (--dspcnt % (nbits + 2) == 0)) {
 					pipe[pipeptr] = f / shift; // display filtered signal		
-					pipeptr = (pipeptr + 1) % (2*symbollen);
+					pipeptr = (pipeptr + 1) % symbollen;
 				}
 
-				if (!progdefaults.RTTY_USB)
-					bit = !bit;
+				if (f > deadzone )
+					bit = true;
+				if (f < -deadzone)
+					bit = false;
+//				if (!progdefaults.RTTY_USB)
+//					bit = !bit;
 	
-				rxflag = rx (reverse ? bit : !bit);
+				rxflag = rx (reverse ? !bit : bit);
 
 				if (rxflag == 2) {
-					if ((metric > progStatus.sldrSquelchValue && progStatus.sqlonoff) || !progStatus.sqlonoff) {
-						update_syncscope();
-					}
-					else
-						clear_syncscope();
-					dspcnt = symbollen * 2 * (nbits + 2);
+					dspcnt = symbollen * (nbits + 2);
 
 					if (poscnt && negcnt) {
 						poserr = posfreq/poscnt;
 						negerr = negfreq/negcnt;
 						
-						Metric();
-				
 // compute the frequency error as the median of + and - relative freq's
 						if (sigsearch) sigsearch--;
 
@@ -513,30 +555,32 @@ int rtty::rx_process(const double *buf, int len)
 						if (fs == 0) avging = 8;
 						else if (fs == 1) avging = 4;
 						else avging = 1;
-						freqerrhi = decayavg(freqerrhi, poserr, avging);
-						freqerrlo = decayavg(freqerrlo, negerr, avging);
 						freqerr   = decayavg(freqerr, ferr,  avging);
 
-// display the FSK +/- signals on the digiscope					
-						set_rtty( 
-							negerr/(halfshift), 
-							poserr/(halfshift), 
-							1.0 );
-					}
-					poscnt = 0; posfreq = 0.0;
-					negcnt = 0; negfreq = 0.0;
+		    			poscnt = 0; posfreq = 0.0;
+			    		negcnt = 0; negfreq = 0.0;
 
-					if (progStatus.afconoff) {
-						if (metric > progStatus.sldrSquelchValue || !progStatus.sqlonoff) {// || sigsearch) {
-							set_freq(frequency - ferr);
-							set_AFCind(freqerr); 
-						} else set_AFCind(0.0);
 					}
 				}
 			}
 		}
-		set_zdata(QI, n);
 	}
+	if (progStatus.afconoff) {
+        if (metric > progStatus.sldrSquelchValue || !progStatus.sqlonoff) // || sigsearch) {
+            set_freq(frequency - ferr);
+    }
+    if (metric < progStatus.sldrSquelchValue && progStatus.sqlonoff) {
+        if (clear_zdata) {
+            for (int i = 0; i < MAX_ZLEN; i++) QI[i].re = QI[i].im = 0.0;
+            set_zdata(QI, MAX_ZLEN);
+            clear_zdata = false;
+            clear_syncscope();
+        }
+    } else {
+        clear_zdata = true;
+        update_syncscope();
+        set_zdata(QI, n);
+    }
 
 	return 0;
 }
@@ -558,6 +602,7 @@ double rtty::nco(double freq)
 double rtty::FSKnco()
 {
 	FSKphaseacc += TWOPI * 1000 / samplerate;
+//	FSKphaseacc += TWOPI * 440 / samplerate;
 
 	if (FSKphaseacc > M_PI)
 
@@ -573,19 +618,18 @@ void rtty::send_symbol(int symbol)
 	
 	if (reverse)
 		symbol = !symbol;
-	if (!progdefaults.RTTY_USB)
-		symbol = !symbol;
 
 	if (symbol)
 		freq = get_txfreq_woffset() + shift / 2.0;
 	else
 		freq = get_txfreq_woffset() - shift / 2.0;
+    
 	for (int i = 0; i < symbollen; i++) {
 		outbuf[i] = nco(freq);
 		if (symbol)
 			FSKbuf[i] = FSKnco();
 		else
-			FSKbuf[i] = 0.0;
+			FSKbuf[i] = 0.0 * FSKnco();
 	}
 
 	if (progdefaults.PseudoFSK)
@@ -598,8 +642,6 @@ void rtty::send_stop()
 {
 	double freq;
 	bool invert = reverse;
-	if (!progdefaults.RTTY_USB)
-		invert = !invert;
 	
 	if (invert)
 		freq = get_txfreq_woffset() - shift / 2.0;
@@ -609,9 +651,9 @@ void rtty::send_stop()
 	for (int i = 0; i < stoplen; i++) {
 		outbuf[i] = nco(freq);
 		if (invert)
-			FSKbuf[i] = FSKnco();
+			FSKbuf[i] = 0.0 * FSKnco();
 		else
-			FSKbuf[i] = 0.0;
+			FSKbuf[i] = FSKnco();
 	}
 	if (progdefaults.PseudoFSK)
 		ModulateStereo(outbuf, FSKbuf, stoplen);
@@ -641,10 +683,16 @@ void rtty::send_char(int c)
 // stop bit(s)
 	send_stop();
 
-	if (nbits == 5)
-		c = baudot_dec(c);
-	if (c)
-		put_echo_char(c);
+	if (nbits == 5) {
+		if (c == 0x1F || c == 0x1B)
+			return;
+		if (txmode == LETTERS)
+			c = letters[c];
+		else
+			c = figures[c];
+		if (c) 
+			put_echo_char(c);
+	}
 }
 
 void rtty::send_idle()
@@ -709,10 +757,9 @@ int rtty::tx_process()
 		++line_char_count;
 	}
 
-	if ( (c != '\n' && c != '\r'))
-		if (progdefaults.rtty_autocrlf == true) 
-			if ( line_char_count == progdefaults.rtty_autocount ||
-			     ((line_char_count > progdefaults.rtty_autocount - 5) && c == ' ') ) {
+	if (progdefaults.rtty_autocrlf && (c != '\n' && c != '\r') &&
+	    (line_char_count == progdefaults.rtty_autocount ||
+	     (line_char_count > progdefaults.rtty_autocount - 5 && c == ' '))) {
 		line_char_count = 0;
 		if (progdefaults.rtty_crcrlf)
 			send_char(0x08); // CR-CR-LF triplet
@@ -766,51 +813,6 @@ int rtty::tx_process()
 	return 0;
 }
 
-//=====================================================================
-// Baudot support
-//=====================================================================
-
-static unsigned char letters[32] = {
-	'\0',	'E',	'\n',	'A',	' ',	'S',	'I',	'U',
-	'\r',	'D',	'R',	'J',	'N',	'F',	'C',	'K',
-	'T',	'Z',	'L',	'W',	'H',	'Y',	'P',	'Q',
-	'O',	'B',	'G',	'',	'M',	'X',	'V',	''
-};
-
-#if 0
-/*
- * ITA-2 version of the figures case.
- */
-static unsigned char figures[32] = {
-	'\0',	'3',	'\n',	'-',	' ',	'\'',	'8',	'7',
-	'\r',	'',	'4',	'\a',	',',	'',	':',	'(',
-	'5',	'+',	')',	'2',	'',	'6',	'0',	'1',
-	'9',	'?',	'',	'',	'.',	'/',	'=',	''
-};
-#endif
-#if 1
-/*
- * U.S. version of the figures case.
- */
-static unsigned char figures[32] = {
-	'\0',	'3',	'\n',	'-',	' ',	'\a',	'8',	'7',
-	'\r',	'$',	'4',	'\'',	',',	'!',	':',	'(',
-	'5',	'"',	')',	'2',	'#',	'6',	'0',	'1',
-	'9',	'?',	'&',	'',	'.',	'/',	';',	''
-};
-#endif
-#if 0
-/*
- * A mix of the two. This is what seems to be what people actually use.
- */
-static unsigned char figures[32] = {
-	'\0',	'3',	'\n',	'-',	' ',	'\'',	'8',	'7',
-	'\r',	'$',	'4',	'\a',	',',	'!',	':',	'(',
-	'5',	'+',	')',	'2',	'#',	'6',	'0',	'1',
-	'9',	'?',	'&',	'',	'.',	'/',	'=',	''
-};
-#endif
-
 int rtty::baudot_enc(unsigned char data)
 {
 	int i, c, mode;
