@@ -22,7 +22,7 @@ using namespace std;
 FSEL* FSEL::inst = 0;
 static std::string filename;
 #if FSEL_THREAD
-static Fl_Thread fsel_thread;
+static pthread_t fsel_thread;
 sem_t fsel_sem;
 #endif
 
@@ -67,7 +67,7 @@ void* FSEL::thread_func(void* arg)
 const char* FSEL::get_file(void)
 {
 	// Calling directory() is apparently not enough on Linux
-#ifdef __linux__
+#if !defined(__CYGWIN__) && !defined(__APPLE__)
 	const char* preset = chooser->preset_file();
 	if (preset && *preset != '/' && chooser->directory()) {
 		filename = chooser->directory();
@@ -77,7 +77,7 @@ const char* FSEL::get_file(void)
 #endif
 
 #if FSEL_THREAD
-	if (fl_create_thread(fsel_thread, thread_func, this) != 0) {
+	if (pthread_create(&fsel_thread, NULL, thread_func, this) != 0) {
 		fl_alert("%s", "could not create file selector thread");
 		return NULL;
 	}
