@@ -34,15 +34,6 @@
 
 using namespace std;
 
-const char *state_names[] = {
-	"PAUSED",
-	"RECEIVE",
-	"TRANSMIT",
-	"TUNING",
-	"ABORTED",
-	"FLUSHING"
-};
-
 // Elements are in enum trx_mode order. Mode name video-id uses the
 // first string (sname), so its length should be a multiple of 2.
 const struct mode_info_t mode_info[NUM_MODES] = {
@@ -83,7 +74,7 @@ const struct mode_info_t mode_info[NUM_MODES] = {
 	{ MODE_PSK63, &psk63_modem, "BPSK63", "BPSK-63", "PSK63", "PSK63" },
 	{ MODE_QPSK63, &qpsk63_modem, "QPSK63", "QPSK-63", "QPSK63", "QPSK63" },
 	{ MODE_PSK125, &psk125_modem, "BPSK125", "BPSK-125", "PSK125", "PSK125" },
-	{ MODE_QPSK125, &qpsk125_modem, "QPSK125", "QPSK-125", "QPSK125", "PSK125" },
+	{ MODE_QPSK125, &qpsk125_modem, "QPSK125", "QPSK-125", "QPSK125", "QPSK125" },
 	{ MODE_PSK250, &psk250_modem, "BPSK250", "BPSK-250", "PSK250", "PSK250" },
 	{ MODE_QPSK250, &qpsk250_modem, "QPSK250", "QPSK-250", "QPSK250", "QPSK250" },
 
@@ -138,32 +129,70 @@ std::istream& operator>>(std::istream& s, qrg_mode_t& m)
 std::string qrg_mode_t::str(void)
 {
 	ostringstream s;
-	s << setiosflags(ios::fixed) << setprecision(3) << rfcarrier/1000.0 << '\t'
-          << rmode << '\t'
-          << (mode < NUM_MODES ? mode_info[mode].sname : "NONE") << '\t'
-          << carrier;
+	s << setiosflags(ios::left | ios::fixed)
+	  << setw(12) << setprecision(3) << rfcarrier/1000.0
+	  << setw(8) << rmode
+	  << setw(10) << (mode < NUM_MODES ? mode_info[mode].sname : "NONE")
+	  << carrier;
 	return s.str();
+}
 
 
-	// This an example of how we would do things if we were not using
-	// Fl_Browser and had to format the fields manually and add the string
-	// to a menu
+band_t band(long long freq_hz)
+{
+	switch (freq_hz / 1000000LL) {
+		case 0: return BAND_LMW;
+		case 1: return BAND_160M;
+		case 3: return BAND_80M;
+		case 4: return BAND_75M;
+		case 5: return BAND_60M;
+		case 7: return BAND_40M;
+		case 10: return BAND_30M;
+		case 14: return BAND_20M;
+		case 18: return BAND_17M;
+		case 21: return BAND_15M;
+		case 24: return BAND_12M;
+		case 28 ... 29: return BAND_10M;
+		case 50 ... 54: return BAND_6M;
+		case 70 ... 71: return BAND_4M;
+		case 144 ... 148: return BAND_2M;
+		case 222 ... 225: return BAND_125CM;
+		case 420 ... 450: return BAND_70CM;
+		case 902 ... 928: return BAND_33CM;
+		case 1240 ... 1325: return BAND_23CM;
+		case 2300 ... 2450: return BAND_13CM;
+		case 3300 ... 3500: return BAND_9CM;
+	}
 
-	// static unsigned max_mode_sname = 0;
+	return BAND_OTHER;
+}
 
-	// if (max_mode_sname == 0)
-	//	   for (size_t i = 0; i < NUM_MODES; i++)
-	//		   if (max_mode_sname < strlen(mode_info[i].sname))
-	//			   max_mode_sname = strlen(mode_info[i].sname);
+static const char* band_names[NUM_BANDS] = {
+	"LMW",
+	"160m",
+	"80m",
+	"75m",
+	"60m",
+	"40m",
+	"30m",
+	"20m",
+	"17m",
+	"15m",
+	"12m",
+	"10m",
+	"6m",
+	"4m",
+	"2m",
+	"125cm",
+	"70cm",
+	"33cm",
+	"23cm",
+	"13cm",
+	"9cm",
+	"other"
+};
 
-	// ostringstream s;
-
-	// s << setw(11) << setiosflags(ios::right) << setiosflags(ios::fixed) << setprecision(3)
-	//   << rfcarrier/1000.0 << ' '
-	//   << setw(max_mode_sname) << resetiosflags(ios::right) << setiosflags(ios::left)
-	//   << (mode < NUM_MODES ? mode_info[mode].sname : "NONE") << ' '
-	//   << setw(4) << resetiosflags(ios::left) << setiosflags(ios::right)
-	//   << carrier << ' ' << rmode;
-
-	// return s.str();
+const char* band_name(band_t b)
+{
+	return band_names[CLAMP(b, 0, NUM_BANDS-1)];
 }
