@@ -39,7 +39,11 @@
 class FTextBase : public Fl_Text_Editor_mod
 {
 public:
-	enum TEXT_ATTR { RECV, XMIT, CTRL, SKIP, ALTR, NATTR };
+	// CLICK_START: same as first clickable style
+	// NATTR: number of styles (last style + 1)
+	enum TEXT_ATTR { RECV, XMIT, CTRL, SKIP, ALTR,
+			 CLICK_START, QSY = CLICK_START, /* FOO, BAR, ..., */
+			 NATTR };
 
 	FTextBase(int x, int y, int w, int h, const char *l = 0);
 	virtual ~FTextBase() { delete tbuf; delete sbuf; }
@@ -55,6 +59,11 @@ public:
 	void		setFont(Fl_Font f, int attr = NATTR);
 	void		setFontSize(int s, int attr = NATTR);
 	void		setFontColor(Fl_Color c, int attr = NATTR);
+	// Override Fl_Text_Display, which stores the font number in an unsigned
+	// character and therefore cannot represent all fonts
+	Fl_Font		textfont(void) { return styles[0].font; }
+	void		textfont(Fl_Font f) { setFont(f); }
+	void		textfont(uchar s) { textfont((Fl_Font)s); }
 
 	void		cursorON(void) { show_cursor(); }
 	virtual void	resize(int X, int Y, int W, int H);
@@ -66,7 +75,7 @@ protected:
 				  int set = SET_FONT | SET_SIZE | SET_COLOR);
 	int		readFile(const char* fn = 0);
 	void		saveFile(void);
-	char		*get_word(int x, int y);
+	char*		get_word(int x, int y, bool ontext = true);
 	void		show_context_menu(void);
 	virtual void	menu_cb(int val) { }
 	int		reset_wrap_col(void);
@@ -89,6 +98,10 @@ protected:
 	int					wrap_col;
 	int					max_lines;
 	bool					scroll_hint;
+
+private:
+	int					oldw, olds;
+	Fl_Font					oldf;
 };
 
 ///
@@ -107,27 +120,53 @@ public:
                 while (*s)
                         add(*s++, attr);
         }
+	void		clear(void);
+	void		QuickEntry(bool b) {
+		quick_entry = b;
+		if (b) view_menu[RX_MENU_QUICK_ENTRY].set();
+		else   view_menu[RX_MENU_QUICK_ENTRY].clear();
+	}
+	bool		QuickEntry() { return quick_entry;}
+	void		WordWrap(bool b) {
+		wrap = b;
+		wrap_mode((wrap), wrap_col);
+		show_insert_position();
+	}
+	bool		WordWrap() { return wrap;}
 
 protected:
 	enum { RX_MENU_QRZ_THIS, RX_MENU_CALL, RX_MENU_NAME, RX_MENU_QTH,
-	       RX_MENU_LOC, RX_MENU_RST_IN, RX_MENU_DIV, RX_MENU_CLEAR,
-	       RX_MENU_COPY,
+	       RX_MENU_STATE, RX_MENU_PROVINCE,RX_MENU_COUNTRY, RX_MENU_LOC,
+	       RX_MENU_RST_IN, RX_MENU_SERIAL, RX_MENU_XCHG,
+	       RX_MENU_DIV, RX_MENU_COPY, RX_MENU_CLEAR, RX_MENU_SELECT_ALL,
 #if 0 //#ifndef NDEBUG
                RX_MENU_READ,
 #endif
-               RX_MENU_SAVE, RX_MENU_WRAP };
+               RX_MENU_SAVE, RX_MENU_QUICK_ENTRY, RX_MENU_WRAP };
 
+	void		handle_clickable(int x, int y);
+	void		handle_qsy(int start, int end);
+	void		handle_qso_data(int start, int end);
+	void		handle_context_menu(void);
 	virtual void	menu_cb(int val);
 	static void	changed_cb(int pos, int nins, int ndel, int nsty,
 				   const char *dtext, void *arg);
 	void		change_keybindings(void);
+	const char*	dxcc_lookup_call(int x, int y);
+	static void	dxcc_tooltip(void* obj);
 
 private:
 	FTextView();
 	FTextView(const FTextView &t);
+	bool	quick_entry;
 
 protected:
 	static Fl_Menu_Item view_menu[];
+	static Fl_Menu_Item view_menu_noicons[];
+	struct {
+		bool enabled;
+		float delay;
+	} tooltips;
 };
 
 
@@ -146,10 +185,16 @@ public:
 	void		clear(void);
 	void		clear_sent(void);
 	int		nextChar(void);
+	void		WordWrap(bool b) {
+		wrap = b;
+		wrap_mode((wrap), wrap_col);
+		show_insert_position();
+	}
+	bool		WordWrap() { return wrap;}
 
 protected:
-	enum { TX_MENU_TX, TX_MENU_RX, TX_MENU_MFSK16_IMG, TX_MENU_CLEAR,
-	       TX_MENU_CUT, TX_MENU_COPY, TX_MENU_PASTE, TX_MENU_READ,
+	enum { TX_MENU_TX, TX_MENU_RX, TX_MENU_ABORT, TX_MENU_MFSK16_IMG,
+	       TX_MENU_CUT, TX_MENU_COPY, TX_MENU_PASTE, TX_MENU_CLEAR, TX_MENU_READ,
 	       TX_MENU_WRAP
 	};
 	int		handle_key(int key);
@@ -157,6 +202,7 @@ protected:
 	int		handle_key_ascii(int key);
 	int		handle_dnd_drag(void);
 	int		handle_dnd_drop(void);
+	void		handle_context_menu(void);
 	virtual void	menu_cb(int val);
 	static void	changed_cb(int pos, int nins, int ndel, int nsty,
 				   const char *dtext, void *arg);
@@ -173,6 +219,7 @@ private:
 
 protected:
 	static Fl_Menu_Item	edit_menu[];
+	static Fl_Menu_Item	edit_menu_noicons[];
 	bool			PauseBreak;
 	int			txpos;
 	static int		*ptxpos;
@@ -202,6 +249,7 @@ private:
 
 protected:
 	static Fl_Menu_Item log_menu[];
+	static Fl_Menu_Item log_menu_noicons[];
 };
 
 
