@@ -37,7 +37,6 @@
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Output.H>
 #include <FL/Fl_Select_Browser.H>
-#include <FL/Fl_Pixmap.H>
 
 #define FL_COMBO_UNIQUE 1
 #define FL_COMBO_UNIQUE_NOCASE 2
@@ -105,7 +104,8 @@ class Fl_ComboBox : public Fl_Group  {
   public:
 
     Fl_ComboBox (int x, int y, int w, int h, const char * = 0);
-
+	~Fl_ComboBox();
+	
     const char *value ();
     void value (const char *);
     void put_value( const char *);
