@@ -37,17 +37,16 @@
 #include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Widget.H>
 #include <FL/fl_draw.H>
-#include "threads.h"
 #include "complex.h"
 
 class Digiscope : public Fl_Widget { 
 public:
 #define DEFAULT_WIDTH	100
 #define DEFAULT_HEIGHT	100
-#define	MAX_LEN			4096
-//#define MAX_ZLEN		16184
-//#define MAX_ZLEN		8192
-#define MAX_ZLEN		4096
+//#define	MAX_LEN			4096
+//#define MAX_ZLEN		4096
+#define	MAX_LEN			1024
+#define MAX_ZLEN		1024
 	enum scope_mode {
 		SCOPE,
 		PHASE,
@@ -61,7 +60,7 @@ public:
 		DOMWF,
 		BLANK
 	};
-	Fl_Mutex *mutex;
+
 private:
 	scope_mode _mode;
 	double _buf[MAX_LEN];
