@@ -26,6 +26,8 @@
 
 #define fl_digi_h
 
+#include <string.h>
+
 #include <FL/Fl.H>
 #include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Menu_Bar.H>
@@ -37,6 +39,7 @@
 #include <FL/Fl_Tile.H>
 #include <FL/Fl_Light_Button.H>
 #include <FL/Fl_Check_Button.H>
+#include <FL/Fl_Menu_Item.H>
 
 #include "combo.h"
 #include "FTextView.h"
@@ -46,9 +49,14 @@
 #include "globals.h"
 #include "mixer.h"
 #include "progress.h"
+#include "FreqControl.h"
+#include <flinput2.h>
+
+using namespace std;
 
 extern Fl_Double_Window *fl_digi_main;
 extern Fl_Double_Window *scopeview;
+//extern Fl_Double_Window *opBrowserView;
 
 extern FTextView		*ReceiveText;
 extern FTextEdit		*TransmitText;
@@ -62,20 +70,42 @@ extern Fl_Button		*MODEstatus;
 extern Fl_Slider		*sldrSquelch;
 extern Progress			*pgrsSquelch;
 extern Fl_Button 		*btnMacro[];
-extern Fl_Input			*inpFreq;
-extern Fl_ComboBox		*cboBand;
-extern Fl_Button		*btnSideband;
-extern Fl_Input			*inpTime;
-extern Fl_Input			*inpCall;
-extern Fl_Input			*inpName;
-extern Fl_Input			*inpRstIn;
-extern Fl_Input			*inpRstOut;
-extern Fl_Input			*inpQth;
-extern Fl_Input			*inpLoc;
-extern Fl_Input			*inpNotes;
-extern Fl_Input			*inpAZ;	// WA5ZNU
+extern Fl_Input2		*inpFreq;
+extern Fl_Input2		*inpTimeOff;
+extern Fl_Input2		*inpTimeOn;
+extern Fl_Input2		*inpCall;
+extern Fl_Input2		*inpName;
+extern Fl_Input2		*inpRstIn;
+extern Fl_Input2		*inpRstOut;
+extern Fl_Input2		*inpQth;
+extern Fl_Input2		*inpState;
+extern Fl_Input2		*inpCountry;
+extern Fl_Input2		*inpSerNo;
+extern Fl_Input2		*outSerNo;
+extern Fl_Input2		*inpXchgIn;
+extern Fl_Box			*lblDup;
+extern Fl_Input2		*inpVEprov;
+extern Fl_Input2		*inpLoc;
+extern Fl_Input2		*inpNotes;
+extern Fl_Input2		*inpAZ;	// WA5ZNU
 extern Fl_Button		*qsoClear;
 extern Fl_Button		*qsoSave;
+extern Fl_Box			*txtRigName;
+
+extern Fl_Group			*qsoFrameView;
+extern Fl_Group			*QsoButtonFrame;
+extern Fl_Group			*QsoInfoFrame;
+extern cFreqControl		*qsoFreqDisp;
+extern Fl_ComboBox		*qso_opMODE;
+extern Fl_ComboBox		*qso_opBW;
+extern Fl_Button		*qso_opPICK;
+extern Fl_Browser		*qso_opBrowser;
+extern Fl_Button		*qso_btnAddFreq;
+extern Fl_Button		*qso_btnSelFreq;
+extern Fl_Button		*qso_btnDelFreq;
+extern Fl_Button		*qso_btnClearList;
+
+
 extern Fl_Value_Slider		*valRcvMixer;
 extern Fl_Value_Slider		*valXmtMixer;
 extern Fl_Button		*btn_afconoff;
@@ -86,7 +116,6 @@ extern Fl_Button		*btnMacroDummy;
 
 
 extern bool				useCheckButtons;
-extern bool				twoscopes;
 extern int				altMacros;
 
 extern waterfall		*wf;
@@ -94,14 +123,14 @@ extern Digiscope		*digiscope;
 
 extern void toggleRSID();
 
+extern void set_menus();
 extern void create_fl_digi_main();
 extern void update_main_title();
-extern Fl_Menu_Item menu_[];
-extern Fl_Menu_Item *getMenuItem(const char *caption);
 extern void activate_rig_menu_item(bool b);
 extern void activate_test_menu_item(bool b);
 extern void activate_mfsk_image_item(bool b);
 
+extern void cb_mnuVisitURL(Fl_Widget*, void* arg);
 
 extern void put_freq(double frequency);
 extern void put_Bandwidth(int bandwidth);
@@ -126,11 +155,16 @@ enum status_timeout {
 };
 extern void put_status(const char *msg, double timeout = 0.0, status_timeout action = STATUS_CLEAR);
 extern void clear_StatusMessages();
+extern void put_MODEstatus(const char* fmt, ...) format__(printf, 1, 2);
 extern void put_MODEstatus(trx_mode mode);
 extern void put_Status1(const char *msg, double timeout = 0.0, status_timeout action = STATUS_CLEAR);
 extern void put_Status2(const char *msg, double timeout = 0.0, status_timeout action = STATUS_CLEAR);
-extern void set_AFCind( double val );
-extern void set_AFCrange(double val);
+
+extern void show_frequency(long long);
+extern void show_mode(const string& mode);
+extern void show_bw(const string& sWidth);
+extern void show_spot(bool v);
+extern void showMacroSet();
 
 extern void put_WARNstatus(double);
 
@@ -143,10 +177,11 @@ extern void resetOLIVIA();
 extern void resetTHOR();
 extern void resetDOMEX();
 extern void resetSoundCard();
-extern void restoreFocus();
+extern void restoreFocus(Fl_Widget* w = 0);
 extern void setReverse(int);
 extern void clearQSO();
 extern void closeRigDialog();
+extern void CloseQsoView();
 
 extern void setAfcOnOff(bool b);
 extern void setSqlOnOff(bool b);
@@ -156,8 +191,6 @@ extern bool QuerySqlOnOff();
 extern void init_modem(trx_mode mode);
 extern void init_modem_sync(trx_mode mode);
 
-extern void change_modem_param(int state);
-
 extern void start_tx();
 extern void abort_tx();
 
@@ -168,7 +201,19 @@ extern void set_rtty_tab_widgets();
 extern void set_olivia_tab_widgets();
 
 extern void stopMacroTimer();
+extern void cb_ResetSerNbr();
+extern void updateOutSerNo();
+
+const char* zdate(void);
+const char* ztime(void);
 
 Fl_Color adjust_color(Fl_Color fg, Fl_Color bg);
 
+extern void setTabColors();
+
+void qsy(long long rfc, long long fmid = -1LL);
+
+unsigned quick_choice_menu(const char* title, unsigned sel, const Fl_Menu_Item* menu);
+unsigned quick_choice(const char* title, unsigned sel, ...);
+
 #endif
