@@ -56,26 +56,13 @@
 #define WFMARKER     6
 #define BTN_HEIGHT	20
 
-#define bwColor		25
-#define bwFFT		30
-#define bwX1		20
-#define bwMov		20
-#define cwCnt		100
-#define cwRef		50
-#define cwMode		85
-#define bwQsy		35
-#define bwRate		45
-#define bwXmtLock	30
-#define bwRev		30
-#define bwMem		40
-#define bwXmtRcv	40
-#define wSpace		3
-
 #define DEFAULT_SW 15
 //#define DEFAULT_HWFALL 144
 #define DEFAULT_HWFALL 124
 #define DEFAULT_HNOM 500
-#define Wwfall		(DEFAULT_HNOM + 2 * BEZEL)
+#define WMIN 645
+#define HMIN 450
+//#define Wwfall		(DEFAULT_HNOM + 2 * BEZEL)
 #define DEFAULT_WNOM (Wwfall + 2* DEFAULT_SW)
 
 //#define EMC_HWFALL 144
@@ -86,14 +73,15 @@ extern int IMAGE_WIDTH;
 extern int Hwfall;
 extern int HNOM;
 extern int WNOM;
+extern int Wwfall;
 
 #define Hmenu		22
 #define Hqsoframe	48
-#define Hnotes		22
+#define Hnotes		24
 #define Hstatus		22
 #define Hmacros		22
 
-#define Htext		(HNOM - 4 - Hwfall - Hmenu - Hstatus - Hmacros - Hqsoframe - Hnotes)
+#define Htext		(DEFAULT_HNOM - 4 - Hwfall - Hmenu - Hstatus - Hmacros - Hqsoframe - Hnotes)
 #define Hrcvtxt		(Htext) / 2
 #define Hxmttxt		(Htext - (Hrcvtxt))
 
