@@ -44,7 +44,6 @@ enum state_t {
 	STATE_IDLE,
 	STATE_NEW_MODEM
 };
-extern const char *state_names[];
 
 enum {
 	MODE_PREV = -2,
@@ -58,6 +57,8 @@ enum {
 	MODE_DOMINOEX11,
 	MODE_DOMINOEX16,
 	MODE_DOMINOEX22,
+	MODE_DOMINOEX_FIRST = MODE_DOMINOEX4,
+	MODE_DOMINOEX_LAST = MODE_DOMINOEX22,
 
 	MODE_FELDHELL,
 	MODE_SLOWHELL,
@@ -66,21 +67,26 @@ enum {
 	MODE_FSKHELL,
 	MODE_FSKH105,
 	MODE_HELL80,
+	MODE_HELL_FIRST = MODE_FELDHELL,
+	MODE_HELL_LAST = MODE_HELL80,
 
 	MODE_MFSK8,
 	MODE_MFSK16,
 	MODE_MFSK32,
-	
 // experimental modes
 	MODE_MFSK4,
 	MODE_MFSK11,
 	MODE_MFSK22,
 	MODE_MFSK31,
 	MODE_MFSK64,
-	
+	MODE_MFSK_FIRST = MODE_MFSK8,
+	MODE_MFSK_LAST = MODE_MFSK64,
+
 	MODE_MT63_500,
 	MODE_MT63_1000,
 	MODE_MT63_2000,
+	MODE_MT63_FIRST = MODE_MT63_500,
+	MODE_MT63_LAST = MODE_MT63_2000,
 
 	MODE_BPSK31,
 	MODE_QPSK31,
@@ -90,6 +96,8 @@ enum {
 	MODE_QPSK125,
 	MODE_PSK250,
 	MODE_QPSK250,
+	MODE_PSK_FIRST = MODE_BPSK31,
+	MODE_PSK_LAST = MODE_QPSK250,
 
 	MODE_OLIVIA,
 
@@ -102,6 +110,8 @@ enum {
 //	MODE_TSOR11,
 	MODE_THOR16,
 	MODE_THOR22,
+	MODE_THOR_FIRST = MODE_THOR4,
+	MODE_THOR_LAST = MODE_THOR22,
 
 	MODE_THROB1,
 	MODE_THROB2,
@@ -109,6 +119,8 @@ enum {
 	MODE_THROBX1,
 	MODE_THROBX2,
 	MODE_THROBX4,
+	MODE_THROB_FIRST = MODE_THROB1,
+	MODE_THROB_LAST = MODE_THROBX4,
 
 	MODE_WWV,
 	MODE_ANALYSIS,
@@ -153,4 +165,14 @@ public:
 std::ostream& operator<<(std::ostream& s, const qrg_mode_t& m);
 std::istream& operator>>(std::istream& s, qrg_mode_t& m);
 
+
+enum band_t {
+	BAND_LMW, BAND_160M, BAND_80M, BAND_75M, BAND_60M, BAND_40M, BAND_30M, BAND_20M,
+	BAND_17M, BAND_15M, BAND_12M, BAND_10M, BAND_6M, BAND_4M, BAND_2M, BAND_125CM,
+	BAND_70CM, BAND_33CM, BAND_23CM, BAND_13CM, BAND_9CM, BAND_OTHER, NUM_BANDS
+};
+
+band_t band(long long freq_hz);
+const char* band_name(band_t b);
+
 #endif
