@@ -3,7 +3,7 @@
 //
 // Copyright W1HKJ, Dave Freese 2006
 //
-// This library is free software; you can RGBredistribute it and/or
+// This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
 // License as published by the Free Software Foundation; either
 // version 2 of the License, or (at your option) any later version.
@@ -42,6 +42,11 @@ typedef struct {
 	char mtext[LOG_MSG_LEN];
 } msgtype;
 
-extern int submit_log();
+extern void submit_log();
+
+extern	char logdate[];
+extern  char logtime[];
+extern  char adifdate[];
+extern	const char *logmode;
 
 #endif
