@@ -45,20 +45,16 @@ struct MACROTEXT {
 	void saveMacros(string);
 	string expandMacro(int n);
 	void execute(int n);
-	MACROTEXT() {
-		changed = false;
-		char szname[5];
-		for (int i = 0; i < MAXMACROS; i++) {
-			snprintf(szname, sizeof(szname), "F-%d", i+1);
-			name[i] = szname;//"";
-			text[i] = "";
-		}
-	}
+	MACROTEXT();
 private:
 	string expanded;
+	void loadnewMACROS(string &s, size_t &i);
 };
 
 extern MACROTEXT macros;
 extern CONTESTCNTR contest_count;
 
+extern string info1msg;
+extern string info2msg;
+
 #endif
