@@ -1,6 +1,7 @@
 #ifndef _MAIN_H
 #define _MAIN_H
 
+#include <config.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -23,6 +24,18 @@
 
 extern string		appname;
 extern string		HomeDir;
+extern string		RigsDir;
+extern string		ScriptsDir;
+extern string		PalettesDir;
+extern string		LogsDir;
+extern string		PicsDir;
+extern string		HelpDir;
+extern string		MacrosDir;
+extern string       WrapDir;
+extern string       TalkDir;
+extern string		TempDir;
+extern string		PskMailDir;
+
 extern string		xmlfname;
 
 extern std::string	 scDevice[2];
@@ -33,7 +46,6 @@ extern Rig			*xcvr;
 
 extern cLogfile		*Maillogfile;
 extern cLogfile		*logfile;
-extern string		PskMailDir;
 extern bool			gmfskmail;
 extern bool			arqmode;
 extern string		ArqFilename;
