@@ -41,6 +41,8 @@ protected:
 	double	rx_corr;
 	double	tx_corr;
 	double	tx_frequency;
+	double  PTTphaseacc;
+	double  PTTchannel[OUTBUFSIZE];
 
 // for CW modem use only
 	bool	cwTrack;
@@ -91,9 +93,9 @@ public:
 //	void 		set_mode(trx_mode);
 	trx_mode	get_mode();
 	const char	*get_mode_name() { return mode_info[get_mode()].sname;}
-	void		set_state(state_t);
-	void		set_state_wait(state_t);
-	state_t		get_state();
+//	void		set_state(state_t);
+//	void		set_state_wait(state_t);
+//	state_t		get_state();
 	void		set_freq(double);
 	int			get_freq();
 	void		init_freqlock();
@@ -121,7 +123,8 @@ public:
 	void		set_stopflag(bool b) { stopflag = b;};
 
 	unsigned	get_cap(void) { return cap; }
-	enum { CAP_AFC = 1 << 0, CAP_REV = 1 << 1, CAP_IMG = 1 << 2 };
+	enum { CAP_AFC = 1 << 0, CAP_AFC_SR = 1 << 1, CAP_REV = 1 << 2,
+	       CAP_IMG = 1 << 3, CAP_BW = 1 << 4 };
 
 // for CW modem use only
 	bool		get_cwTrack();
@@ -151,6 +154,8 @@ private:
 
 	void	wfid_sendchar(char c);
 	void	wfid_sendchars(std::string s);
+	
+	double  PTTnco();
 
 public:
 	void	wfid_text(const std::string& s);
