@@ -31,6 +31,8 @@
 #include "filters.h"
 #include "pskcoeff.h"
 #include "pskvaricode.h"
+#include "viewpsk.h"
+#include "pskeval.h"
 
 //=====================================================================
 #define	PskSampleRate	(8000)
@@ -87,6 +89,9 @@ private:
 	complex			quality;
 	int				acquire;
 
+	viewpsk*		pskviewer;
+	pskeval*		evalpsk;
+
 	void			rx_symbol(complex symbol);
 	void 			rx_bit(int bit);
 	void			rx_qpsk(int bits);
