@@ -4,6 +4,8 @@
 //
 // Copyright (C) 2006
 //		Dave Freese, W1HKJ
+// Copyright (C) 2008
+//		Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.  Adapted from code contained in gmfsk source code 
 // distribution.
@@ -27,36 +29,70 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
-#ifndef _PTT_H
-#define _PTT_H
+#ifndef PTT_H_
+#define PTT_H_
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
-#include <termios.h>
-#include <string>
+#include <config.h>
 
-using namespace std;
+#if HAVE_LINUX_PPDEV_H || HAVE_DEV_PPBUS_PPI_H
+#  define HAVE_PARPORT 1
+#else
+#  define HAVE_PARPORT 0
+#endif
+
+#ifdef __APPLE__
+#  define HAVE_UHROUTER 1
+#  define UHROUTER_FIFO_PREFIX "/tmp/microHamRouter"
+#else
+#  define HAVE_UHROUTER 0
+#endif
+
+struct termios;
 
 class PTT {
-private:
-	int pttfd;
-//	int pttinv;
-//	int pttmode;
-	int pttarg;
-	int pttdev;
-	string pttdevName;
-	struct termios oldtio;
-	void  openptt();
-	void reset_(int dev);//, int mode, bool inverted);
 public:
-	PTT(int dev = 0);//, int mode = 0, bool inverted = false);
-	~PTT();
-	void reset(int dev = 0);//, int mode = 0, bool inverted = false);
-	void set(bool on);
+	// The ptt_t enums must be defined even if the corresponding
+	// code is not compiled.  New tags go to the end of the list.
+	enum ptt_t {
+		PTT_INVALID = -1, PTT_NONE, PTT_HAMLIB, PTT_MEMMAP,
+		PTT_RIGCAT, PTT_TTY, PTT_PARPORT, PTT_UHROUTER,
+	};
 
+	PTT(ptt_t dev = PTT_NONE);
+	~PTT();
+	void set(bool on);
+	void reset(ptt_t dev);
+private:
+	ptt_t pttdev;
+
+	// tty and parport
+	int pttfd;
+	struct termios* oldtio;
+
+#if HAVE_UHROUTER
+	// uhrouter
+	int uhkfd[2]; // keyer
+	int uhfd[2];  // ptt
+#endif
+
+	void close_all(void);
+
+	void open_tty(void);
+	void set_tty(bool ptt);
+	void close_tty(void);
+
+#if HAVE_PARPORT
+	void open_parport(void);
+	void set_parport(bool ptt);
+	void close_parport(void);
+#endif
+
+#if HAVE_UHROUTER
+	void open_uhrouter(void);
+	void set_uhrouter(bool ptt);
+	void close_uhrouter(void);
+#endif
 };
 
 
-#endif
+#endif // PTT_H_
