@@ -24,10 +24,12 @@
 #define QRUNNER_H_
 
 #ifndef NDEBUG
+#    include <inttypes.h>
 #    include "debug.h"
 #endif
 
-#include <errno.h>
+#include <unistd.h>
+#include <cerrno>
 #include <stdexcept>
 #include <cstring>
 
@@ -66,15 +68,15 @@ private:
 struct fsignal
 {
         typedef void result_type;
-        Fl_Mutex *m;
-        Fl_Cond *c;
+        pthread_mutex_t* m;
+        pthread_cond_t* c;
 
-        fsignal(Fl_Mutex *m_, Fl_Cond *c_) : m(m_), c(c_) { }
+        fsignal(pthread_mutex_t* m_, pthread_cond_t* c_) : m(m_), c(c_) { }
         void operator()(void) const
         {
-                fl_lock(m);
-                fl_cond_signal(c);
-                fl_unlock(m);
+                pthread_mutex_lock(m);
+                pthread_cond_signal(c);
+                pthread_mutex_unlock(m);
         }
 };
 
@@ -107,7 +109,7 @@ public:
                 }
 
 #ifndef NDEBUG
-		LOG_ERROR("qrunner: thread %d fifo full!", GET_THREAD_ID());
+		LOG_ERROR("qrunner: thread %" PRIdPTR " fifo full!", GET_THREAD_ID());
 #endif
                 return false;
         }
@@ -123,17 +125,17 @@ public:
                                 break;
                         sched_yield();
                 }
-                Fl_Mutex m = PTHREAD_MUTEX_INITIALIZER;
-                Fl_Cond c = PTHREAD_COND_INITIALIZER;
+                pthread_mutex_t m = PTHREAD_MUTEX_INITIALIZER;
+                pthread_cond_t c = PTHREAD_COND_INITIALIZER;
                 fsignal s(&m, &c);
-                fl_lock(&m);
+                pthread_mutex_lock(&m);
                 for (;;) {
                         if (request(s))
                                 break;
                         sched_yield();
                 }
-                fl_cond_wait(&c, &m);
-                fl_unlock(&m);
+                pthread_cond_wait(&c, &m);
+                pthread_mutex_unlock(&m);
 
                 return true;
         }
@@ -155,6 +157,13 @@ public:
 
 extern qrunner *cbq[NUM_QRUNNER_THREADS];
 
+#if BENCHMARK_MODE
+#define REQ(...) ((void)0)
+#define REQ_DROP(...) ((void)0)
+#define REQ_SYNC(...) ((void)0)
+#define REQ_FLUSH(...) ((void)0)
+#define QRUNNER_DROP(...) ((void)0)
+#else
 
 #define REQ REQ_ASYNC
 #define REQ_DROP REQ_ASYNC_DROP
@@ -195,13 +204,15 @@ extern qrunner *cbq[NUM_QRUNNER_THREADS];
                         qrbind::bind(__VA_ARGS__)();			\
         } while (0)
 
-#define REQ_FLUSH()                                                     \
+#define REQ_FLUSH(t_)                                                   \
         do {                                                            \
-                if (GET_THREAD_ID() != FLMAIN_TID)                      \
-                        cbq[GET_THREAD_ID()]->request_sync(nop());      \
-                else                                                    \
-                        for (int i = 0; i < NUM_QRUNNER_THREADS; i++)   \
-                                cbq[i]->flush();                        \
+		if (GET_THREAD_ID() != FLMAIN_TID)			\
+			cbq[GET_THREAD_ID()]->request_sync(nop());	\
+		else if (t_ < NUM_QRUNNER_THREADS)			\
+			cbq[t_]->flush();				\
+		else							\
+			for (int i = 0; i < NUM_QRUNNER_THREADS; i++)	\
+				cbq[i]->flush();			\
         } while (0)
 
 #define QRUNNER_DROP(v_)					\
@@ -209,6 +220,7 @@ extern qrunner *cbq[NUM_QRUNNER_THREADS];
 		if ((GET_THREAD_ID() != FLMAIN_TID))		\
 			cbq[GET_THREAD_ID()]->drop_flag = v_;	\
 	} while (0)
+#endif // BENCHMARK_MODE
 
 #endif // QRUNNER_H_
 
