@@ -15,6 +15,8 @@
 	#include "hamlib.h"
 #endif
 
+using namespace std;
+
 extern Fl_Double_Window *rigcontrol;
 extern string windowTitle;
 extern Cserial rigio;
@@ -29,16 +31,31 @@ extern void updateSelect();
 extern size_t addtoList(long val);
 extern void buildlist();
 extern int  movFreq();
+extern int  qso_movFreq();
+extern int	cb_qso_opMODE();
+extern int  cb_qso_opBW();
+extern void qso_setMode();
+
 extern void selectFreq();
 extern void delFreq();
 extern void addFreq();
 extern void saveFreqList();
 
+extern void qso_addFreq();
+extern void qso_delFreq();
+extern void qso_selectFreq();
+extern void qso_setFreq();
+extern void qso_clearList();
+
 extern bool readRigXML();
 extern bool init_Xml_RigDialog();
+extern bool init_NoRig_RigDialog();
+extern bool init_rigMEM_RigDialog();
+
 #if USE_HAMLIB
 extern bool init_Hamlib_RigDialog();
 extern void selMode(rmode_t m);
+extern string modeString(rmode_t m);
 #endif
 
 extern Fl_Double_Window * createRigDialog();
