@@ -25,9 +25,10 @@
 
 #include <config.h>
 
-#include <sys/time.h>
+#include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <sys/time.h>
 
 #include <string>
 #include <exception>
@@ -76,8 +77,8 @@ class Address
 {
 
 public:
-	Address(const std::string& host = "", int port = 0);
-	Address(const std::string& host, const std::string& port_name);
+	Address(const char* host = "", int port = 0, const char* proto_name = "tcp");
+	Address(const char* host, const char* port_name, const char* proto_name = "tcp");
 	Address(const Address& addr);
 	~Address();
 	Address& operator=(const Address& rhs);
@@ -86,7 +87,7 @@ public:
 	const addr_info_t* get(size_t n = 0) const;
 
 private:
-	void lookup(void);
+	void lookup(const char* proto_name);
 
 	std::string node;
 	std::string service;
@@ -118,6 +119,7 @@ public:
 
 	// Server
         void bind(void);
+	void listen(int backlog = SOMAXCONN);
 	Socket accept(void);
 	Socket accept1(void);
 
@@ -136,6 +138,7 @@ public:
 	void set_nonblocking(bool v = true);
 	void set_nodelay(bool v = true);
 	void set_timeout(const struct timeval& t);
+	void set_timeout(double t);
 	void set_autoclose(bool v) const;
 	void set_close_on_exec(bool v, int fd = -1);
 
