@@ -1,6 +1,8 @@
 #ifndef _status_H
 #define _status_H
 
+#include <string>
+
 #include "globals.h"
 
 struct status {
@@ -35,12 +37,32 @@ struct status {
 	bool	scopeVisible;
 	int		scopeW;
 	int		scopeH;
+	string	LastMacroFile;
+	bool	spot_recv;
+	bool	spot_log;
+	bool	contest;
+	bool	quick_entry;
+	bool	rx_word_wrap;
+	bool	tx_word_wrap;
+
+	int		logbook_x;
+	int		logbook_y;
+	int		logbook_w;
+	int		logbook_h;
+	bool	logbook_reverse;
+	int		logbook_col_0;
+	int		logbook_col_1;
+	int		logbook_col_2;
+	int		logbook_col_3;
+	int		logbook_col_4;
+	int		logbook_col_5;
 
 	bool	bLastStateRead;
 
 	void initLastState();
 	void saveLastState();
 	void loadLastState();
+	
 };
 
 extern status progStatus;
