@@ -7,7 +7,10 @@
 extern "C" {
 #endif
 
-#include <stdint.h>
+#ifndef __STDC_FORMAT_MACROS
+#    define __STDC_FORMAT_MACROS 1
+#endif
+#include <inttypes.h>
 
 #ifndef powerof2
 #    define powerof2(n) ((((n) - 1) & (n)) == 0)
@@ -71,6 +74,7 @@ extern "C" {
 #    define inline__     inline __attribute__ ((__always_inline__))
 #    define noinline__   __attribute__ ((__noinline__))
 #    define nonnull__(x) __attribute__ ((__nonnull__(x)))
+#    define format__(type_, index_, first_) __attribute__ ((format(type_, index_, first_)))
 #else
 #    define likely(x)    (x)
 #    define unlikely(x)  (x)
@@ -86,6 +90,7 @@ extern "C" {
 #    define inline__
 #    define noinline__
 #    define nonnull__(x)
+#    define format__(type_, index_, first_)
 #endif
 
 #if defined(__GNUC__) && ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
@@ -106,10 +111,31 @@ char* strcasestr(const char* haystack, const char* needle);
 
 int set_cloexec(int fd, unsigned char v);
 
+unsigned long ver2int(const char* version);
+
+void save_signals(void);
+void restore_signals(void);
+
 #ifdef __cplusplus
 } // extern "C"
 #endif
 
+#include <stddef.h>
+
+#ifdef __cplusplus
+uint32_t simple_hash_data(const unsigned char* buf, size_t len, uint32_t code = 0);
+uint32_t simple_hash_str(const unsigned char* str, uint32_t code = 0);
+#endif
+
+#ifdef __cplusplus
+const char* str2hex(const unsigned char* str, size_t len);
+const char* str2hex(const char* str, size_t len = 0);
+#else
+const char* str2hex(const unsigned* str, size_t len);
+#endif
+
+const char* uint2bin(unsigned u, size_t len);
+
 #if !defined(NDEBUG) && defined(deprecated__) && defined(__GNUC__)
 #include <stdio.h>
 #include <string.h>
