@@ -29,13 +29,13 @@
 #include "trx.h"
 #include "filters.h"
 #include "fldigi-config.h"
+#include "pskeval.h"
 
 //=====================================================================
 #define	VPSKSAMPLERATE	(8000)
-#define VSNTHRESHOLD 2.0 // 3 db s/n
 #define VAFCDECAY 8
 #define MAXCHANNELS 30
-#define VSEARCHWIDTH 50
+#define VSEARCHWIDTH 70
 #define VSIGSEARCH 5
 #define VWAITCOUNT 4
 //=====================================================================
@@ -60,22 +60,18 @@ private:
 	C_FIR_filter	*fir1[MAXCHANNELS];
 	C_FIR_filter	*fir2[MAXCHANNELS];
 	
-	double			sigpwr[4000];
-	double			sigavg;
-	double			sigmin;
-	Cmovavg			*power[MAXCHANNELS];
-
 	int				bits[MAXCHANNELS];
 	double 			bitclk[MAXCHANNELS];
 	double 			syncbuf[MAXCHANNELS * 16];
 	unsigned int	dcdshreg[MAXCHANNELS];
 	int 			dcd[MAXCHANNELS];
 	int				dcdbits;
-	int				sigsearch[MAXCHANNELS];
 	int				waitcount[MAXCHANNELS];
 	time_t			now;
 	time_t			timeout[MAXCHANNELS];
 
+	pskeval*		evalpsk;
+
 	void			rx_symbol(int ch, complex symbol);
 	void 			rx_bit(int ch, int bit);
 
@@ -83,7 +79,7 @@ private:
 	void			afc(int);
 	
 public:
-	viewpsk(trx_mode mode);
+	viewpsk(pskeval* eval, trx_mode mode);
 	~viewpsk();
 	void init();
 	void restart(trx_mode mode);
@@ -95,4 +91,6 @@ public:
 	void set_freq(int n, double f) { frequency[n] = f; nomfreq[n] = (int)f; }
 };
 
+extern viewpsk *pskviewer;
+
 #endif
