@@ -45,6 +45,11 @@
 #include <FL/Fl_Choice.H>
 #include <FL/Enumerations.H>
 
+enum {
+	WF_FFT_RECTANGULAR, WF_FFT_BLACKMAN, WF_FFT_HAMMING,
+	WF_FFT_HANNING, WF_FFT_TRIANGULAR
+};
+
 /*
 #ifdef HAVE_DFFTW_H
 #  include <dfftw.h>
@@ -54,10 +59,6 @@
 #endif
 */
 
-// recommended minimum size for the control is width = 504, height = 104;
-// the actual waterfall will be width -4 (bezel size) and
-//                              height - 4 - 24 (bezel, text, scale & marker)
-
 #define FFT_LEN		4096
 
 #define SC_SMPLRATE	8000
@@ -70,12 +71,6 @@ struct RGB {
 	uchar B;
 };
 
-struct RGBint {
-	int R;
-	int G;
-	int B;
-};
-
 struct RGBI {
 	uchar R;
 	uchar G;
@@ -89,7 +84,8 @@ extern	RGB		palette[9];
 enum WFmode {
 	WATERFALL,
 	SPECTRUM,
-	SCOPE
+	SCOPE,
+	NUM_WF_MODES
 };
 
 #define MAG_1 1
@@ -158,7 +154,6 @@ public:
 	void draw();
 //	void resize (int, int, int, int);
 	int handle(int event);
-	void handle_mouse_wheel(int event);
 	void update_sigmap();
 	void update_waterfall();
 	void checkoffset();
@@ -179,7 +174,7 @@ public:
 	bool USB() {return usb;};
 	long long rfcarrier() { return rfc;};
 	
-	void useBands(bool b) { usebands = b;};
+//	void useBands(bool b) { usebands = b;};
 	
 	void updateMarker() { 
 		drawMarker();};
@@ -187,12 +182,13 @@ public:
 	double powerDensity(double f0, double bw);
 	void setPrefilter(int v) {
 		switch (v) {
-			case 0: RectWindow(fftwindow, FFT_LEN*2); break;
-			case 1: BlackmanWindow(fftwindow, FFT_LEN*2); break;
-			case 2: HammingWindow(fftwindow, FFT_LEN*2); break;
-			case 3: HanningWindow(fftwindow, FFT_LEN*2); break;
-			case 4: TriangularWindow(fftwindow, FFT_LEN*2); break;
+			case WF_FFT_RECTANGULAR: RectWindow(fftwindow, FFT_LEN*2); break;
+			case WF_FFT_BLACKMAN: BlackmanWindow(fftwindow, FFT_LEN*2); break;
+			case WF_FFT_HAMMING: HammingWindow(fftwindow, FFT_LEN*2); break;
+			case WF_FFT_HANNING: HanningWindow(fftwindow, FFT_LEN*2); break;
+			case WF_FFT_TRIANGULAR: TriangularWindow(fftwindow, FFT_LEN*2); break;
 		}
+		prefilter = v;
 //		switch (v) {
 //			case 0: wfft->setWindow(FFT_NONE); break;
 //			case 1: wfft->setWindow(FFT_BLACKMAN); break;
@@ -221,7 +217,7 @@ private:
 	bool	overload;
 	bool	usb;
 	long long	rfc;
-	bool	usebands;
+//	bool	usebands;
 	int		offset;
 	int		sigoffset;
 	int		step;
@@ -256,6 +252,7 @@ private:
 	int			ptrCB;
 	double		*pwr;
 	Cfft		*wfft;
+	int     prefilter;
 
 
 	int checkMag();
@@ -357,16 +354,20 @@ public:
 			qsy->activate();
 		else
 			qsy->deactivate();
-		wfdisp->useBands(!on);
+//		wfdisp->useBands(!on);
 	}
 	void setXMLRPC(bool on) {
-		wfdisp->useBands(!on);
+//		wfdisp->useBands(!on);
 	}
 	double Pwr(int i) { return wfdisp->Pwr(i); }	
 	
 	int handle(int event);
-/*
-*/
+
+	enum { WF_NOP, WF_AFC_BW, WF_SIGNAL_SEARCH, WF_SQUELCH,
+	       WF_CARRIER, WF_MODEM, WF_SCROLL };
+	static const char wf_wheel_action[];
+	void handle_mouse_wheel(int what, int d);
+
 	Fl_Button	*btnRev;
 	Fl_Counter	*wfcarrier;
 	Fl_Counter	*wfRefLevel;
@@ -391,4 +392,6 @@ private:
 	bool	reverse;
 };
 
+void note_qrg(bool check = true, char prefix = ' ', char suffix = ' ');
+
 #endif
