@@ -75,7 +75,7 @@ public:
 	~wwv();
 	void	init();
 	void	rx_init();
-	void	tx_init(cSound *sc);
+	void	tx_init(SoundBase *sc);
 	void 	restart() {};
 	int		rx_process(const double *buf, int len);
 	int		tx_process() {return -1;}
