@@ -29,7 +29,7 @@
 #include <cstdlib>
 #include <getopt.h>
 #include <sys/types.h>
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(__APPLE__)
 #  include <sys/ipc.h>
 #  include <sys/msg.h>
 #endif
@@ -46,6 +46,7 @@
 #include <FL/Fl_Shared_Image.H>
 #include <FL/x.H>
 
+#include "gettext.h"
 #include "main.h"
 #include "waterfall.h"
 #include "fft.h"
@@ -58,12 +59,17 @@
 #include "cw.h"
 #include "mfsk.h"
 #include "confdialog.h"
+#include "colorsfonts.h"
 #include "configuration.h"
 #include "macros.h"
 #include "status.h"
 #include "fileselect.h"
 #include "timeops.h"
 #include "debug.h"
+#include "pskrep.h"
+#include "logbook.h"
+#include "dxcc.h"
+#include "newinstall.h"
 
 #if USE_HAMLIB
 	#include "rigclass.h"
@@ -79,28 +85,40 @@
 	#include "xmlrpc.h"
 #endif
 
+#if BENCHMARK_MODE
+	#include "benchmark.h"
+#endif
+
 using namespace std;
 
 string appname;
 
 string scDevice[2];
 
-char szHomedir[120] = "";
-char szPskMailDir[120] = "";
+string HomeDir;
+string RigsDir;
+string ScriptsDir;
+string PalettesDir;
+string LogsDir;
+string PicsDir;
+string HelpDir;
+string MacrosDir;
+string WrapDir;
+string TalkDir;
+string TempDir;
 string PskMailDir;
+
 string PskMailFile;
 string ArqFilename;
-string HomeDir;
 string xmlfname;
 
-
 PTT		*push2talk = (PTT *)0;
 #if USE_HAMLIB
 Rig		*xcvr = (Rig *)0;
 #endif
 
 bool tlfio = false;
-cLogfile	*logfile = 0;;
+cLogfile	*logfile = 0;
 cLogfile	*Maillogfile = (cLogfile *)0;
 FILE	*server;
 FILE	*client;
@@ -123,7 +141,8 @@ void generate_version_text(void);
 void debug_exec(char** argv);
 void set_platform_ui(void);
 double speed_test(int converter, unsigned repeat);
-
+static void setup_signal_handlers(void);
+static void checkdirectories(void);
 
 int main(int argc, char ** argv)
 {
@@ -139,50 +158,42 @@ int main(int argc, char ** argv)
 
 	set_unexpected(handle_unexpected);
 	set_terminate(diediedie);
-	signal(SIGSEGV, handle_signal);
-	signal(SIGILL, handle_signal);
-	signal(SIGABRT, handle_signal);
-	signal(SIGCHLD, SIG_IGN);
-	signal(SIGPIPE, SIG_IGN);
+	setup_signal_handlers();
 
+#ifndef ENABLE_NLS
 	setlocale(LC_TIME, "");
-
-#ifdef __CYGWIN__
-	fl_filename_expand(szHomedir, 119, "$USERPROFILE/fldigi.files/");
-	HomeDir = szHomedir;
-#else
-	fl_filename_expand(szHomedir, 119, "$HOME/.fldigi/");
-	HomeDir = szHomedir;
 #endif
 
+	{
+		char dirbuf[FL_PATH_MAX + 1];
+#ifdef __CYGWIN__
+		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$USERPROFILE/fldigi.files/");
+#else
+		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$HOME/.fldigi/");
+#endif
+		HomeDir = dirbuf;
+		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$HOME/");
+		PskMailDir = dirbuf;
+	}
+
 	set_platform_ui();
 
 	generate_option_help();
 	generate_version_text();
 	int arg_idx;
 	if (Fl::args(argc, argv, arg_idx, parse_args) != argc) {
-	    cerr << PACKAGE_NAME << ": unrecognized option `" << argv[arg_idx]
+	    cerr << PACKAGE_NAME << ": bad option `" << argv[arg_idx]
 		 << "'\nTry `" << PACKAGE_NAME
 		 << " --help' for more information.\n";
 	    exit(EXIT_FAILURE);
 	}
 
-	{
-		DIR *dir = opendir(HomeDir.c_str());
-		if (dir == 0) {
-			if ( mkdir(HomeDir.c_str(), 0777) == -1) {
-				cerr << "Could not make directory " << HomeDir << ": "
-				     << strerror(errno) << endl;
-				exit(EXIT_FAILURE);
-			}
-		} else
-			closedir(dir);
-	}
-
+	checkdirectories();
+	bool have_config = progdefaults.readDefaultsXML();
 	try {
 		debug::start(string(HomeDir).append("status_log.txt").c_str());
 		time_t t = time(NULL);
-		LOG(debug::QUIET_LEVEL, "%s log started on %s", PACKAGE_STRING, ctime(&t));
+		LOG(debug::QUIET_LEVEL, debug::LOG_OTHER, _("%s log started on %s"), PACKAGE_STRING, ctime(&t));
 	}
 	catch (const char* error) {
 		cerr << error << '\n';
@@ -191,23 +202,17 @@ int main(int argc, char ** argv)
 
 	xmlfname = HomeDir; 
 	xmlfname.append("rig.xml");
-	
-	string lfname = HomeDir;
-	lfname.append("fldigi.log");
-	logfile = new cLogfile(lfname);
-	logfile->log_to_file_start();
 
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(__APPLE__)
    	txmsgid = msgget( (key_t) progdefaults.tx_msgid, 0666 );
 #else
 	txmsgid = -1;
 #endif
 
-	fl_filename_expand(szPskMailDir, 119, "$HOME/");
-	PskMailDir = szPskMailDir;
 	checkTLF();
 
-	FL_LOCK_E();  // start the gui thread!!	
+
+	Fl::lock();  // start the gui thread!!
 	Fl::visual(FL_RGB); // insure 24 bit color operation
 	
 	fl_register_images();
@@ -215,7 +220,7 @@ int main(int argc, char ** argv)
 	
 	rigcontrol = createRigDialog();
 
-	if (!progdefaults.readDefaultsXML()) {
+	if (!have_config) {
 		double speed = speed_test(SRC_SINC_FASTEST, 8);
 
 		if (speed > 150.0) {      // fast
@@ -238,14 +243,37 @@ int main(int argc, char ** argv)
 		LOG_INFO("speed factor=%f, slowcpu=%d, sample_converter=\"%s\"", speed,
 			 progdefaults.slowcpu, src_get_name(progdefaults.sample_converter));
 	}
+	if (progdefaults.XmlRigFilename.empty())
+		progdefaults.XmlRigFilename = xmlfname;
 
 	progStatus.loadLastState();
 
+	if (progStatus.LOGenabled == true) {
+    	Date tdy;
+	    string lfname = HomeDir;
+	    lfname.append("fldigi");
+	    lfname.append(tdy.szDate(2));
+	    lfname.append(".log");
+	    logfile = new cLogfile(lfname);
+	    logfile->log_to_file_start();
+    }
+
 	Fl::scheme(progdefaults.ui_scheme.c_str());
+	progdefaults.initFonts();
 	create_fl_digi_main();
+
+#if BENCHMARK_MODE
+	return setup_benchmark();
+#endif
+
 	FSEL::create();
 
 	createConfig();
+	make_colorsfonts();
+	setTabColors();
+
+	start_logbook();
+
 	progdefaults.testCommPorts();
 
 	macros.loadDefault();
@@ -263,20 +291,37 @@ int main(int argc, char ** argv)
 	trx_start();
 
 	progdefaults.initInterface();
-	
-#ifdef __CYGWIN__
-	fl_digi_main->icon((char*)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON)));
-#endif
 
+// OS X will prevent the main window from being resized if we change its
+// size *after* it has been shown. With some X11 window managers, OTOH,
+// the main window will not be restored at its exact saved position if
+// we move it *after* it has been shown.
+#ifndef __APPLE__
 	fl_digi_main->show(argc, argv);
 	progStatus.initLastState();
-	
+#else
+	progStatus.initLastState();
+	fl_digi_main->show(argc, argv);
+#endif
+
 	arq_init();
 
 #if USE_XMLRPC
 	XML_RPC_Server::start(progdefaults.xmlrpc_address.c_str(), progdefaults.xmlrpc_port.c_str());
 #endif
 
+	if (progdefaults.usepskrep)
+		if (!pskrep_start())
+			LOG_ERROR("Could not start PSK reporter: %s", pskrep_error());
+
+	if (progdefaults.rxtext_tooltips || progdefaults.autofill_qso_fields)
+		dxcc_open(string(HomeDir).append("cty.dat").c_str());
+	if (progdefaults.rxtext_tooltips) {
+		qsl_open(string(HomeDir).append("lotw1.txt").c_str(), QSL_LOTW);
+		if (!qsl_open(string(HomeDir).append("eqsl.txt").c_str(), QSL_EQSL))
+			qsl_open(string(HomeDir).append("AGMemberList.txt").c_str(), QSL_EQSL);
+	}
+
 	int ret = Fl::run();
 
 	arq_close();
@@ -285,6 +330,9 @@ int main(int argc, char ** argv)
 	XML_RPC_Server::stop();
 #endif
 
+	if (progdefaults.usepskrep)
+		pskrep_stop();
+
 	for (int i = 0; i < NUM_QRUNNER_THREADS; i++) {
 		cbq[i]->detach();
 		delete cbq[i];
@@ -306,7 +354,7 @@ void generate_option_help(void) {
 	     << "    Look for configuration files in DIRECTORY\n"
 	     << "    The default is: " << HomeDir << "\n\n"
 
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(__APPLE__)
 	     << "  --rx-ipc-key KEY\n"
 	     << "    Set the receive message queue key\n"
 	     << "    May be given in hex if prefixed with \"0x\"\n"
@@ -334,6 +382,51 @@ void generate_option_help(void) {
 	     << "  --xmlrpc-server-port PORT\n"
 	     << "    Set the XML-RPC server port\n"
 	     << "    The default is: " << progdefaults.xmlrpc_port << "\n\n"
+	     << "  --xmlrpc-allow REGEX\n"
+	     << "    Allow only the methods whose names match REGEX\n\n"
+	     << "  --xmlrpc-deny REGEX\n"
+	     << "    Allow only the methods whose names don't match REGEX\n\n"
+	     << "  --xmlrpc-list\n"
+	     << "    List all available methods\n\n"
+#endif
+
+#if BENCHMARK_MODE
+	     << "  --benchmark-modem ID\n"
+	     << "    Specify the modem\n"
+	     << "    Default: " << mode_info[benchmark.modem].sname << "\n\n"
+	     << "  --benchmark-frequency FREQ\n"
+	     << "    Specify the modem frequency\n"
+	     << "    Default: " << benchmark.freq << "\n\n"
+	     << "  --benchmark-afc BOOLEAN\n"
+	     << "    Set modem AFC\n"
+	     << "    Default: " << benchmark.afc
+	     << " (" << boolalpha << benchmark.afc << noboolalpha << ")\n\n"
+	     << "  --benchmark-squelch BOOLEAN\n"
+	     << "    Set modem squelch\n"
+	     << "    Default: " << benchmark.sql
+	     << " (" << boolalpha << benchmark.sql << noboolalpha << ")\n\n"
+	     << "  --benchmark-squelch-level LEVEL\n"
+	     << "    Set modem squelch level\n"
+	     << "    Default: " << benchmark.sqlevel << " (%)\n\n"
+	     << "  --benchmark-input INPUT\n"
+	     << "    Specify the input\n"
+	     << "    Must be a positive integer indicating the number of samples\n"
+		"    of silence to generate as the input"
+#  if USE_SNDFILE
+		", or a filename containing\n"
+		"    non-digit characters"
+#endif
+		"\n\n"
+
+	     << "  --benchmark-output FILE\n"
+	     << "    Specify the output data file\n"
+	     << "    Default: decoder output is discarded\n\n"
+	     << "  --benchmark-src-ratio RATIO\n"
+	     << "    Specify the sample rate conversion ratio\n"
+	     << "    Default: 1.0 (input is not resampled)\n\n"
+	     << "  --benchmark-src-type TYPE\n"
+	     << "    Specify the sample rate conversion type\n"
+	     << "    Default: " << benchmark.src_type << " (" << src_get_name(benchmark.src_type) << ")\n\n"
 #endif
 
 	     << "  --debug-level LEVEL\n"
@@ -368,7 +461,7 @@ void generate_option_help(void) {
 	     << "   -g GEOMETRY, -geometry GEOMETRY\n"
 	     << "    Set the initial window size and position\n"
 	     << "    GEOMETRY format is ``WxH+X+Y''\n"
-	     << "    ** " << PACKAGE_NAME << " may override this settting **\n\n"
+	     << "    ** " << PACKAGE_NAME << " may override this setting **\n\n"
 
 	     << "   -i, -iconic\n"
 	     << "    Start " << PACKAGE_NAME << " in iconified state\n\n"
@@ -380,10 +473,7 @@ void generate_option_help(void) {
 	     << "    Set the window class to CLASSNAME\n\n"
 
 	     << "   -ti WINDOWTITLE, -title WINDOWTITLE\n"
-	     << "    Set the window title\n\n"
-
-	     << "   -to, -tooltips or -not, -notooltips\n"
-	     << "    Enable or disable tooltips\n\n";
+	     << "    Set the window title\n\n";
 
 	help << "Additional UI options:\n\n"
 
@@ -392,18 +482,7 @@ void generate_option_help(void) {
 	     << "    The default is: " << Fl::get_font(FL_HELVETICA)
 	     << ':' << FL_NORMAL_SIZE << "\n\n"
 
-	     << "  --wfall-width WIDTH\n"
-	     << "    WIDTH may be 2000 to 4000 in Hz, recommend 50 Hz increments.\n\n"
-
-	     << "  --wfall-height HEIGHT\n"
-	     << "    HEIGHT in pixels, ie 100 - 200, recommend 10 pixel increments.\n\n"
-
-	     << "  --twoscopes\n"
-	     << "    Dock a second digiscope adjacent to the waterfall\n\n"
-
-	     << "  --toggle-check-buttons\n"
-	     << "    Use lighted or check buttons for AFC / SQL.\n";
-
+		;
 
 	option_help = help.str();
 }
@@ -420,20 +499,28 @@ int parse_args(int argc, char **argv, int& idx)
 #ifndef __CYGWIN__
 	       OPT_RX_IPC_KEY, OPT_TX_IPC_KEY,
 #endif
-	       OPT_CONFIG_DIR, OPT_EXPERIMENTAL, OPT_ARQ_ADDRESS, OPT_ARQ_PORT,
+	       OPT_CONFIG_DIR,
+	       OPT_ARQ_ADDRESS, OPT_ARQ_PORT,
 #if USE_XMLRPC
-	       OPT_CONFIG_XMLRPC, OPT_CONFIG_XMLRPC_ADDRESS, OPT_CONFIG_XMLRPC_PORT,
+	       OPT_CONFIG_XMLRPC_ADDRESS, OPT_CONFIG_XMLRPC_PORT,
+	       OPT_CONFIG_XMLRPC_ALLOW, OPT_CONFIG_XMLRPC_DENY, OPT_CONFIG_XMLRPC_LIST,
 #endif
-               OPT_FONT, OPT_WFALL_WIDTH, OPT_WFALL_HEIGHT,
+
+#if BENCHMARK_MODE
+	       OPT_BENCHMARK_MODEM, OPT_BENCHMARK_AFC, OPT_BENCHMARK_SQL, OPT_BENCHMARK_SQLEVEL,
+	       OPT_BENCHMARK_FREQ, OPT_BENCHMARK_INPUT, OPT_BENCHMARK_OUTPUT,
+	       OPT_BENCHMARK_SRC_RATIO, OPT_BENCHMARK_SRC_TYPE,
+#endif
+
+               OPT_FONT, OPT_WFALL_HEIGHT, OPT_WFALL_WIDTH, 
                OPT_WINDOW_WIDTH, OPT_WINDOW_HEIGHT, 
-               OPT_TOGGLE_CHECK,
+//               OPT_TOGGLE_CHECK,
 #if USE_PORTAUDIO
                OPT_FRAMES_PER_BUFFER,
 #endif
-               OPT_TWO_SCOPES,
 	       OPT_DEBUG_LEVEL,
                OPT_EXIT_AFTER,
-               OPT_HELP, OPT_VERSION };
+               OPT_DEPRECATED, OPT_HELP, OPT_VERSION };
 
 	const char shortopts[] = "+";
 	static struct option longopts[] = {
@@ -442,24 +529,39 @@ int parse_args(int argc, char **argv, int& idx)
 		{ "tx-ipc-key",	   1, 0, OPT_TX_IPC_KEY },
 #endif
 		{ "config-dir",	   1, 0, OPT_CONFIG_DIR },
-		{ "experimental",  0, 0, OPT_EXPERIMENTAL },
+		{ "experimental",  0, 0, OPT_DEPRECATED },
 
 		{ "arq-server-address", 1, 0, OPT_ARQ_ADDRESS },
 		{ "arq-server-port",    1, 0, OPT_ARQ_PORT },
 
 #if USE_XMLRPC
-		{ "xmlrpc-server",         0, 0, OPT_CONFIG_XMLRPC },
+		{ "xmlrpc-server",         0, 0, OPT_DEPRECATED },
 		{ "xmlrpc-server-address", 1, 0, OPT_CONFIG_XMLRPC_ADDRESS },
 		{ "xmlrpc-server-port",    1, 0, OPT_CONFIG_XMLRPC_PORT },
+		{ "xmlrpc-allow",          1, 0, OPT_CONFIG_XMLRPC_ALLOW },
+		{ "xmlrpc-deny",           1, 0, OPT_CONFIG_XMLRPC_DENY },
+		{ "xmlrpc-list",           0, 0, OPT_CONFIG_XMLRPC_LIST },
 #endif
+
+#if BENCHMARK_MODE
+		{ "benchmark-modem", 1, 0, OPT_BENCHMARK_MODEM },
+		{ "benchmark-frequency", 1, 0, OPT_BENCHMARK_FREQ },
+		{ "benchmark-afc", 1, 0, OPT_BENCHMARK_AFC },
+		{ "benchmark-squelch", 1, 0, OPT_BENCHMARK_SQL },
+		{ "benchmark-squelch-level", 1, 0, OPT_BENCHMARK_SQLEVEL },
+		{ "benchmark-input", 1, 0, OPT_BENCHMARK_INPUT },
+		{ "benchmark-output", 1, 0, OPT_BENCHMARK_OUTPUT },
+		{ "benchmark-src-ratio", 1, 0, OPT_BENCHMARK_SRC_RATIO },
+		{ "benchmark-src-type", 1, 0, OPT_BENCHMARK_SRC_TYPE },
+#endif
+
 		{ "font",	   1, 0, OPT_FONT },
 
 		{ "wfall-width",   1, 0, OPT_WFALL_WIDTH },
 		{ "wfall-height",  1, 0, OPT_WFALL_HEIGHT },
 		{ "window-width",  1, 0, OPT_WINDOW_WIDTH },
 		{ "window-height", 1, 0, OPT_WINDOW_HEIGHT },
-		{ "twoscopes",     0, 0, OPT_TWO_SCOPES },
- 		{ "toggle-check-buttons",    0, 0, OPT_TOGGLE_CHECK },
+		{ "twoscopes",     0, 0, OPT_DEPRECATED },
 
 #if USE_PORTAUDIO
 		{ "frames-per-buf",1, 0, OPT_FRAMES_PER_BUFFER },
@@ -484,7 +586,7 @@ int parse_args(int argc, char **argv, int& idx)
 			// handle options with non-0 flag here
 			return 0;
 
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(__APPLE__)
 		case OPT_RX_IPC_KEY: case OPT_TX_IPC_KEY:
 		{
 			errno = 0;
@@ -499,13 +601,13 @@ int parse_args(int argc, char **argv, int& idx)
 			break;
 #endif
 
-		case OPT_CONFIG_DIR:
-			HomeDir = optarg;
+		case OPT_CONFIG_DIR: {
+			char buf[FL_PATH_MAX + 1];
+			fl_filename_absolute(buf, sizeof(buf) - 1, optarg);
+			HomeDir = buf;
+		}
 			if (*HomeDir.rbegin() != '/')
-				HomeDir += '/';
-			break;
-		case OPT_EXPERIMENTAL:
-			progdefaults.experimental = true;
+			       HomeDir += '/';
 			break;
 
 		case OPT_ARQ_ADDRESS:
@@ -516,16 +618,73 @@ int parse_args(int argc, char **argv, int& idx)
 			break;
 
 #if USE_XMLRPC
-		case OPT_CONFIG_XMLRPC:
-			cerr << "W: the --" << longopts[longindex].name
-			     << " option will be removed in the next version\n";
-			break;
 		case OPT_CONFIG_XMLRPC_ADDRESS:
 			progdefaults.xmlrpc_address = optarg;
 			break;
 		case OPT_CONFIG_XMLRPC_PORT:
 			progdefaults.xmlrpc_port = optarg;
 			break;
+		case OPT_CONFIG_XMLRPC_ALLOW:
+			progdefaults.xmlrpc_allow = optarg;
+			break;
+		case OPT_CONFIG_XMLRPC_DENY:
+			if (!progdefaults.xmlrpc_allow.empty())
+				cerr << "W: --" << longopts[longindex].name
+				     << " cannot be used together with --"
+				     << longopts[OPT_CONFIG_XMLRPC_ALLOW-1].name
+				     << " and will be ignored\n";
+			else
+				progdefaults.xmlrpc_deny = optarg;
+			break;
+		case OPT_CONFIG_XMLRPC_LIST:
+			XML_RPC_Server::list_methods(cout);
+			exit(EXIT_SUCCESS);
+#endif
+
+#if BENCHMARK_MODE
+		case OPT_BENCHMARK_MODEM:
+			benchmark.modem = strtol(optarg, NULL, 10);
+			if (!(benchmark.modem >= 0 && benchmark.modem < NUM_MODES)) {
+				cerr << "Bad modem id\n";
+				exit(EXIT_FAILURE);
+			}
+			break;
+
+		case OPT_BENCHMARK_FREQ:
+			benchmark.freq = strtol(optarg, NULL, 10);
+			if (benchmark.freq < 0) {
+				cerr << "Bad frequency\n";
+				exit(EXIT_FAILURE);
+			}
+			break;
+
+		case OPT_BENCHMARK_AFC:
+			benchmark.afc = strtol(optarg, NULL, 10);
+			break;
+
+		case OPT_BENCHMARK_SQL:
+			benchmark.sql = strtol(optarg, NULL, 10);
+			break;
+
+		case OPT_BENCHMARK_SQLEVEL:
+			benchmark.sqlevel = strtod(optarg, NULL);
+			break;
+
+		case OPT_BENCHMARK_INPUT:
+			benchmark.input = optarg;
+			break;
+
+		case OPT_BENCHMARK_OUTPUT:
+			benchmark.output = optarg;
+			break;
+
+		case OPT_BENCHMARK_SRC_RATIO:
+			benchmark.src_ratio = strtod(optarg, NULL);
+			break;
+
+		case OPT_BENCHMARK_SRC_TYPE:
+			benchmark.src_type = strtol(optarg, NULL, 10);
+			break;
 #endif
 
 		case OPT_FONT:
@@ -540,11 +699,11 @@ int parse_args(int argc, char **argv, int& idx)
 			break;
 
 		case OPT_WFALL_WIDTH:
-			IMAGE_WIDTH = strtol(optarg, NULL, 10);
+			progdefaults.wfwidth = strtol(optarg, NULL, 10);
 			break;
 
 		case OPT_WFALL_HEIGHT:
-			Hwfall = strtol(optarg, NULL, 10);
+			progdefaults.wfheight = strtol(optarg, NULL, 10);
 			break;
 
 		case OPT_WINDOW_WIDTH:
@@ -555,14 +714,6 @@ int parse_args(int argc, char **argv, int& idx)
 			HNOM = strtol(optarg, NULL, 10);
 			break;
 
-		case OPT_TWO_SCOPES:
-			twoscopes = true;
-			break;
-
-		case OPT_TOGGLE_CHECK:
-			useCheckButtons = !useCheckButtons;
-			break;
-
 #if USE_PORTAUDIO
 		case OPT_FRAMES_PER_BUFFER:
 			progdefaults.PortFramesPerBuffer = strtol(optarg, 0, 10);
@@ -580,6 +731,11 @@ int parse_args(int argc, char **argv, int& idx)
 		}
 			break;
 
+		case OPT_DEPRECATED:
+			cerr << "W: the --" << longopts[longindex].name
+			     << " option has been deprecated and will be removed in a future version\n";
+			break;
+
 		case OPT_HELP:
 			cout << option_help;
 			exit(EXIT_SUCCESS);
@@ -610,7 +766,13 @@ int parse_args(int argc, char **argv, int& idx)
 void generate_version_text(void)
 {
 	ostringstream s;
-	s << PACKAGE_NAME << ' ' << PACKAGE_VERSION << "\n\nSystem: ";
+	s << PACKAGE_STRING << '\n'
+	  << "Copyright (c) 2008 " PACKAGE_AUTHORS "\n" <<
+	   _("License GPLv2+: GNU GPL version 2 or later <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>\n"
+	     "This is free software: you are free to change and redistribute it.\n"
+	     "There is NO WARRANTY, to the extent permitted by law.\n");
+
+	s << _("\nSystem: ");
         struct utsname u;
         if (uname(&u) != -1) {
 		s << u.sysname << ' ' << u.nodename
@@ -680,15 +842,11 @@ void debug_exec(char** argv)
 
 void set_platform_ui(void)
 {
-#if defined (__linux__)
-       FL_NORMAL_SIZE = 12;
-       useCheckButtons = false;
-#elif defined(__APPLE__)
+#if defined(__APPLE__)
        FL_NORMAL_SIZE = 12;
        progdefaults.WaterfallFontsize = 12;
        progdefaults.RxFontsize = 12;
        progdefaults.TxFontsize = 12;
-       useCheckButtons = false;
 #elif defined(__CYGWIN__)
        Fl::set_font(FL_HELVETICA, "Tahoma");
        FL_NORMAL_SIZE = 11;
@@ -696,7 +854,8 @@ void set_platform_ui(void)
        progdefaults.WaterfallFontsize = 12;
        progdefaults.RxFontsize = 12;
        progdefaults.TxFontsize = 12;
-       useCheckButtons = true;
+#else
+       FL_NORMAL_SIZE = 12;
 #endif
 }
 
@@ -743,3 +902,103 @@ double speed_test(int converter, unsigned repeat)
 	t0 = t1 - t0;
 	return repeat / (t0.tv_sec + t0.tv_nsec/1e9);
 }
+
+static void setup_signal_handlers(void)
+{
+#if !defined(__CYGWIN__) && !defined(__MINGW32__)
+	struct sigaction action;
+	memset(&action, 0, sizeof(struct sigaction));
+
+	// no child stopped notifications, no zombies
+	action.sa_handler = SIG_DFL;
+	action.sa_flags = SA_NOCLDSTOP | SA_NOCLDWAIT;
+	sigaction(SIGCHLD, &action, NULL);
+	action.sa_flags = 0;
+
+	action.sa_handler = handle_signal;
+	sigaction(SIGSEGV, &action, NULL);
+	sigaction(SIGILL, &action, NULL);
+	sigaction(SIGABRT, &action, NULL);
+	sigaction(SIGUSR2, &action, NULL);
+
+	action.sa_handler = SIG_IGN;
+	sigaction(SIGPIPE, &action, NULL);
+
+	sigemptyset(&action.sa_mask);
+	sigaddset(&action.sa_mask, SIGUSR2);
+	pthread_sigmask(SIG_BLOCK, &action.sa_mask, NULL);
+#else
+	signal(SIGSEGV, handle_signal);
+	signal(SIGILL, handle_signal);
+	signal(SIGABRT, handle_signal);
+#endif
+}
+
+#ifdef ENABLE_NLS
+int setup_nls(void)
+{
+	static int nls_set_up = 0;
+	if (nls_set_up)
+		return nls_set_up;
+
+	setlocale (LC_MESSAGES, "");
+	setlocale (LC_CTYPE, "C");
+	setlocale (LC_TIME, "");
+	// setting LC_NUMERIC might break the config read/write routines
+
+	const char* ldir;
+	char buf[4096];
+	if (!(ldir = getenv("FLDIGI_LOCALE_DIR"))) {
+		if (getcwd(buf, sizeof(buf) - strlen("/locale") - 1)) {
+			strcpy(buf + strlen(buf), "/locale");
+			struct stat s;
+			if (stat(buf, &s) != -1 && S_ISDIR(s.st_mode))
+				ldir = buf;
+			else
+				ldir = LOCALEDIR;
+		}
+	}
+
+	bindtextdomain(PACKAGE, ldir);
+	/* fltk-1.1.x only knows about Latin-1 */
+	bind_textdomain_codeset(PACKAGE, "ISO-8859-1");
+	textdomain(PACKAGE);
+
+	return nls_set_up = 1;
+}
+#endif
+
+static void checkdirectories(void)
+{
+	struct {
+		string& dir;
+		const char* suffix;
+		void (*new_dir_func)(void);
+	} dirs[] = {
+		{ HomeDir, 0, 0 },
+		{ RigsDir, "rigs", 0 },
+		{ ScriptsDir, "scripts", 0 },
+		{ PalettesDir, "palettes", create_new_palettes },
+		{ LogsDir, "logs", 0 },
+		{ PicsDir, "images", 0 },
+		{ HelpDir, "help", 0 },
+		{ MacrosDir, "macros", create_new_macros },
+		{ WrapDir, "wrap", 0 },
+		{ TalkDir, "talk", 0 },
+		{ TempDir, "temp", 0 },
+	};
+
+	int r;
+	for (size_t i = 0; i < sizeof(dirs)/sizeof(*dirs); i++) {
+		if (dirs[i].suffix)
+			dirs[i].dir.assign(HomeDir).append(dirs[i].suffix).append("/");
+
+		if ((r = mkdir(dirs[i].dir.c_str(), 0777)) == -1 && errno != EEXIST) {
+			cerr << _("Could not make directory") << ' ' << dirs[i].dir
+			     << ": " << strerror(errno) << '\n';
+			exit(EXIT_FAILURE);
+		}
+		else if (r == 0 && dirs[i].new_dir_func)
+			dirs[i].new_dir_func();
+	}
+}
