@@ -32,7 +32,6 @@
 
 #include "mfsk.h"
 #include "modem.h"
-#include "afcind.h"
 
 #include "configuration.h"
 #include "status.h"
@@ -78,7 +77,6 @@ void  mfsk::rx_init()
 	memset(picheader, ' ', PICHEADER - 1);
 	picheader[PICHEADER -1] = 0;
 	put_MODEstatus(mode);
-	set_AFCrange (tonespacing / 10.0);
 	syncfilter->reset();
 	staticburst = false;
 }
@@ -105,6 +103,7 @@ mfsk::~mfsk()
 		picTxWin->hide();
 	if (picRxWin)
 		picRxWin->hide();
+	activate_mfsk_image_item(false);
 
 	if (bpfilt) delete bpfilt;
 	if (rxinlv) delete rxinlv;
@@ -257,10 +256,12 @@ mfsk::mfsk(trx_mode mfsk_mode) : modem()
 	
 // picTxWin and picRxWin are created once to support all instances of mfsk
 	if (!picTxWin) createTxViewer();
-	if (!picRxWin) {
+	if (!picRxWin)
 		createRxViewer();
-		activate_mfsk_image_item(true);
-	}
+	activate_mfsk_image_item(true);
+	afcmetric = 0.0;
+	datashreg = 1;
+
 	init();
 
 }
@@ -637,7 +638,6 @@ void mfsk::afc()
 	if ( fabs(f1 - f) < ts) {
 		freqerr = decayavg(freqerr, (f1 - f), 32);
 		set_freq(frequency - freqerr);
-		set_AFCind( freqerr );
 	}
 
 }
@@ -688,11 +688,11 @@ int mfsk::rx_process(const double *buf, int len)
 				rxstate = RX_STATE_DATA;
 				put_status("");
 #if USE_LIBPNG
-				string autosave_dir = HomeDir + "mfsk_pics/";
+				string autosave_dir = PicsDir;
 				picRx->save_png(autosave_dir.c_str());
 #else
 #  if USE_LIBJPEG
-				string autosave_dir = HomeDir + "mfsk_pics/";
+				string autosave_dir = PicsDir;
 				picRx->save_jpeg(autosave_dir.c_str());
 #  endif
 #endif
@@ -944,7 +944,7 @@ int mfsk::tx_process()
 				}
 				i += blocklen;
 			}
-			REQ_FLUSH();
+			REQ_FLUSH(GET_THREAD_ID());
 
 			txstate = TX_STATE_DATA;
 			put_status("Send picture: done");
