@@ -9,6 +9,8 @@
 #include "macros.h"
 #include "macroedit.h"
 #include "globals.h"
+#include "status.h"
+#include "fileselect.h"
 
 #include <string>
 
@@ -19,8 +21,8 @@ Fl_Double_Window *MacroEditDialog = (Fl_Double_Window *)0;
 Fl_Button	*btnMacroEditOK = (Fl_Button *)0;
 Fl_Button	*btnMacroEditCancel = (Fl_Button *)0;
 Fl_Button	*btnInsertMacro = (Fl_Button *)0;
-Fl_Input	*macrotext = (Fl_Input *)0;
-Fl_Input	*labeltext = (Fl_Input *)0;
+Fl_Input2	*macrotext = (Fl_Input2 *)0;
+Fl_Input2	*labeltext = (Fl_Input2 *)0;
 static int widths[] = {90, 0};
 
 Fl_Hold_Browser *macroDefs=(Fl_Hold_Browser *)0;
@@ -44,6 +46,11 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add("<NAME>\tremote name");
 	w->add("<QTH>\tother qth");
 	w->add("<RST>\tother RST");
+	w->add("<INFO1>\ts/n etc.");
+	w->add("<INFO2>\timd etc.");
+	
+	w->add(LINE_SEP);
+	w->add("<CLRRX>\tclear Rx pane");
 	
 	w->add(LINE_SEP);
 	w->add("<GET>\ttext to NAME/QTH");
@@ -53,6 +60,7 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add("<MODE>\tmode");
 
 	w->add(LINE_SEP);
+	w->add("<QSOTIME>\tqso time HHMM");
 	w->add("<LDT>\tLocal datetime");
 	w->add("<ILDT>\tLDT in iso-8601 format");
 	w->add("<ZDT>\tZulu datetime");
@@ -62,6 +70,7 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add("<CNTR>\tcontest cnt");
 	w->add("<DECR>\tdecr cnt");
 	w->add("<INCR>\tincr cnt");
+	w->add("<XOUT>\tExchange out");
 
 	w->add(LINE_SEP);
 	w->add("<ID>\tMode ID'r");
@@ -73,7 +82,18 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add("<TX>\ttransmit");
 	w->add("<LOG>\tsave QSO data");
 	w->add("<VER>\tFldigi + version");
-	w->add("<TIMER>\trepeat every NNN sec");
+	w->add("<TIMER:NN>\trepeat every NN sec");
+	w->add("<IDLE:NN>\tidle signal for NN sec");
+	
+	w->add(LINE_SEP);
+	w->add("<WPM:NN>\tCW wpm");
+	w->add("<RISE:nn.n>\tCW risetime");
+	w->add("<PRE:nn.n>\tQSK pre-timing");
+	w->add("<POST:+/-nn.n>\tQSK post-timing");
+	
+	w->add(LINE_SEP);
+	w->add("<FILE:>\tinsert text file");
+	w->add("<MACROS:>\tchange macro defs file");
 
 	w->add(LINE_SEP);
 	char s[256];
@@ -83,7 +103,7 @@ void loadBrowser(Fl_Widget *widget) {
 	}
 
 	glob_t gbuf;
-	glob(string(HomeDir).append("scripts/*").c_str(), 0, NULL, &gbuf);
+	glob(string(ScriptsDir).append("*").c_str(), 0, NULL, &gbuf);
 	if (gbuf.gl_pathc == 0) {
 		globfree(&gbuf);
 		return;
@@ -134,15 +154,35 @@ void cbInsertMacro(Fl_Widget *, void *)
 		text.erase(tab);
 	if (text == LINE_SEP)
 		return;
+	if (text == "<FILE:>") {
+		string filters = "Text\t*." "txt";
+		const char* p = FSEL::select("Text file to insert", filters.c_str(),
+					 "text." "txt");
+		if (p) {
+			text.insert(6, p);
+		} else
+			text = "";
+	} else if (text == "<MACROS:>") {
+		string filters = "Macrost\t*." "mdf";
+		const char* p = FSEL::select("Change to Macro file", filters.c_str(),
+					 "macros." "mdf");
+		if (p) {
+			text.insert(8, p);
+		} else
+			text = "";
+	} 
 	macrotext->insert(text.c_str());
 	macrotext->take_focus();
 }
 
-Fl_Double_Window* make_macroeditor() {
-	Fl_Double_Window* w;
-	{ Fl_Double_Window* o = new Fl_Double_Window(700, 230, "Edit User Macro");
-		w = o;
-		labeltext = new Fl_Input(114, 15, 95, 25, "Label:");
+static string editor_label;
+
+Fl_Double_Window* make_macroeditor(void)
+{
+	editor_label.append("Macro editor - ").append(progStatus.LastMacroFile);
+
+	Fl_Double_Window* w = new Fl_Double_Window(700, 230, editor_label.c_str());
+		labeltext = new Fl_Input2(114, 15, 95, 25, "Label:");
 		
 		btnMacroEditOK = new Fl_Button(500, 15, 75, 25, "OK");
 		btnMacroEditOK->callback(cbMacroEditOK);
@@ -150,7 +190,7 @@ Fl_Double_Window* make_macroeditor() {
 		btnMacroEditCancel = new Fl_Button(600, 15, 75, 25, "Cancel");
 		btnMacroEditCancel->callback(cbMacroEditCancel);
 		
-		macrotext = new Fl_Input(5, 60, 450, 165, "Text");
+		macrotext = new Fl_Input2(5, 60, 450, 165, "Text");
 		macrotext->type(4);
 		macrotext->align(FL_ALIGN_TOP_LEFT);
 		
@@ -160,14 +200,18 @@ Fl_Double_Window* make_macroeditor() {
 		macroDefs = new Fl_Hold_Browser(490, 60, 200, 165);
 		macroDefs->column_widths(widths);
 		loadBrowser(macroDefs);
-		o->end();
-	}
+	w->end();
 	return w;
 }
 
 void editMacro(int n)
 {
 	if (!MacroEditDialog) MacroEditDialog = make_macroeditor();
+	else {
+		editor_label = "";
+		editor_label.append("Macro editor - ").append(progStatus.LastMacroFile);
+		MacroEditDialog->label(editor_label.c_str());
+	}
 	macrotext->value(macros.text[n].c_str());
 	labeltext->value(macros.name[n].c_str());
 	iMacro = n;
