@@ -11,6 +11,7 @@
 #include "newinstall.h"
 #include "globals.h"
 #include "debug.h"
+#include "status.h"
 
 #include <FL/Fl.H>
 #include "fileselect.h"
@@ -41,6 +42,7 @@ void pMYLOC(string &, size_t &);
 void pMYNAME(string &, size_t &);
 void pMYQTH(string &, size_t &);
 void pMYRST(string &, size_t &);
+void pQSOTIME(string &, size_t &);
 void pLDT(string &, size_t &);
 void pILDT(string &, size_t &);
 void pZDT(string &, size_t &);
@@ -54,13 +56,24 @@ void pVER(string &, size_t &);
 void pCNTR(string &, size_t &);
 void pDECR(string &, size_t &);
 void pINCR(string &, size_t &);
+void pXOUT(string &, size_t &);
 void pLOG(string &, size_t &);
 void pTIMER(string &, size_t &);
+void pIDLE(string &, size_t &);
 void pMODEM(string &, size_t &);
 void pEXEC(string &, size_t &);
 void pSTOP(string &, size_t &);
 void pCONT(string &, size_t &);
 void pGET(string &, size_t &);
+void pINFO1(string &, size_t &);
+void pINFO2(string &, size_t &);
+void pCLRRX(string &, size_t &);
+void pFILE(string &, size_t &);
+void pWPM(string &, size_t &);
+void pRISETIME(string &, size_t &);
+void pPRE(string &, size_t &);
+void pPOST(string &, size_t &);
+//void pMACROS(string &, size_t &);
 
 MTAGS mtags[] = {
 {"<CALL>",		pCALL},
@@ -75,6 +88,9 @@ MTAGS mtags[] = {
 {"<MYNAME>",	pMYNAME},
 {"<MYQTH>",		pMYQTH},
 {"<MYRST>",		pMYRST},
+{"<QSOTIME>",	pQSOTIME},
+{"<INFO1>",		pINFO1},
+{"<INFO2>",		pINFO2},
 {"<LDT>",		pLDT},
 {"<ILDT>",		pILDT},
 {"<ZDT>",		pZDT},
@@ -88,23 +104,166 @@ MTAGS mtags[] = {
 {"<CNTR>",		pCNTR},
 {"<DECR>",		pDECR},
 {"<INCR>",		pINCR},
+{"<X1>",		pXOUT},
+{"<XOUT>",      pXOUT},
 {"<LOG>",		pLOG},
-{"<TIMER>",		pTIMER},
+{"<TIMER:",		pTIMER},
+{"<IDLE:",		pIDLE},
 {"<MODEM>",		pMODEM},
 {"<EXEC>",		pEXEC},
 {"<STOP>",		pSTOP},
 {"<CONT>",		pCONT},
-{"<GET>",	pGET},
+{"<GET>",		pGET},
+{"<CLRRX>",		pCLRRX},
+{"<FILE:",		pFILE},
+{"<WPM:",       pWPM},
+{"<RISE:",      pRISETIME},
+{"<PRE:",       pPRE},
+{"<POST:",      pPOST},
+//{"<MACROS:",	pMACROS},
 {0, 0}
 };
 
 static bool expand;
 static bool GET = false;
 
+string info1msg = "";
+string info2msg = "";
+
 size_t mystrftime( char *s, size_t max, const char *fmt, const struct tm *tm) {
 	return strftime(s, max, fmt, tm);
 }
 
+void pFILE(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	string fname = s.substr(i+6, endbracket - i - 6);
+	if (fname.length() > 0) {
+		FILE *toadd = fopen(fname.c_str(), "r");
+		string buffer;
+		char c = getc(toadd);
+		while (c && !feof(toadd)) {
+			if (c != '\r') buffer += c; // damn MSDOS txt files
+			c = getc(toadd);
+			}
+		s.replace(i, endbracket - i + 1, buffer);
+		fclose(toadd);
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+void pTIMER(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	int number;
+	string sTime = s.substr(i+7, endbracket - i - 7);
+	if (sTime.length() > 0) {
+		sscanf(sTime.c_str(), "%d", &number);
+		s.replace(i, endbracket - i + 1, "");
+		progdefaults.timeout = number;
+		progdefaults.macronumber = mNbr;
+		progdefaults.useTimer = true;
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+void pWPM(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	int number;
+	string sTime = s.substr(i+5, endbracket - i - 5);
+	if (sTime.length() > 0) {
+		sscanf(sTime.c_str(), "%d", &number);
+		if (number < 5) number = 5;
+		if (number > 200) number = 200;
+		s.replace(i, endbracket - i + 1, "");
+		progdefaults.CWspeed = number;
+        sldrCWxmtWPM->value(number);
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+void pRISETIME(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	float number;
+	string sVal = s.substr(i+6, endbracket - i - 6);
+	if (sVal.length() > 0) {
+		sscanf(sVal.c_str(), "%f", &number);
+		if (number < 0) number = 0;
+		if (number > 20) number = 20;
+		s.replace(i, endbracket - i + 1, "");
+		progdefaults.CWrisetime = number;
+    	cntCWrisetime->value(number);
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+void pPRE(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	float number;
+	string sVal = s.substr(i+5, endbracket - i - 5);
+	if (sVal.length() > 0) {
+		sscanf(sVal.c_str(), "%f", &number);
+		if (number < 0) number = 0;
+		if (number > 20) number = 20;
+		s.replace(i, endbracket - i + 1, "");
+		progdefaults.CWpre = number;
+		cntPreTiming->value(number);
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+void pPOST(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	float number;
+	string sVal = s.substr(i+6, endbracket - i - 6);
+	if (sVal.length() > 0) {
+		sscanf(sVal.c_str(), "%f", &number);
+		s.replace(i, endbracket - i + 1, "");
+		if (number < -20) number = -20;
+		if (number > 20) number = 20;
+		progdefaults.CWpost = number;
+		cntPostTiming->value(number);
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+bool useIdle = false;
+int  idleTime = 0;
+
+void pIDLE(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	int number;
+	string sTime = s.substr(i+6, endbracket - i - 6);
+	if (sTime.length() > 0) {
+		sscanf(sTime.c_str(), "%d", &number);
+		s.replace(i, endbracket - i + 1, "");
+		useIdle = true;
+		idleTime = number;
+	} else
+		s.replace(i, endbracket - i + 1, "");
+}
+
+
+void pINFO1(string &s, size_t &i)
+{
+	s.replace( i, 7, info1msg );
+}
+
+void pINFO2(string &s, size_t &i)
+{
+	s.replace( i, 7, info2msg );
+}
+
+void pCLRRX(string &s, size_t &i)
+{
+	s.replace( i, 7, "" );
+	ReceiveText->clear();
+}
 
 void pCALL(string &s, size_t &i)
 {
@@ -142,6 +301,11 @@ void pQTH(string &s, size_t &i)
 	s.replace( i,5, inpQth->value() );
 }
 
+void pQSOTIME(string &s, size_t &i)
+{
+	s.replace( i, 9, inpTimeOff->value() );
+}
+
 void pRST(string &s, size_t &i)
 {
 	s.replace( i, 5, inpRstOut->value() );
@@ -190,7 +354,7 @@ void pILDT(string &s, size_t &i)
 	tm sTime;
 	time (&tmptr);
 	localtime_r(&tmptr, &sTime);
-	mystrftime(szDt, 79, "%Y-%m-%d %H:%M:%S%z", &sTime);
+	mystrftime(szDt, 79, "%Y-%m-%d %H:%M%z", &sTime);
 	s.replace( i, 6, szDt);
 }
 
@@ -201,7 +365,7 @@ void pZDT(string &s, size_t &i)
 	tm sTime;
 	time (&tmptr);
 	gmtime_r(&tmptr, &sTime);
-	mystrftime(szDt, 79, "%x %H:%M %Z", &sTime);
+	mystrftime(szDt, 79, "%x %H:%MZ", &sTime);
 	s.replace( i, 5, szDt);
 }
 
@@ -212,7 +376,7 @@ void pIZDT(string &s, size_t &i)
 	tm sTime;
 	time (&tmptr);
 	gmtime_r(&tmptr, &sTime);
-	mystrftime(szDt, 79, "%Y-%m-%d %H:%M:%SZ", &sTime);
+	mystrftime(szDt, 79, "%Y-%m-%d %H:%MZ", &sTime);
 	s.replace( i, 6, szDt);
 }
 
@@ -256,53 +420,45 @@ void pVER(string &s, size_t &i)
 void pCNTR(string &s, size_t &i)
 {
 	int  contestval;
-	contestval = contest_count.count + progdefaults.ContestStart;
-	contest_count.Format(progdefaults.ContestDigits, progdefaults.UseLeadingZeros);
-	snprintf(contest_count.szCount, sizeof(contest_count.szCount), contest_count.fmt.c_str(), contestval);
-	s.replace (i, 6, contest_count.szCount);
+	contestval = contest_count.count;
+	if (contestval) {
+		contest_count.Format(progdefaults.ContestDigits, progdefaults.UseLeadingZeros);
+		snprintf(contest_count.szCount, sizeof(contest_count.szCount), contest_count.fmt.c_str(), contestval);
+		s.replace (i, 6, contest_count.szCount);
+	} else
+		s.replace (i, 6, "");
 }
 
 void pDECR(string &s, size_t &i)
 {
 	int  contestval;
 	contest_count.count--;
-	contestval = contest_count.count + progdefaults.ContestStart;
+	if (contest_count.count < 0) contest_count.count = 0;
+	contestval = contest_count.count;
 	s.replace (i, 6, "");
-	snprintf(contest_count.szCount, sizeof(contest_count.szCount), contest_count.fmt.c_str(), contestval);
-	snprintf(contest_count.szDisp, sizeof(contest_count.szDisp), "Next Nbr: %s", contest_count.szCount);
-	put_status(contest_count.szDisp);
+	updateOutSerNo();
 }
 
 void pINCR(string &s, size_t &i)
 {
 	int  contestval;
 	contest_count.count++;
-	contestval = contest_count.count + progdefaults.ContestStart;
+	contestval = contest_count.count;
 	s.replace (i, 6, "");
-	snprintf(contest_count.szCount, sizeof(contest_count.szCount), contest_count.fmt.c_str(), contestval);
-	snprintf(contest_count.szDisp, sizeof(contest_count.szDisp), "Next Nbr: %s", contest_count.szCount);
-	put_status(contest_count.szDisp);
+	updateOutSerNo();
+}
+
+void pXOUT(string &s, size_t &i)
+{
+	s.replace (i, 6, progdefaults.myXchg);
 }
 
 void pLOG(string &s, size_t &i)
 {
 	submit_log();
 	s.replace(i, 5, "");
-	clearQSO();
-}
-
-void pTIMER(string &s, size_t &i)
-{
-	int number;
-	sscanf(s.substr(i+7).c_str(), "%d", &number);
-	size_t i2;
-	i2 = s.find(" ",i);
-	if (i2 == string::npos)
-		i2 = s.find("\n", i);
-	s.replace (i, i2 - i, "");
-	progdefaults.timeout = number;
-	progdefaults.macronumber = mNbr;
-	progdefaults.useTimer = true;
+	if (progdefaults.ClearOnSave)
+		clearQSO();
 }
 
 void pMODEM(string &s, size_t &i)
@@ -339,7 +495,7 @@ void set_env(void)
 	       FLDIGI_MODEM, FLDIGI_MODEM_LONG_NAME, FLDIGI_DIAL_FREQUENCY,
 	       FLDIGI_AUDIO_FREQUENCY, FLDIGI_FREQUENCY,
 
-	       FLDIGI_LOG_FREQUENCY, FLDIGI_LOG_TIME, FLDIGI_LOG_CALL, FLDIGI_LOG_NAME,
+	       FLDIGI_LOG_FREQUENCY, FLDIGI_LOG_TIME_ON, FLDIGI_LOG_TIME_OFF, FLDIGI_LOG_CALL, FLDIGI_LOG_NAME,
 	       FLDIGI_LOG_RST_IN, FLDIGI_LOG_RST_OUT, FLDIGI_LOG_QTH, FLDIGI_LOG_LOCATOR,
 	       FLDIGI_LOG_NOTES, FLDIGI_AZ, ENV_SIZE
 	};
@@ -367,7 +523,8 @@ void set_env(void)
 
 		// logging frame
 		{ "FLDIGI_LOG_FREQUENCY", inpFreq->value() },
-		{ "FLDIGI_LOG_TIME", inpTime->value() },
+		{ "FLDIGI_LOG_TIME_ON", inpTimeOn->value() },
+		{ "FLDIGI_LOG_TIME_OFF", inpTimeOff->value() },
 		{ "FLDIGI_LOG_CALL", inpCall->value() },
 		{ "FLDIGI_LOG_NAME", inpName->value() },
 		{ "FLDIGI_LOG_RST_IN", inpRstIn->value() },
@@ -379,16 +536,11 @@ void set_env(void)
 	};
 
 	// PATH
-	static string path;
-	if (path.length() == 0) {
-		path = HomeDir;
-		if (*path.rbegin() != '/')
-			path += '/';
-		path.append("scripts");
-		const char* p;
-		if ((p = getenv("PATH")))
-			path.append(":").append(p);
-	}
+	static string path = ScriptsDir;
+	path.erase(path.length()-1,1);
+	const char* p;
+	if ((p = getenv("PATH")))
+		path.append(":").append(p);
 	env[PATH].val = path.c_str();
 
 	// IPC keys
@@ -517,15 +669,12 @@ int MACROTEXT::loadMacros(string filename)
 	ifstream mFile(filename.c_str());
 	
 	if (!mFile) {
-		createDotFldigi();
+		create_new_macros();
 		for (int i = 0; i < 12; i++) {
 			btnMacro[i]->label( name[i].c_str());
 			btnMacro[i]->redraw_label();
 		}
 		return 0;
-//		mFile.open(filename.c_str());
-//		if (!mFile)
-//			return -1;
 	}
 	
 	mFile.getline(szLine, 4095);
@@ -570,14 +719,22 @@ int MACROTEXT::loadMacros(string filename)
 		text[mNumber] = text[mNumber] + mLine;
 	}
 	mFile.close();
+	altMacros = 0;
+	btnAltMacros->label("1");
+	btnAltMacros->redraw_label();
 	return 0;
 }
 
 void MACROTEXT::loadDefault()
 {
 	int erc;
-	string Filename = HomeDir;
-	Filename.append("macros.mdf");
+	string Filename = MacrosDir;
+	if (progdefaults.UseLastMacro == true)
+		Filename.append(progStatus.LastMacroFile);
+	else {
+		Filename.append("macros.mdf");
+		progStatus.LastMacroFile = "macros.mdf";
+	}
 	if ((erc = loadMacros(Filename)) != 0)
 #ifndef __CYGWIN__
 		LOG_ERROR("Error #%d loading %s\n", erc, Filename.c_str());
@@ -588,22 +745,38 @@ void MACROTEXT::loadDefault()
 
 void MACROTEXT::openMacroFile()
 {
-	string deffilename = HomeDir;
-	deffilename.append("macros.mdf");
+	string deffilename = MacrosDir;
+	deffilename.append(progStatus.LastMacroFile);
     const char *p = FSEL::select("Open macro file", "Fldigi macro definition file\t*.mdf", deffilename.c_str());
-    if (p)
+    if (p) {
 		loadMacros(p);
+		progStatus.LastMacroFile = fl_filename_name(p);
+	}
+	showMacroSet();
 }
 
 void MACROTEXT::saveMacroFile()
 {
-	string deffilename = HomeDir;
-	deffilename.append("macros.mdf");
+	string deffilename = MacrosDir;
+	deffilename.append(progStatus.LastMacroFile);
     const char *p = FSEL::saveas("Save macro file", "Fldigi macro definition file\t*.mdf", deffilename.c_str());
-    if (p)
+    if (p) {
 		saveMacros(p);
+		progStatus.LastMacroFile = fl_filename_name(p);
+	}
 }
 
+void MACROTEXT::loadnewMACROS(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	string fname = s.substr(i+8, endbracket - i - 8);
+	if (fname.length() > 0) {
+		loadMacros(fname);
+		progStatus.LastMacroFile = fl_filename_name(fname.c_str());
+	}
+	s.replace(i, endbracket - i + 1, "");
+	showMacroSet();
+}
 
 string MACROTEXT::expandMacro(int n)
 {
@@ -615,6 +788,11 @@ string MACROTEXT::expandMacro(int n)
 	MTAGS *pMtags;
 
 	while ((idx = expanded.find('<', idx)) != string::npos) {
+		if (expanded.find("<MACROS:",idx) == idx) {
+			loadnewMACROS(expanded, idx);
+			idx++;
+			continue;
+		}
 		// we must handle this specially
 		if (expanded.find("<CONT>", idx) == idx)
 			pCONT(expanded, idx);
@@ -657,16 +835,44 @@ string MACROTEXT::expandMacro(int n)
 	return expanded;
 }
 
+string text2send = "";
+
+void insertTextAfter(void *)
+{
+	TransmitText->add( text2send.c_str() );
+	text2send.clear();
+}
+
 void MACROTEXT::execute(int n) 
 {
-	TransmitText->add( (expandMacro(n)).c_str() );
+	text2send = expandMacro(n);
 	if ( TransmitON ) {
 		active_modem->set_stopflag(false);
 		start_tx();
 		TransmitON = false;
 	}
+	if (useIdle && idleTime > 0) {
+		Fl::add_timeout(idleTime , insertTextAfter);
+	}
+	else {
+		TransmitText->add( text2send.c_str() );
+		text2send.clear();
+	}
+	useIdle = false;
 }
 
+MACROTEXT::MACROTEXT()
+{
+	changed = false;
+	char szname[5];
+	for (int i = 0; i < MAXMACROS; i++) {
+		snprintf(szname, sizeof(szname), "F-%d", i+1);
+		name[i] = szname;//"";
+		text[i] = "";
+	}
+}
+
+
 string mtext = 
 "//fldigi macro definition file extended\n\
 // This file defines the macro structe(s) for the digital modem program, fldigi\n\
