@@ -21,6 +21,8 @@
 // ----------------------------------------------------------------------------
 
 #include <config.h>
+
+#include <unistd.h>
 #include <iostream>
 #include <cstdlib>
 #include <csignal>
@@ -80,6 +82,8 @@ void handle_unexpected(void)
 // this may not give us anything useful, but we can try...
 void handle_signal(int s)
 {
-        signum = s;
-        diediedie();
+	if (s != SIGUSR2) {
+		signum = s;
+		diediedie();
+	}
 }
