@@ -6,6 +6,7 @@
 
 #include <FL/Fl_Preferences.H>
 
+#include "gettext.h"
 #include "main.h"
 #include "globals.h"
 
@@ -32,15 +33,19 @@
 
 #include "Viewer.h"
 
+#include "lgbook.h"
+#include "logsupport.h"
+#include "qso_db.h"
+
 #define STATUS_FILENAME "status"
 
 status progStatus = {
 	MODE_BPSK31,		// trx_mode	lastmode;
 	50,					// int mainX;
 	50,					// int mainY;
-	(WNOM > 600 ? WNOM : 600),	// int mainW;
-	HNOM,				// int mainH;
-	Hrcvtxt,			// int RxTextHeight;
+	0,				// int mainW;
+	0,				// int mainH;
+	0,				// int RxTextHeight;
 	false,				// bool rigShown;
 	50,					// int rigX;
 	50,					// int rigY;
@@ -66,14 +71,38 @@ status progStatus = {
 	false,				// bool	scopeVisible;
 	50,					// int	scopeW;
 	50,					// int	scopeH;
+	"macros.mdf",		// string LastMacroFile;
+	false,			// bool spot_recv
+	false,			// bool spot_log
+	false,			// bool contest
+
+	false,				// bool quick_entry
+	true,				// bool rx_word_wrap
+	true,				// bool tx_word_wrap
+
+	50,					// int logbook_x;
+	50,					// int logbook_y;
+	380,				// int logbook_w;
+	290,				// int logbook_h;
+	false,				// bool logbook_reverse;
+	85,					// int		logbook_browser_col_0;
+	50,					// int		logbook_browser_col_1;
+	100,				// int		logbook_browser_col_2;
+	110,				// int		logbook_browser_col_3;
+	120,				// int		logbook_browser_col_4;
+	90,					// int		logbook_browser_col_5;
 
 	false				// bool bLastStateRead;
 };
 
 void status::saveLastState()
 {
-	mainX = fl_digi_main->x();
-	mainY = fl_digi_main->y();
+    int mX = fl_digi_main->x();
+    int mY = fl_digi_main->y();
+    if (mX >= 0 && mX >= 0) {
+    	mainX = mX;
+	    mainY = mY;
+    }
 	mainW = fl_digi_main->w();
 	mainH = fl_digi_main->h();
 	RxTextHeight = ReceiveText->h();
@@ -83,11 +112,22 @@ void status::saveLastState()
 	reflevel = progdefaults.wfRefLevel;
 	ampspan = progdefaults.wfAmpSpan;
 
-	LOGenabled = false;
-	Fl_Menu_Item *mnulogging = getMenuItem("Log File");
-	if (mnulogging)
-		LOGenabled = mnulogging->value();
-
+	logbook_x = dlgLogbook->x();
+	logbook_y = dlgLogbook->y();
+	logbook_w = dlgLogbook->w();
+	logbook_h = dlgLogbook->h();
+	logbook_reverse = cQsoDb::reverse;
+	logbook_col_0 = wBrowser->columnWidth(0);
+	logbook_col_1 = wBrowser->columnWidth(1);
+	logbook_col_2 = wBrowser->columnWidth(2);
+	logbook_col_3 = wBrowser->columnWidth(3);
+	logbook_col_4 = wBrowser->columnWidth(4);
+	logbook_col_5 = wBrowser->columnWidth(5);
+	
+	quick_entry = ReceiveText->QuickEntry();
+	rx_word_wrap = ReceiveText->WordWrap();
+	tx_word_wrap = TransmitText->WordWrap();
+	
 	VIEWERvisible = false;
 	if (dlgViewer && dlgViewer->visible()) {
 		VIEWERxpos = dlgViewer->x();
@@ -154,19 +194,44 @@ void status::saveLastState()
 	spref.set("scope_y", scopeY);
 	spref.set("scope_w", scopeW);
 	spref.set("scope_h", scopeH);
+
+	spref.set("last_macro_file", LastMacroFile.c_str());
+
+	spref.set("spot_recv", spot_recv);
+	spref.set("spot_log", spot_recv);
+
+	spref.set("contest", contest);
+	spref.set("quick_entry", quick_entry);
+	spref.set("rx_word_wrap", rx_word_wrap);
+	spref.set("tx_word_wrap", tx_word_wrap);
+
+	spref.set("logbook_x", logbook_x);
+	spref.set("logbook_y", logbook_y);
+	spref.set("logbook_w", logbook_w);
+	spref.set("logbook_h", logbook_h);
+	spref.set("logbook_reverse", logbook_reverse);
+	spref.set("logbook_col_0", logbook_col_0);
+	spref.set("logbook_col_1", logbook_col_1);
+	spref.set("logbook_col_2", logbook_col_2);
+	spref.set("logbook_col_3", logbook_col_3);
+	spref.set("logbook_col_4", logbook_col_4);
+	spref.set("logbook_col_5", logbook_col_5);
+
 }
 
 void status::loadLastState()
 {
 	Fl_Preferences spref(HomeDir.c_str(), "w1hkj.com", PACKAGE_TARNAME);
+	
 	char version[64]; version[sizeof(version)-1] = '\0';
+	
 	bLastStateRead = spref.get("version", version, "", sizeof(version)-1);
 	// Skip loading the rest of the status variables if we didn't read a
 	// version name/value pair; also clear everything to avoid creating
 	// entries out of existing file contents.
 	if (!bLastStateRead) {
-		while (spref.entries())
-			spref.deleteEntry(spref.entry(0));
+//		while (spref.entries())
+//			spref.deleteEntry(spref.entry(0));
 		return;
 	}
 
@@ -211,6 +276,32 @@ void status::loadLastState()
 	spref.get("scope_y", scopeY, scopeY);
 	spref.get("scope_w", scopeW, scopeW);
 	spref.get("scope_h", scopeH, scopeH);
+
+	char *defbuffer;
+	spref.get("last_macro_file", defbuffer, "macros.mdf");
+	LastMacroFile = defbuffer;
+	if (defbuffer) free(defbuffer);
+
+	spref.get("spot_recv", i, i); spot_recv = i;
+	spref.get("spot_log", i, i); spot_log = i;
+
+	spref.get("contest", i, i); contest = i;
+	spref.get("quick_entry", i, i); quick_entry = i;
+	spref.get("rx_word_wrap", i, i); rx_word_wrap = i;
+	spref.get("tx_word_wrap", i, i); tx_word_wrap = i;
+
+	spref.get("logbook_x", logbook_x, logbook_x);
+	spref.get("logbook_y", logbook_y, logbook_y);
+	spref.get("logbook_w", logbook_w, logbook_w);
+	spref.get("logbook_h", logbook_h, logbook_h);
+	spref.get("logbook_reverse", i, i); logbook_reverse = i;
+	spref.get("logbook_col_0", logbook_col_0, logbook_col_0);
+	spref.get("logbook_col_1", logbook_col_1, logbook_col_1);
+	spref.get("logbook_col_2", logbook_col_2, logbook_col_2);
+	spref.get("logbook_col_3", logbook_col_3, logbook_col_3);
+	spref.get("logbook_col_4", logbook_col_4, logbook_col_4);
+	spref.get("logbook_col_5", logbook_col_5, logbook_col_5);
+
 }
 
 void status::initLastState()
@@ -218,9 +309,7 @@ void status::initLastState()
 	if (!bLastStateRead)
 		loadLastState();
 
-	init_modem(lastmode);
-
-	while (!active_modem) MilliSleep(100);
+	init_modem_sync(lastmode);
 
  	wf->opmode();
 	wf->Mag(mag);
@@ -228,34 +317,24 @@ void status::initLastState()
 	wf->setRefLevel();
 	wf->setAmpSpan();
 	wf->movetocenter();
-	
-	FL_LOCK_D();
+
 	btn_afconoff->value(afconoff);
 	btn_sqlonoff->value(sqlonoff);
 	sldrSquelch->value(sldrSquelchValue);
 	valRcvMixer->value(RcvMixer * 100.0);
 	valXmtMixer->value(XmtMixer * 100.0);
 
-	FL_UNLOCK_D();
+	if (mainW == 0)
+		mainW = MAX(WMIN, Fl::w() / 2);
+	if (mainH == 0)
+		mainH = MAX(HMIN, Fl::h() / 2);
+	fl_digi_main->resize(mainX, mainY, mainW, mainH);
 
-	{
-		fl_digi_main->resize(mainX, mainY, mainW, mainH);
+	if (!(RxTextHeight > 0 && RxTextHeight < TiledGroup->h()))
+		RxTextHeight = TiledGroup->h() / 3 * 2;
+	TiledGroup->position(0, TransmitText->y(), 0, TiledGroup->y() + RxTextHeight);
 
-		int X, Y, W, H, Yx, Hx;
-		X = ReceiveText->x();
-		Y = ReceiveText->y();
-		W = ReceiveText->w();
-		H = ReceiveText->h();
-		Yx = TransmitText->y();
-		Hx = TransmitText->h();	
-
-//		ReceiveText->resize(X,Y,W,RxTextHeight);
-//		FHdisp->resize(X,Y,W,RxTextHeight);
-//		TransmitText->resize(X, Y + RxTextHeight, W, H + Hx - RxTextHeight);
-		TiledGroup->position( X, Y + H, X, Y + RxTextHeight);
-	}
-
-	if (rigShown == true) {
+	if (!progdefaults.docked_rig_control && rigShown == true) {
 		if (!rigcontrol)
 			createRigDialog();
 		rigcontrol->resize(rigX, rigY, rigW, rigH);
@@ -269,11 +348,23 @@ void status::initLastState()
 		if (scopeVisible == true)
 			scopeview->show();
 	}
+
+	cQsoDb::reverse = logbook_reverse;
+	if (cQsoDb::reverse) {
+		qsodb.SortByDate();
+		loadBrowser();
+	}
+	
+	dlgLogbook->resize(logbook_x, logbook_y, logbook_w, logbook_h);
+	wBrowser->columnWidth(0, logbook_col_0);
+	wBrowser->columnWidth(1, logbook_col_1);
+	wBrowser->columnWidth(2, logbook_col_2);
+	wBrowser->columnWidth(3, logbook_col_3);
+	wBrowser->columnWidth(4, logbook_col_4);
+	wBrowser->columnWidth(5, logbook_col_5);
+	
+	ReceiveText->QuickEntry(quick_entry);
+	ReceiveText->WordWrap(rx_word_wrap);
+	TransmitText->WordWrap(tx_word_wrap);
 	
-	if (LOGenabled) {
-		Fl_Menu_Item *mnulogging = getMenuItem("Log File");
-		if (!mnulogging)
-			return;
-		mnulogging->set();
-	}		
 }
