@@ -4,6 +4,10 @@
 // Copyright (C) 2008
 //              Stelios Bounanos, M0GLD
 //
+// See EOF for a list of method names. Run "fldigi --xmlrpc-list"
+// to see a list of method names, signatures and descriptions.
+//
+//
 // This file is part of fldigi.
 //
 // fldigi is free software; you can redistribute it and/or modify
@@ -24,12 +28,17 @@
 
 #include "xmlrpc.h"
 
+#include <iostream>
+#include <iomanip>
 #include <string>
 #include <vector>
+#include <list>
 #include <map>
 #include <exception>
 #include <cstdlib>
 
+#include <signal.h>
+
 #include <xmlrpc-c/base.hpp>
 #include <xmlrpc-c/registry.hpp>
 #include <xmlrpc-c/server_abyss.hpp>
@@ -54,6 +63,10 @@
 #include "rigMEM.h"
 #include "rigio.h"
 #include "debug.h"
+#include "re.h"
+#include "pskrep.h"
+
+LOG_SET_SOURCE(debug::LOG_RPC);
 
 using namespace std;
 
@@ -64,27 +77,33 @@ struct rpc_method
 	xmlrpc_c::methodPtr method;
 	const char* name;
 };
-vector<rpc_method>* methods;
+typedef list<rpc_method> methods_t;
+methods_t* methods = 0;
 
-static Fl_Thread* server_thread;
+static pthread_t* server_thread = 0;
 
 XML_RPC_Server* XML_RPC_Server::inst = 0;
 
 XML_RPC_Server::XML_RPC_Server()
 {
 	server_socket = new Socket;
-	methods = new vector<rpc_method>;
 	add_methods();
-
-	server_thread = new Fl_Thread;
+	server_thread = new pthread_t;
 	run = true;
 }
+
 XML_RPC_Server::~XML_RPC_Server()
 {
 	run = false;
-	pthread_join(*server_thread, NULL);
-	delete server_thread;
+	if (server_thread) {
+		CANCEL_THREAD(*server_thread);
+		pthread_join(*server_thread, NULL);
+		delete server_thread;
+		server_thread = 0;
+	}
 	delete methods;
+	delete server_socket;
+	methods = 0;
 }
 
 
@@ -97,13 +116,22 @@ void XML_RPC_Server::start(const char* node, const char* service)
 
 	try {
 		inst->server_socket->open(Address(node, service));
+		inst->server_socket->bind();
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+		inst->server_socket->listen();
+		inst->server_socket->set_timeout(0.1);
+#endif
 	}
 	catch (const SocketException& e) {
 		LOG_ERROR("Could not start XML-RPC server (%s)", e.what());
+		delete server_thread;
+		server_thread = 0;
+		delete inst;
+		inst = 0;
 		return;
 	}
 
-	fl_create_thread(*server_thread, thread_func, NULL);
+	pthread_create(server_thread, NULL, thread_func, NULL);
 }
 
 void XML_RPC_Server::stop(void)
@@ -119,47 +147,55 @@ void* XML_RPC_Server::thread_func(void*)
 	SET_THREAD_ID(XMLRPC_TID);
 
 	xmlrpc_c::registry reg;
-	for (vector<rpc_method>::iterator i = methods->begin(); i != methods->end(); i++)
+	for (methods_t::iterator i = methods->begin(); i != methods->end(); ++i)
 		reg.addMethod(i->name, i->method);
 
-
+	save_signals();
 	xmlrpc_c::serverAbyss server(xmlrpc_c::serverAbyss::constrOpt()
 				     .registryP(&reg)
 #ifndef NDEBUG
 				     .logFileName(HomeDir + "xmlrpc.log")
 #endif
 	    		      );
+	restore_signals();
 
-	try {
-		inst->server_socket->bind();
-		struct timeval t = { 0, 200000 };
-		inst->server_socket->set_timeout(t);
-		inst->server_socket->set_nonblocking();
-	}
-	catch (const SocketException& e) {
-		LOG_ERROR("Could not start XML-RPC server (%s)", e.what());
-		goto ret;
-	}
+	SET_THREAD_CANCEL();
 
+	// On POSIX we block indefinitely and are interrupted by a signal.
+	// On woe32 we block for a short time and test for cancellation.
 	while (inst->run) {
-		Socket client;
 		try {
-			server.runConn(inst->server_socket->accept().fd());
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+			if (inst->server_socket->wait(0))
+#endif
+				server.runConn(inst->server_socket->accept().fd());
+			TEST_THREAD_CANCEL();
 		}
 		catch (const SocketException& e) {
-			if (e.error() != ETIMEDOUT) {
+			if (e.error() != EINTR)
 				LOG_ERROR("%s", e.what());
-				break;
-			}
+			break;
+		}
+		catch (...) {
+			break;
 		}
 	}
 
-ret:
 	inst->server_socket->close();
-
 	return NULL;
 }
 
+ostream& XML_RPC_Server::list_methods(ostream& out)
+{
+	add_methods();
+
+	ios_base::fmtflags f = out.flags(ios::left);
+	for (methods_t::const_iterator i = methods->begin(); i != methods->end(); ++i)
+		out << setw(32) << i->name << setw(8) << i->method->signature()
+		    << i->method->help() << '\n';
+
+	return out << setiosflags(f);
+}
 
 // =============================================================================
 // helper functions
@@ -174,6 +210,12 @@ static void set_valuator(Fl_Valuator* valuator, double value)
 	valuator->value(value);
 	valuator->do_callback();
 }
+static void set_text(Fl_Input* textw, string& value)
+{
+	textw->value(value.c_str());
+	textw->do_callback();
+}
+
 
 // =============================================================================
 
@@ -192,7 +234,7 @@ public:
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
 		vector<xmlrpc_c::value> help;
-		for (vector<rpc_method>::const_iterator i = methods->begin(); i != methods->end(); i++) {
+		for (methods_t::const_iterator i = methods->begin(); i != methods->end(); ++i) {
 			map<string, xmlrpc_c::value> item;
 			item["name"] = xmlrpc_c::value_string(i->name);
 			item["signature"] = xmlrpc_c::value_string(i->method->signature());
@@ -264,6 +306,20 @@ public:
 	}
 };
 
+class Fldigi_config_dir : public xmlrpc_c::method
+{
+public:
+	Fldigi_config_dir()
+	{
+		_signature = "s:n";
+		_help = "Returns the name of theconfiguration directory.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		*retval = xmlrpc_c::value_string(HomeDir);
+	}
+};
+
 // =============================================================================
 
 class Modem_get_name : public xmlrpc_c::method
@@ -338,19 +394,15 @@ public:
         {
 		const char* cur = mode_info[active_modem->get_mode()].sname;
 
-		bool found = false;
 		string s = params.getString(0);
 		for (size_t i = 0; i < NUM_MODES; i++) {
 			if (s == mode_info[i].sname) {
 				REQ_SYNC(init_modem_sync, i);
-				found = true;
-				break;
+				*retval = xmlrpc_c::value_string(cur);
+				return;
 			}
 		}
-		if (!found)
-			throw xmlrpc_c::fault("No such modem", xmlrpc_c::fault::code_t(0));
-
-		*retval = xmlrpc_c::value_string(cur);
+		throw xmlrpc_c::fault("No such modem");
 	}
 };
 
@@ -436,6 +488,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
+		if (!(active_modem->get_cap() & modem::CAP_AFC_SR))
+			throw xmlrpc_c::fault("Operation not supported by modem");
+
 		*retval = xmlrpc_c::value_int((int)cntSearchRange->value());
 	}
 };
@@ -450,6 +505,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
+		if (!(active_modem->get_cap() & modem::CAP_AFC_SR))
+			throw xmlrpc_c::fault("Operation not supported by modem");
+
 		int v = (int)(cntSearchRange->value());
 		REQ(set_valuator, cntSearchRange, params.getInt(0, (int)cntSearchRange->minimum(), (int)cntSearchRange->maximum()));
 		*retval = xmlrpc_c::value_int(v);
@@ -466,6 +524,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
+		if (!(active_modem->get_cap() & modem::CAP_AFC_SR))
+			throw xmlrpc_c::fault("Operation not supported by modem");
+
 		int v = (int)(cntSearchRange->value() + params.getInt(0));
 		REQ(set_valuator, cntSearchRange, v);
 		*retval = xmlrpc_c::value_int(v);
@@ -476,6 +537,19 @@ public:
 
 extern Fl_Value_Slider* sldrHellBW; // FIXME: export in header
 extern Fl_Value_Slider* sldrCWbandwidth; // FIXME: export in header
+static Fl_Valuator* get_bw_val(void)
+{
+	if (!(active_modem->get_cap() & modem::CAP_BW))
+		throw xmlrpc_c::fault("Operation not supported by modem");
+
+	trx_mode m = active_modem->get_mode();
+	if (m >= MODE_HELL_FIRST && m <= MODE_HELL_LAST)
+		return sldrHellBW;
+	else if (m == MODE_CW)
+		return sldrCWbandwidth;
+
+	throw xmlrpc_c::fault("Unknown CAP_BW modem");
+}
 
 class Modem_get_bw : public xmlrpc_c::method
 {
@@ -487,19 +561,7 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		Fl_Valuator* val;
-		switch (active_modem->get_mode()) {
-		case MODE_FELDHELL:
-			val = sldrHellBW;
-			break;
-		case MODE_CW:
-			val = sldrCWbandwidth;
-			break;
-		default:
-			throw xmlrpc_c::fault("Operation not supported by modem", xmlrpc_c::fault::code_t(0));
-		}
-
-		*retval = xmlrpc_c::value_int((int)val->value());
+		*retval = xmlrpc_c::value_int((int)get_bw_val()->value());
 	}
 };
 
@@ -513,18 +575,7 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		Fl_Valuator* val;
-		switch (active_modem->get_mode()) {
-		case MODE_FELDHELL:
-			val = sldrHellBW;
-			break;
-		case MODE_CW:
-			val = sldrCWbandwidth;
-			break;
-		default:
-			throw xmlrpc_c::fault("Operation not supported by modem", xmlrpc_c::fault::code_t(0));
-		}
-
+		Fl_Valuator* val = get_bw_val();
 		int v = (int)(val->value());
 		REQ(set_valuator, val, params.getInt(0, (int)val->minimum(), (int)val->maximum()));
 		*retval = xmlrpc_c::value_int(v);
@@ -541,18 +592,7 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		Fl_Valuator* val;
-		switch (active_modem->get_mode()) {
-		case MODE_FELDHELL:
-			val = sldrHellBW;
-			break;
-		case MODE_CW:
-			val = sldrCWbandwidth;
-			break;
-		default:
-			throw xmlrpc_c::fault("Operation not supported by modem", xmlrpc_c::fault::code_t(0));
-		}
-
+		Fl_Valuator* val = get_bw_val();
 		int v = (int)(val->value() + params.getInt(0));
 		REQ(set_valuator, val, v);
 		*retval = xmlrpc_c::value_int(v);
@@ -649,6 +689,34 @@ public:
 	}
 };
 
+class Main_set_sb : public xmlrpc_c::method
+{
+public:
+	Main_set_sb()
+	{
+		_signature = "n:s";
+		_help = "Sets the sideband to USB or LSB.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		string s = params.getString(0);
+		if (s != "LSB" && s != "USB")
+			throw xmlrpc_c::fault("Invalid argument");
+
+		if (progdefaults.chkUSERIGCATis)
+			rigCAT_setmode(s);
+#if USE_HAMLIB
+		else if (progdefaults.chkUSEHAMLIBis)
+			hamlib_setmode(s == "LSB" ? RIG_MODE_LSB : RIG_MODE_USB);
+#endif
+		else if (progdefaults.chkUSEXMLRPCis)
+			REQ(static_cast<void (waterfall::*)(bool)>(&waterfall::USB), wf, s == "USB");
+
+		*retval = xmlrpc_c::value_nil();
+	}
+};
+
+
 class Main_get_freq : public xmlrpc_c::method
 {
 public:
@@ -663,17 +731,18 @@ public:
 	}
 };
 
-void xmlrpc_set_qsy(long long f, long long fmid)
+void xmlrpc_set_qsy(long long rfc, long long fmid)
 {
-	LOG_DEBUG("rfc = %ld, audio = %ld", f, fmid);
-	if (active_modem->freqlocked() == true) {
+	if (active_modem->freqlocked()) {
 		active_modem->set_freqlock(false);
 		active_modem->set_freq((int)fmid);
 		active_modem->set_freqlock(true);
-	} else
+	}
+	else
 		active_modem->set_freq((int)fmid);
-	wf->rfcarrier(f);
+	wf->rfcarrier(rfc);
 	wf->movetocenter();
+	qsoFreqDisp->value(rfc);
 }
 
 class Main_set_freq : public xmlrpc_c::method
@@ -686,20 +755,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		double d = params.getDouble(0, 0.0);
-		long long int rfc = (long long int)d;
-		int mc = active_modem->get_freq();
-		if (progdefaults.chkUSEXMLRPCis)
-			xmlrpc_set_qsy(rfc, mc);
-		else if (progdefaults.chkUSERIGCATis)
-			rigCAT_set_qsy(rfc, mc);
-		else if (progdefaults.chkUSEMEMMAPis)
-			rigMEM_set_qsy(rfc, mc);
-#if USE_HAMLIB
-		else if (progdefaults.chkUSEHAMLIBis)
-			hamlib_set_qsy(rfc, mc);
-#endif
-		*retval = xmlrpc_c::value_double(d);
+		double rfc = wf->rfcarrier();
+		qsy((long long int)params.getDouble(0, 0.0));
+		*retval = xmlrpc_c::value_double(rfc);
 	}
 };
 
@@ -714,22 +772,11 @@ public:
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
 		double rfc = wf->rfcarrier() + params.getDouble(0);
-		int mc = active_modem->get_freq();
-		if (progdefaults.chkUSEXMLRPCis)
-			xmlrpc_set_qsy((long long int)rfc, mc);
-		if (progdefaults.chkUSERIGCATis)
-			rigCAT_set_qsy((long long int)rfc, mc);
-		if (progdefaults.chkUSEMEMMAPis)
-			rigMEM_set_qsy((long long int)rfc, mc);
-#if USE_HAMLIB
-		if (progdefaults.chkUSEHAMLIBis)
-			hamlib_set_qsy((long long int)rfc, mc);
-#endif
+		qsy((long long int)rfc);
 		*retval = xmlrpc_c::value_double(rfc);
 	}
 };
 
-
 // =============================================================================
 
 class Main_get_afc : public xmlrpc_c::method
@@ -772,9 +819,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		bool v = btn_afconoff->value();
-		REQ(set_button, btn_afconoff, !v);
-		*retval = xmlrpc_c::value_boolean(!v);
+		bool v = !btn_afconoff->value();
+		REQ(set_button, btn_afconoff, v);
+		*retval = xmlrpc_c::value_boolean(v);
 	}
 };
 
@@ -820,9 +867,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		bool v = btn_sqlonoff->value();
-		REQ(set_button, btn_sqlonoff, !v);
-		*retval = xmlrpc_c::value_boolean(!v);
+		bool v = !btn_sqlonoff->value();
+		REQ(set_button, btn_sqlonoff, v);
+		*retval = xmlrpc_c::value_boolean(v);
 	}
 };
 
@@ -853,8 +900,8 @@ public:
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
 		double v = sldrSquelch->value();
-		// Squelch slider min/max are reversed when !twoscopes. Argh.
-		if (twoscopes)
+		// Squelch slider min/max are reversed when !docked_scope. Argh.
+		if (progdefaults.docked_scope)
 			REQ(set_valuator, sldrSquelch, params.getDouble(0, sldrSquelch->minimum(), sldrSquelch->maximum()));
 		else
 			REQ(set_valuator, sldrSquelch, params.getDouble(0, sldrSquelch->maximum(), sldrSquelch->minimum()));
@@ -920,9 +967,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		bool v = wf->btnRev->value();
-		REQ(set_button, wf->btnRev, !v);
-		*retval = xmlrpc_c::value_boolean(!v);
+		bool v = !wf->btnRev->value();
+		REQ(set_button, wf->btnRev, v);
+		*retval = xmlrpc_c::value_boolean(v);
 	}
 };
 
@@ -968,9 +1015,9 @@ public:
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		bool v = wf->xmtlock->value();
-		REQ(set_button, wf->xmtlock, !v);
-		*retval = xmlrpc_c::value_boolean(!v);
+		bool v = !wf->xmtlock->value();
+		REQ(set_button, wf->xmtlock, v);
+		*retval = xmlrpc_c::value_boolean(v);
 	}
 };
 
@@ -1134,17 +1181,31 @@ public:
 	}
 };
 
-class Log_get_time : public xmlrpc_c::method
+class Log_get_time_on : public xmlrpc_c::method
 {
 public:
-	Log_get_time()
+	Log_get_time_on()
 	{
 		_signature = "s:n";
-		_help = "Returns the Time field contents.";
+		_help = "Returns the Time-On field contents.";
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		*retval = xmlrpc_c::value_string(inpTime->value());
+		*retval = xmlrpc_c::value_string(inpTimeOn->value());
+	}
+};
+
+class Log_get_time_off : public xmlrpc_c::method
+{
+public:
+	Log_get_time_off()
+	{
+		_signature = "s:n";
+		_help = "Returns the Time-Off field contents.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		*retval = xmlrpc_c::value_string(inpTimeOff->value());
 	}
 };
 
@@ -1162,6 +1223,22 @@ public:
 	}
 };
 
+class Log_set_call : public xmlrpc_c::method
+{
+public:
+	Log_set_call()
+	{
+		_signature = "n:s";
+		_help = "Sets the Call field contents.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		REQ(set_text, inpCall, params.getString(0));
+
+		*retval = xmlrpc_c::value_nil();
+	}
+};
+
 class Log_get_name : public xmlrpc_c::method
 {
 public:
@@ -1204,6 +1281,62 @@ public:
 	}
 };
 
+class Log_get_serial_number : public xmlrpc_c::method
+{
+public:
+	Log_get_serial_number()
+	{
+		_signature = "s:n";
+		_help = "Returns the serial number field contents.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		*retval = xmlrpc_c::value_string(inpSerNo->value());
+	}
+};
+
+class Log_get_state : public xmlrpc_c::method
+{
+public:
+	Log_get_state()
+	{
+		_signature = "s:n";
+		_help = "Returns the State field contents.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		*retval = xmlrpc_c::value_string(inpState->value());
+	}
+};
+
+class Log_get_province : public xmlrpc_c::method
+{
+public:
+	Log_get_province()
+	{
+		_signature = "s:n";
+		_help = "Returns the Province field contents.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		*retval = xmlrpc_c::value_string(inpVEprov->value());
+	}
+};
+
+class Log_get_country : public xmlrpc_c::method
+{
+public:
+	Log_get_country()
+	{
+		_signature = "s:n";
+		_help = "Returns the Country field contents.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+        {
+		*retval = xmlrpc_c::value_string(inpCountry->value());
+	}
+};
+
 class Log_get_qth : public xmlrpc_c::method
 {
 public:
@@ -1224,11 +1357,11 @@ public:
 	Log_get_band()
 	{
 		_signature = "s:n";
-		_help = "Returns the Band selection.";
+		_help = "Returns the current band name.";
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		*retval = xmlrpc_c::value_string(cboBand->value());
+		*retval = xmlrpc_c::value_string(band_name(band(wf->rfcarrier())));
 	}
 };
 
@@ -1238,11 +1371,11 @@ public:
 	Log_get_sb()
 	{
 		_signature = "s:n";
-		_help = "Returns the sideband button label.";
+		_help = "Returns the current sideband.";
 	}
 	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
         {
-		*retval = xmlrpc_c::value_string(btnSideband->label());
+		*retval = xmlrpc_c::value_string(wf->USB() ? "USB" : "LSB");
 	}
 };
 
@@ -1411,98 +1544,200 @@ public:
 
 // =============================================================================
 
+extern Fl_Button* btnAutoSpot; // FIXME: export in fl_digi.h
+
+class Spot_get_auto : public xmlrpc_c::method
+{
+public:
+	Spot_get_auto()
+	{
+		_signature = "b:n";
+		_help = "Returns the autospotter state.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+	{
+		*retval = xmlrpc_c::value_boolean(btnAutoSpot->value());
+	}
+};
+
+class Spot_set_auto : public xmlrpc_c::method
+{
+public:
+	Spot_set_auto()
+	{
+		_signature = "n:b";
+		_help = "Sets the autospotter state. Returns the old state.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+	{
+		bool v = btnAutoSpot->value();
+		REQ(set_button, btnAutoSpot, params.getBoolean(0));
+		*retval = xmlrpc_c::value_boolean(v);
+	}
+};
+
+class Spot_toggle_auto : public xmlrpc_c::method
+{
+public:
+	Spot_toggle_auto()
+	{
+		_signature = "n:b";
+		_help = "Toggles the autospotter state. Returns the new state.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+	{
+		bool v = !btnAutoSpot->value();
+		REQ(set_button, btnAutoSpot, v);
+		*retval = xmlrpc_c::value_boolean(v);
+	}
+};
+
+class Spot_pskrep_get_count : public xmlrpc_c::method
+{
+public:
+	Spot_pskrep_get_count()
+	{
+		_signature = "i:n";
+		_help = "Returns the number of callsigns spotted in the current session.";
+	}
+	void execute(const xmlrpc_c::paramList& params, xmlrpc_c::value* retval)
+	{
+		*retval = xmlrpc_c::value_int(static_cast<unsigned>(pskrep_count()));
+	}
+};
+
+// =============================================================================
+
 // End XML-RPC interface
 
+// method list: ELEM_(class_name, "method_name")
+#undef ELEM_
+#define METHOD_LIST							\
+	ELEM_(Fldigi_list, "fldigi.list")				\
+	ELEM_(Fldigi_name, "fldigi.name")				\
+	ELEM_(Fldigi_version_struct, "fldigi.version_struct")		\
+	ELEM_(Fldigi_version_string, "fldigi.version")			\
+	ELEM_(Fldigi_name_version, "fldigi.name_version")		\
+	ELEM_(Fldigi_config_dir, "fldigi.config_dir")			\
+									\
+	ELEM_(Modem_get_name, "modem.get_name")				\
+	ELEM_(Modem_get_names, "modem.get_names")			\
+	ELEM_(Modem_get_id, "modem.get_id")				\
+	ELEM_(Modem_get_max_id, "modem.get_max_id")			\
+	ELEM_(Modem_set_by_name, "modem.set_by_name")			\
+	ELEM_(Modem_set_by_id, "modem.set_by_id")			\
+									\
+	ELEM_(Modem_set_carrier, "modem.set_carrier")			\
+	ELEM_(Modem_inc_carrier, "modem.inc_carrier")			\
+	ELEM_(Modem_get_carrier, "modem.get_carrier")			\
+									\
+	ELEM_(Modem_get_afc_sr, "modem.get_afc_search_range")		\
+	ELEM_(Modem_set_afc_sr, "modem.set_afc_search_range")		\
+	ELEM_(Modem_inc_afc_sr, "modem.inc_afc_search_range")		\
+									\
+	ELEM_(Modem_get_bw, "modem.get_bandwidth")			\
+	ELEM_(Modem_set_bw, "modem.set_bandwidth")			\
+	ELEM_(Modem_inc_bw, "modem.inc_bandwidth")			\
+									\
+	ELEM_(Modem_get_quality, "modem.get_quality")			\
+	ELEM_(Modem_search_up, "modem.search_up")			\
+	ELEM_(Modem_search_down, "modem.search_down")			\
+									\
+	ELEM_(Main_get_status1, "main.get_status1")			\
+	ELEM_(Main_get_status2, "main.get_status2")			\
+									\
+	ELEM_(Main_get_sb, "main.get_sideband")				\
+	ELEM_(Main_set_sb, "main.set_sideband")				\
+	ELEM_(Main_get_freq, "main.get_frequency")			\
+	ELEM_(Main_set_freq, "main.set_frequency")			\
+	ELEM_(Main_inc_freq, "main.inc_frequency")			\
+									\
+	ELEM_(Main_get_afc, "main.get_afc")				\
+	ELEM_(Main_set_afc, "main.set_afc")				\
+	ELEM_(Main_toggle_afc, "main.toggle_afc")			\
+									\
+	ELEM_(Main_get_sql, "main.get_squelch")				\
+	ELEM_(Main_set_sql, "main.set_squelch")				\
+	ELEM_(Main_toggle_sql, "main.toggle_squelch")			\
+									\
+	ELEM_(Main_get_sql_level, "main.get_squelch_level")		\
+	ELEM_(Main_set_sql_level, "main.set_squelch_level")		\
+	ELEM_(Main_inc_sql_level, "main.inc_squelch_level")		\
+									\
+	ELEM_(Main_get_rev, "main.get_reverse")				\
+	ELEM_(Main_set_rev, "main.set_reverse")				\
+	ELEM_(Main_toggle_rev, "main.toggle_reverse")			\
+									\
+	ELEM_(Main_get_lock, "main.get_lock")				\
+	ELEM_(Main_set_lock, "main.set_lock")				\
+	ELEM_(Main_toggle_lock, "main.toggle_lock")			\
+									\
+	ELEM_(Main_get_trx_status, "main.get_trx_status")		\
+	ELEM_(Main_tx, "main.tx")					\
+	ELEM_(Main_tune, "main.tune")					\
+	ELEM_(Main_rsid, "main.rsid")					\
+	ELEM_(Main_rx, "main.rx")					\
+	ELEM_(Main_abort, "main.abort")					\
+									\
+	ELEM_(Main_run_macro, "main.run_macro")				\
+	ELEM_(Main_get_max_macro_id, "main.get_max_macro_id")		\
+									\
+	ELEM_(Log_get_freq, "log.get_frequency")			\
+	ELEM_(Log_get_time_on, "log.get_time_on")				\
+	ELEM_(Log_get_time_off, "log.get_time_off")				\
+	ELEM_(Log_get_call, "log.get_call")				\
+	ELEM_(Log_get_name, "log.get_name")				\
+	ELEM_(Log_get_rst_in, "log.get_rst_in")				\
+	ELEM_(Log_get_rst_out, "log.get_rst_out")			\
+	ELEM_(Log_get_serial_number, "log.get_serial_number")		\
+	ELEM_(Log_get_state, "log.get_state")			\
+	ELEM_(Log_get_province, "log.get_province")			\
+	ELEM_(Log_get_country, "log.get_country")			\
+	ELEM_(Log_get_qth, "log.get_qth")				\
+	ELEM_(Log_get_band, "log.get_band")				\
+	ELEM_(Log_get_sb, "log.get_sideband")				\
+	ELEM_(Log_get_notes, "log.get_notes")				\
+	ELEM_(Log_get_locator, "log.get_locator")			\
+	ELEM_(Log_get_az, "log.get_az")					\
+	ELEM_(Log_clear, "log.clear")					\
+	ELEM_(Log_set_call, "log.set_call")				\
+									\
+	ELEM_(Text_get_rx_length, "text.get_rx_length")			\
+	ELEM_(Text_get_rx, "text.get_rx")				\
+	ELEM_(Text_clear_rx, "text.clear_rx")				\
+	ELEM_(Text_add_tx, "text.add_tx")				\
+	ELEM_(Text_add_tx_bytes, "text.add_tx_bytes")			\
+	ELEM_(Text_clear_tx, "text.clear_tx")				\
+									\
+	ELEM_(Spot_get_auto, "spot.get_auto")				\
+	ELEM_(Spot_set_auto, "spot.set_auto")				\
+	ELEM_(Spot_toggle_auto, "spot.toggle_auto")			\
+	ELEM_(Spot_pskrep_get_count, "spot.pskrep.get_count")
+
+
+struct rm_pred
+{
+	re_t filter;
+	bool allow;
+	rm_pred(const char* re, bool allow_)
+		: filter(re, REG_EXTENDED | REG_NOSUB), allow(allow_) { }
+	bool operator()(const methods_t::value_type& v)
+	{
+		return filter.match(v.name) ^ allow && !strstr(v.name, "fldigi.");
+	}
+};
+
 void XML_RPC_Server::add_methods(void)
 {
-	methods->clear();
-	methods->reserve(72);
+	if (methods)
+		return;
+#undef ELEM_
+#define ELEM_(class_, name_) rpc_method(new class_, name_),
+	rpc_method m[] = { METHOD_LIST };
+	methods = new methods_t(m, m + sizeof(m)/sizeof(*m));
 
-	methods->push_back(rpc_method(new Fldigi_list, "fldigi.list"));
-	methods->push_back(rpc_method(new Fldigi_name, "fldigi.name"));
-	methods->push_back(rpc_method(new Fldigi_version_struct, "fldigi.version_struct"));
-	methods->push_back(rpc_method(new Fldigi_version_string, "fldigi.version"));
-	methods->push_back(rpc_method(new Fldigi_name_version, "fldigi.name_version"));
-
-	methods->push_back(rpc_method(new Modem_get_name, "modem.get_name"));
-	methods->push_back(rpc_method(new Modem_get_names, "modem.get_names"));
-	methods->push_back(rpc_method(new Modem_get_id, "modem.get_id"));
-	methods->push_back(rpc_method(new Modem_get_max_id, "modem.get_max_id"));
-	methods->push_back(rpc_method(new Modem_set_by_name, "modem.set_by_name"));
-	methods->push_back(rpc_method(new Modem_set_by_id, "modem.set_by_id"));
-
-	methods->push_back(rpc_method(new Modem_set_carrier, "modem.set_carrier"));
-	methods->push_back(rpc_method(new Modem_inc_carrier, "modem.inc_carrier"));
-	methods->push_back(rpc_method(new Modem_get_carrier, "modem.get_carrier"));
-
-	methods->push_back(rpc_method(new Modem_get_afc_sr, "modem.get_afc_search_range"));
-	methods->push_back(rpc_method(new Modem_set_afc_sr, "modem.set_afc_search_range"));
-	methods->push_back(rpc_method(new Modem_inc_afc_sr, "modem.inc_afc_search_range"));
-
-	methods->push_back(rpc_method(new Modem_get_bw, "modem.get_bandwidth"));
-	methods->push_back(rpc_method(new Modem_set_bw, "modem.set_bandwidth"));
-	methods->push_back(rpc_method(new Modem_inc_bw, "modem.inc_bandwidth"));
-
-	methods->push_back(rpc_method(new Modem_get_quality, "modem.get_quality"));
-	methods->push_back(rpc_method(new Modem_search_up, "modem.search_up"));
-	methods->push_back(rpc_method(new Modem_search_down, "modem.search_down"));
-
-	methods->push_back(rpc_method(new Main_get_status1, "main.get_status1"));
-	methods->push_back(rpc_method(new Main_get_status2, "main.get_status2"));
-
-	methods->push_back(rpc_method(new Main_get_sb, "main.get_sideband"));
-	methods->push_back(rpc_method(new Main_get_freq, "main.get_frequency"));
-	methods->push_back(rpc_method(new Main_set_freq, "main.set_frequency"));
-	methods->push_back(rpc_method(new Main_inc_freq, "main.inc_frequency"));
-
-	methods->push_back(rpc_method(new Main_get_afc, "main.get_afc"));
-	methods->push_back(rpc_method(new Main_set_afc, "main.set_afc"));
-	methods->push_back(rpc_method(new Main_toggle_afc, "main.toggle_afc"));
-
-	methods->push_back(rpc_method(new Main_get_sql, "main.get_squelch"));
-	methods->push_back(rpc_method(new Main_set_sql, "main.set_squelch"));
-	methods->push_back(rpc_method(new Main_toggle_sql, "main.toggle_squelch"));
-
-	methods->push_back(rpc_method(new Main_get_sql_level, "main.get_squelch_level"));
-	methods->push_back(rpc_method(new Main_set_sql_level, "main.set_squelch_level"));
-	methods->push_back(rpc_method(new Main_inc_sql_level, "main.inc_squelch_level"));
-
-	methods->push_back(rpc_method(new Main_get_rev, "main.get_reverse"));
-	methods->push_back(rpc_method(new Main_set_rev, "main.set_reverse"));
-	methods->push_back(rpc_method(new Main_toggle_rev, "main.toggle_reverse"));
-
-	methods->push_back(rpc_method(new Main_get_lock, "main.get_lock"));
-	methods->push_back(rpc_method(new Main_set_lock, "main.set_lock"));
-	methods->push_back(rpc_method(new Main_toggle_lock, "main.toggle_lock"));
-
-	methods->push_back(rpc_method(new Main_get_trx_status, "main.get_trx_status"));
-	methods->push_back(rpc_method(new Main_tx, "main.tx"));
-	methods->push_back(rpc_method(new Main_tune, "main.tune"));
-	methods->push_back(rpc_method(new Main_rsid, "main.rsid"));
-	methods->push_back(rpc_method(new Main_rx, "main.rx"));
-	methods->push_back(rpc_method(new Main_abort, "main.abort"));
-
-	methods->push_back(rpc_method(new Main_run_macro, "main.run_macro"));
-	methods->push_back(rpc_method(new Main_get_max_macro_id, "main.get_max_macro_id"));
-
-	methods->push_back(rpc_method(new Log_get_freq, "log.get_frequency"));
-	methods->push_back(rpc_method(new Log_get_time, "log.get_time"));
-	methods->push_back(rpc_method(new Log_get_call, "log.get_call"));
-	methods->push_back(rpc_method(new Log_get_name, "log.get_name"));
-	methods->push_back(rpc_method(new Log_get_rst_in, "log.get_rst_in"));
-	methods->push_back(rpc_method(new Log_get_rst_out, "log.get_rst_out"));
-	methods->push_back(rpc_method(new Log_get_qth, "log.get_qth"));
-	methods->push_back(rpc_method(new Log_get_band, "log.get_band"));
-	methods->push_back(rpc_method(new Log_get_sb, "log.get_sideband"));
-	methods->push_back(rpc_method(new Log_get_notes, "log.get_notes"));
-	methods->push_back(rpc_method(new Log_get_locator, "log.get_locator"));
-	methods->push_back(rpc_method(new Log_get_az, "log.get_az"));
-	methods->push_back(rpc_method(new Log_clear, "log.clear"));
-
-	methods->push_back(rpc_method(new Text_get_rx_length, "text.get_rx_length"));
-	methods->push_back(rpc_method(new Text_get_rx, "text.get_rx"));
-	methods->push_back(rpc_method(new Text_clear_rx, "text.clear_rx"));
-	methods->push_back(rpc_method(new Text_add_tx, "text.add_tx"));
-	methods->push_back(rpc_method(new Text_add_tx_bytes, "text.add_tx_bytes"));
-	methods->push_back(rpc_method(new Text_clear_tx, "text.clear_tx"));
+	if (!progdefaults.xmlrpc_deny.empty())
+		methods->remove_if(rm_pred(progdefaults.xmlrpc_deny.c_str(), false));
+	else if (!progdefaults.xmlrpc_allow.empty())
+		methods->remove_if(rm_pred(progdefaults.xmlrpc_allow.c_str(), true));
 }
