@@ -26,6 +26,8 @@
 #include <config.h>
 
 #include <string>
+#include <iostream>
+
 #include <FL/Fl.H>
 #include <FL/fl_ask.H>
 
@@ -51,7 +53,6 @@ void mt63::rx_init()
 	set_freq(500.0 + bandwidth / 2.0);
 	InpLevel->Preset(64.0, 0.75);
 	escape = 0;
-	set_AFCind(0.0);
 }
 
 double peak = 0.0;
@@ -61,6 +62,8 @@ int mt63::tx_process()
 	int c;
 	double maxval = 0;
 
+	rx_flush();
+	
 	c = get_tx_char();
 	if (c == 0x03)  {
 		stopflag = true;
@@ -116,6 +119,11 @@ int mt63::rx_process(const double *buf, int len)
 	double snr;
 	unsigned int c;
 	int i;
+	static char msg1[20];
+	static char msg2[20];
+
+	if (get_freq() != (500.0 + bandwidth / 2.0))
+	    set_freq(500.0 + bandwidth / 2.0);
 
 	if (Interleave != progdefaults.mt63_interleave) {
 		Interleave = progdefaults.mt63_interleave;
@@ -135,18 +143,25 @@ int mt63::rx_process(const double *buf, int len)
 	Rx->Process(InpBuff);
 
 	snr = Rx->FEC_SNR();
+	
+	if (progStatus.sqlonoff && snr < progStatus.sldrSquelchValue) {
+	    put_Status1("");
+	    put_Status2("");
+	    display_metric(0);
+		return 0;
+    }
+
 	if (snr > 99.9)
 		snr = 99.9;
 	display_metric(snr);
 
-//	static char msg1[15];
-//	double s2n = 10.0*log10( snr );
-//	snprintf(msg1, sizeof(msg1), "s/n %2d dB", (int)(floor(s2n))); 
-//  put_Status1(msg1);
-
-	if (progStatus.sqlonoff && snr < progStatus.sldrSquelchValue)
-		return 0;
+	double s2n = 10.0*log10( snr == 0 ? 0.001 : snr);
+	snprintf(msg1, sizeof(msg1), "s/n %2d dB", (int)(floor(s2n))); 
+    put_Status1(msg1);
 
+    snprintf(msg2, sizeof(msg2), "f/o %+4.1f Hz", Rx->TotalFreqOffset());
+    put_Status2(msg2, 5, STATUS_CLEAR);
+	
 	for (i = 0; i < Rx->Output.Len; i++) {
 		c = Rx->Output.Data[i];
 
@@ -170,10 +185,62 @@ int mt63::rx_process(const double *buf, int len)
 
 		put_rx_char(c);
 	}
-
+	flushbuffer = true;
+		
 	return 0;
 }
 
+void mt63::rx_flush()
+{
+	unsigned int c;
+	int len = 512;
+	int dlen = 0;
+
+	if (!flushbuffer) return;
+	
+	if (emptyBuff->EnsureSpace(len) == -1) {
+		flushbuffer = false;
+		return;
+	}
+
+	for (int j = 0; j < len; j++)
+		emptyBuff->Data[j] = 0.0;
+	emptyBuff->Len = len;
+	InpLevel->Process(emptyBuff);
+	Rx->Process(emptyBuff);
+	dlen = Rx->Output.Len;
+
+	while (Rx->SYNC_LockStatus()) {
+		for (int i = 0; i < dlen; i++) {
+			c = Rx->Output.Data[i];
+			if (!progdefaults.mt63_8bit) {
+				put_rx_char(c);
+				continue;
+			}
+			if ((c < 8) && (escape == 0))
+				continue;
+			if (c == 127) {
+				escape = 1;
+				continue;
+			}
+			if (escape) {
+				c += 128;
+				escape = 0;
+			}
+			put_rx_char(c);
+		}
+		for (int j = 0; j < len; j++)
+			emptyBuff->Data[j] = 0.0;
+		emptyBuff->Len = len;
+		InpLevel->Process(emptyBuff);
+		Rx->Process(emptyBuff);
+		dlen = Rx->Output.Len;
+	}
+	flushbuffer = false;
+	
+	return;
+}
+
 void mt63::restart()
 {
 	int err;
@@ -187,6 +254,10 @@ void mt63::restart()
 		fprintf(stderr, "mt63_txinit: init failed\n");
 	flush = Tx->DataInterleave;
 
+// Rx->Preset( int BandWidth = 1000,
+//               int LongInterleave = 0,
+//               int Integ = 16,
+//  	         void (*Display)(double *Spectra, int Len) = NULL);
 	err = Rx->Preset((int)bandwidth, Interleave == 64 ? 1 : 0, IntegLen);
 	if (err)
 		fprintf(stderr, "mt63_rxinit: init failed\n");
@@ -197,6 +268,7 @@ void mt63::init()
 {
 	modem::init();
 	restart();
+	flushbuffer = false;
 }
 
 mt63::mt63 (trx_mode mt63_mode) : modem()
@@ -220,6 +292,7 @@ mt63::mt63 (trx_mode mt63_mode) : modem()
 
 	InpLevel = new dspLevelMonitor;
 	InpBuff = new double_buff;
+	emptyBuff = new double_buff;
 
 	samplerate = 8000;
 	fragmentsize = 1024;
