@@ -27,16 +27,16 @@
 
 #include <config.h>
 
-#include <string>
 #include <FL/Fl.H>
 #include <FL/fl_ask.H>
 
 #include "olivia.h"
 #include "sound.h"
-#include "status.h"
 
 #include "confdialog.h"
 
+#include "status.h"
+
 using namespace std;
 
 double olivia::nco(double freq)
@@ -86,8 +86,13 @@ void olivia::tx_init(SoundBase *sc)
 void olivia::send_preamble()
 {
 	double freqa, freqb;
-    int i, sr4 = samplerate / 4;
-
+    int i, j, sr4 = samplerate / 4;
+    double ampshape[sr4];
+    
+    for (int i = 0; i < sr4; i++) ampshape[i] = 1.0;
+    for (int i = 0; i < sr4 / 4; i++)
+        ampshape[i] = ampshape[sr4 - 1 - i] = 0.5 * (1.0 - cos(M_PI * i / (sr4/4)));
+        
 	if (reverse) { 
 		freqa = txbasefreq + (bandwidth / 2.0); 
 		freqb = txbasefreq - (bandwidth / 2.0); 
@@ -96,22 +101,27 @@ void olivia::send_preamble()
 		freqb = txbasefreq + (bandwidth / 2.0); 
 	}
 
-	for (i = 0; i < sr4; i++)
-		outbuf[i] = nco(freqa);
-	for (i = sr4; i < 2*sr4; i++)
-		outbuf[i] = nco(freqb);
-	for (i = 2*sr4; i < 3*sr4; i++)
-		outbuf[i] = nco(freqa);
-	for (i = 3*sr4; i < samplerate; i++)
-		outbuf[i] = nco(freqb);
+	for (i = 0, j = 0; i < sr4; i++, j++)
+		outbuf[i] = nco(freqa) * ampshape[j];
+	for (i = sr4, j = 0; i < 2*sr4; i++, j++)
+		outbuf[i] = nco(freqb) * ampshape[j];
+	for (i = 2*sr4, j = 0; i < 3*sr4; i++, j++)
+		outbuf[i] = nco(freqa) * ampshape[j];
+	for (i = 3*sr4, j = 0; i < samplerate; i++, j++)
+		outbuf[i] = nco(freqb) * ampshape[j];
 	ModulateXmtr(outbuf, samplerate);
 }
 
 void olivia::send_postamble()
 {
 	double freqa, freqb;
-	int i, sr4 = samplerate / 4;
-
+	int i, j, sr4 = samplerate / 4;
+    double ampshape[sr4];
+    
+    for (int i = 0; i < sr4; i++) ampshape[i] = 1.0;
+    for (int i = 0; i < sr4 / 8; i++)
+        ampshape[i] = ampshape[sr4 - 1 - i] = 0.5 * (1.0 - cos(M_PI * i / (sr4/8)));
+        
 	if (reverse) { 
 		freqa = txbasefreq + (bandwidth / 2.0); 
 		freqb = txbasefreq - (bandwidth / 2.0); 
@@ -120,14 +130,14 @@ void olivia::send_postamble()
 		freqb = txbasefreq + (bandwidth / 2.0); 
 	}
 
-	for (i = 0; i < sr4; i++)
-		outbuf[i] = nco(freqa);
-	for (i = sr4; i < 2*sr4; i++)
-		outbuf[i] = nco(freqb);
-	for (i = 2*sr4; i < 3*sr4; i++)
-		outbuf[i] = nco(freqa);
-	for (i = 3*sr4; i < samplerate; i++)
-		outbuf[i] = nco(freqb);
+	for (i = 0, j = 0; i < sr4; i++, j++)
+		outbuf[i] = nco(freqa) * ampshape[j];
+	for (i = sr4, j = 0; i < 2*sr4; i++, j++)
+		outbuf[i] = nco(freqb) * ampshape[j];
+	for (i = 2*sr4, j = 0; i < 3*sr4; i++, j++)
+		outbuf[i] = nco(freqa) * ampshape[j];
+	for (i = 3*sr4, j = 0; i < samplerate; i++, j++)
+		outbuf[i] = nco(freqb) * ampshape[j];
 	ModulateXmtr(outbuf, samplerate);
 }
 
@@ -135,7 +145,6 @@ void olivia::rx_init()
 {
 	Rx->Reset();
 	escape = 0;
-	set_AFCind(0.0);
 }
 
 int olivia::unescape(int c)
@@ -158,7 +167,7 @@ int olivia::unescape(int c)
 
 int olivia::tx_process()
 {
-	int c, i, len;
+	int c, len;//, i;
 	unsigned char ch;
 
 	if (tones	!= progdefaults.oliviatones ||
@@ -184,7 +193,7 @@ int olivia::tx_process()
 		} else {
 			/* Replace un-representable characters with a dot */
 			if (c == -1)
-                                c = 0;
+                c = 0;
 			if (c > (progdefaults.olivia8bit ? 255 : 127))
 				c = '.';
 			if (c > 127) {
@@ -199,12 +208,8 @@ int olivia::tx_process()
 		if ((c = unescape(ch)) != -1)
 			put_echo_char(c);
 
-	if ((len = Tx->Output(txbuffer)) > 0) {
-		for (i = 0; i < len; i++)
-			txfbuffer[i] = (double) (txbuffer[i] / 32767.0);
-
+    if ((len = Tx->Output(txfbuffer)) > 0)
 		ModulateXmtr(txfbuffer, len);
-	}
 
 	if (stopflag && Tx->DoPostambleYet() == 1 && postamblesent != 1) {
 		postamblesent = 1; 
@@ -216,18 +221,18 @@ int olivia::tx_process()
 		cwid();
 		return -1;
 	}
-
+ 
 	return 0;
 }
 
+
 int olivia::rx_process(const double *buf, int len)
 {
-	int i, c;
+	int c;
 	unsigned char ch = 0;
 	double snr;
 	static char msg1[20];
 	static char msg2[20];
-//	static char msg3[60];
 
 	if (tones	!= progdefaults.oliviatones ||
 		bw 		!= progdefaults.oliviabw ||
@@ -248,35 +253,37 @@ int olivia::rx_process(const double *buf, int len)
 		Rx->Preset();
 	}
 
-	if (len > rxbufferlen) {
-		delete [] rxbuffer;
-		rxbuffer = new short int[len];
-		rxbufferlen = len;
-	}
-
-	for (i = 0; i < len; i++)
-		rxbuffer[i] = (short int) (buf[i] * 32767.0);
-
 	Rx->SyncThreshold = progStatus.sqlonoff ? progStatus.sldrSquelchValue / 2.0 : 0.0;
 
-	Rx->Process(rxbuffer, len);
+    Rx->Process(buf, len);
 
-	snr = Rx->SignalToNoiseRatio();
-
-	set_metric(snr);
-	display_metric(snr > 100.0 ? 100.0 : snr * 2.0);
 	
-	double s2n = 20.0 * log10(snr < 0.1 ? 0.1 : snr);
+	bool gotchar = false;
+	while (Rx->GetChar(ch) > 0) {
+		if ((c = unescape(ch)) != -1 && c > 7) {
+    		put_rx_char(c);
+            gotchar = true;
+        }
+    }
+    if (gotchar) {
+        double noisepwr1 = wf->powerDensity((1.0 * frequency - Rx->Bandwidth / 2.0 - 100.0), 50.0),
+               noisepwr2 = wf->powerDensity((1.0 * frequency + Rx->Bandwidth / 2.0 + 100.0), 50.0),
+               noisepwr = 0,
+               sigpwr   = wf->powerDensity(1.0 * frequency, 1.0 * Rx->Bandwidth);
+        noisepwr = noisepwr1 < noisepwr ? noisepwr1 : noisepwr2;
+        if (noisepwr == 0) noisepwr = 1.0e-10;
+    	metric = decayavg( metric, sigpwr / noisepwr, 8);
+    	
+	    snprintf(msg1, sizeof(msg1), "s/n %4.1f dB", 10.0 * log10(metric));
+	    put_Status1(msg1, 5, STATUS_CLEAR);
 
-	snprintf(msg1, sizeof(msg1), "s/n %4.1f dB", s2n);
-	put_Status1(msg1);
-	snprintf(msg2, sizeof(msg2), "Freq: %+4.1f", Rx->FrequencyOffset());
-	put_Status2(msg2);
-
-	while (Rx->GetChar(ch) > 0)
-		if ((c = unescape(ch)) != -1 && c > 7)
-			put_rx_char(c);
+	    snprintf(msg2, sizeof(msg2), "f/o %+4.1f Hz", Rx->FrequencyOffset());
+        put_Status2(msg2, 5, STATUS_CLEAR);
 
+    	snr = 3 * Rx->SignalToNoiseRatio();
+	    display_metric(snr > 100.0 ? 100.0 : snr );
+    }
+    
 	return 0;
 }
 
@@ -309,15 +316,10 @@ void olivia::restart()
 	}
 		
 	txbufferlen = Tx->MaxOutputLen;
-	if (txbuffer) delete [] txbuffer;
-	txbuffer = new short int[txbufferlen];
 	
 	if (txfbuffer) delete [] txfbuffer;
 	txfbuffer = new double[txbufferlen];
 
-	rxbufferlen = 0;
-	rxbuffer = 0;
-	
 	Rx->Tones = Tx->Tones;
 	Rx->Bandwidth = Tx->Bandwidth;
 	Rx->SyncMargin = smargin;
@@ -340,9 +342,10 @@ void olivia::restart()
 		return;
 	}
 	fragmentsize = 1024;
-	set_bandwidth(Tx->Bandwidth);
+	set_bandwidth(Tx->Bandwidth - Tx->Bandwidth / Tx->Tones);
 
-	put_MODEstatus(mode);
+	put_MODEstatus("%s %zu/%zu", get_mode_name(), Tx->Tones, Tx->Bandwidth);
+	metric = 0;
 }
 
 void olivia::init()
@@ -356,15 +359,11 @@ olivia::olivia()
 {
 	cap = CAP_REV;
 
-	txbuffer = 0;
 	txfbuffer = 0;
-	rxbuffer = 0;
 	samplerate = 8000;
 
-	FL_LOCK();
-		Tx = new MFSK_Transmitter< float >;
-		Rx = new MFSK_Receiver< float >;
-	FL_UNLOCK();
+	Tx = new MFSK_Transmitter< double >;
+	Rx = new MFSK_Receiver< double >;
 
 	mode = MODE_OLIVIA;
 	lastfreq = 0;
@@ -375,8 +374,6 @@ olivia::~olivia()
 {
 	if (Tx) delete Tx;
 	if (Rx) delete Rx;
-	if (txbuffer) delete [] txbuffer;
 	if (txfbuffer) delete [] txfbuffer;
-	if (rxbuffer) delete [] rxbuffer;
 }
 
