@@ -60,7 +60,6 @@ extern waterfall *wf;
 
 char pskmsg[80];
 viewpsk *pskviewer = (viewpsk *)0;
-pskeval *evalpsk = (pskeval *)0;
 
 void psk::tx_init(SoundBase *sc)
 {
@@ -86,15 +85,12 @@ void psk::rx_init()
 	put_MODEstatus(mode);
 	resetSN_IMD();
 	imdValid = false;
-	set_AFCrange(1.0);
 	afcmetric = 0.0;
 }
 
 void psk::restart()
 {
-	if (!pskviewer) pskviewer = new viewpsk(mode);
-	else		    pskviewer->restart(mode);
-	if (!evalpsk) evalpsk = new pskeval;
+	pskviewer->restart(mode);
 	evalpsk->setbw(bandwidth);
 }
 
@@ -118,11 +114,15 @@ psk::~psk()
 	if (fir2) delete fir2;
 	if (snfilt) delete snfilt;
 	if (imdfilt) delete imdfilt;
+	if (::pskviewer == pskviewer)
+		::pskviewer = 0;
+	delete pskviewer;
+	delete evalpsk;
 }
 
 psk::psk(trx_mode pskmode) : modem()
 {
-	cap = CAP_AFC;
+	cap = CAP_AFC | CAP_AFC_SR;
 
 	mode = pskmode;
 
@@ -177,7 +177,7 @@ psk::psk(trx_mode pskmode) : modem()
 		_qpsk = false;
 		dcdbits = 32;
 	}
-	fir1 = fir2 = (C_FIR_filter *)0;
+
 	enc = (encoder *)0;
 	dec = (viterbi *)0;
 	
@@ -185,29 +185,49 @@ psk::psk(trx_mode pskmode) : modem()
 	double fir1c[64];
 	double fir2c[64];
 
-// use the original gmfsk matched filters
-//	for (int i = 0; i < 64; i++) {
-//		fir1c[i] = gmfir1c[i];
-//		fir2c[i] = gmfir2c[i];
-//	}
-// or matched sync filters
-
-	wsincfilt(fir1c, 1.0 / symbollen, true);	// creates fir1c matched sin(x)/x filter w blackman
-	wsincfilt(fir2c, 1.0 / 16.0, true);			// creates fir2c matched sin(x)/x filter w blackman
-
-//	wsincfilt(fir1c, 1.0 / symbollen, false);	// creates fir1c matched sin(x)/x filter w hamming
-//	wsincfilt(fir2c, 1.0 / 16.0, false);		// creates fir2c matched sin(x)/x filter w hamming
-//	wsincfilt(fir2c, 1.0 / 22.0, false);    	// 1/22 with Hamming window
-                                            	// nearly identical to gmfir2c
-// experimental raised cosine filter
-//	raisedcosfilt(fir1c);	// creates fir1c
-
 	fir1 = new C_FIR_filter();
-	fir1->init(FIRLEN, symbollen / 16, fir1c, fir1c);
-
 	fir2 = new C_FIR_filter();
-	fir2->init(FIRLEN, 1, fir2c, fir2c);
-	
+
+    switch (progdefaults.PSK_filter) {
+        case 1:
+// use the original gmfsk matched filters
+        	for (int i = 0; i < 64; i++) {
+		        fir1c[i] = gmfir1c[i];
+		        fir2c[i] = gmfir2c[i];
+	        }
+        	fir1->init(FIRLEN, symbollen / 16, fir1c, fir1c);
+	        fir2->init(FIRLEN, 1, fir2c, fir2c);
+            break;
+        case 2:
+// creates fir1c matched sin(x)/x filter w hamming
+	        wsincfilt(fir1c, 1.0 / symbollen, false);
+        	fir1->init(FIRLEN, symbollen / 16, fir1c, fir1c);
+// creates fir2c matched sin(x)/x filter w hamming
+	        wsincfilt(fir2c, 1.0 / 16.0, false);
+	        fir2->init(FIRLEN, 1, fir2c, fir2c);
+            break;
+        case 3:
+// creates fir1c matched sin(x)/x filter w hamming
+	        wsincfilt(fir1c, 1.0 / symbollen, false);
+        	fir1->init(FIRLEN, symbollen / 16, fir1c, fir1c);
+// 1/22 with Hamming window nearly identical to gmfir2c
+	        wsincfilt(fir2c, 1.0 / 22.0, false);
+	        fir2->init(FIRLEN, 1, fir2c, fir2c);
+            break;
+        case 4:
+            fir1->init_lowpass (FIRLEN, 16, 1.5 / symbollen);
+        	wsincfilt(fir2c, 1.5 / 16.0, true);
+            fir2->init(FIRLEN, 1, fir2c, fir2c);
+        case 0:
+        default :
+// creates fir1c matched sin(x)/x filter w blackman
+        	wsincfilt(fir1c, 1.0 / symbollen, true);
+        	fir1->init(FIRLEN, symbollen / 16, fir1c, fir1c);
+// creates fir2c matched sin(x)/x filter w blackman
+        	wsincfilt(fir2c, 1.0 / 16.0, true);
+	        fir2->init(FIRLEN, 1, fir2c, fir2c);
+    }
+    
 	snfilt = new Cmovavg(16);
 	imdfilt = new Cmovavg(16);
 
@@ -235,7 +255,10 @@ psk::psk(trx_mode pskmode) : modem()
 		syncbuf[i] = 0.0;
 	E1 = E2 = E3 = 0.0;
 	acquire = 0;
-	
+
+	evalpsk = new pskeval;
+	::pskviewer = pskviewer = new viewpsk(evalpsk, mode);
+
 	init();
 }
 
@@ -379,8 +402,7 @@ void psk::phaseafc()
 		freqerr = decayavg( freqerr, error, AFCDECAYSLOW);
 		frequency -= freqerr;
 		set_freq (frequency);
-		set_AFCind(freqerr);
-	} else set_AFCind(0.0);
+	}
 	if (acquire) acquire--;
 }
 
@@ -480,8 +502,8 @@ void psk::update_syncscope()
 	snprintf(msg2, sizeof(msg2), "imd %3d dB", (int)(floor(imd))); 
 
 	if (imdValid) {
-		put_Status1(msg1, 5.0, STATUS_DIM);
-		put_Status2(msg2, 5.0, STATUS_DIM);
+		put_Status1(msg1, progdefaults.StatusTimeout, progdefaults.StatusDim ? STATUS_DIM : STATUS_CLEAR);
+		put_Status2(msg2, progdefaults.StatusTimeout, progdefaults.StatusDim ? STATUS_DIM : STATUS_CLEAR);
 	}
 }
 
@@ -596,10 +618,7 @@ void psk::tx_symbol(int sym)
 	}
 	symbol = prevsymbol * symbol;	// complex multiplication
 
-	if (mailserver && progdefaults.PSKmailSweetSpot)
-		delta = 2.0 * M_PI * (progdefaults.PSKsweetspot  - progdefaults.TxOffset) / samplerate;
-	else
-		delta = 2.0 * M_PI * get_txfreq_woffset() / samplerate;
+	delta = 2.0 * M_PI * get_txfreq_woffset() / samplerate;
 
 	for (int i = 0; i < symbollen; i++) {
 		
