@@ -8,9 +8,6 @@
 #include <config.h>
 
 #include <ctime>
-#include <sys/ipc.h>
-#include <sys/msg.h>
-#include <sys/shm.h>
 #include <sys/time.h>
 #include <iostream>
 #include <list>
@@ -33,13 +30,15 @@
 #include "debug.h"
 #include "threads.h"
 #include "qrunner.h"
+#include "confdialog.h"
+
+LOG_SET_SOURCE(debug::LOG_RIGCONTROL);
 
 using namespace std;
 
-
 Cserial rigio;
-static Fl_Mutex		rigCAT_mutex = PTHREAD_MUTEX_INITIALIZER;
-static Fl_Thread	rigCAT_thread;
+static pthread_mutex_t	rigCAT_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_t		*rigCAT_thread = 0;
 
 static bool				rigCAT_exit = false;
 static bool				rigCAT_open = false;
@@ -51,111 +50,54 @@ static string 			sRigWidth = "";
 static string 			sRigMode = "";
 static long long		llFreq = 0;
 
-static int dummy = 0;
 static bool nonCATrig = false;
-long long  noCATfreq = 0L;
+static bool noXMLfile = false;
+long long  noCATfreq = 3580000L;
 string noCATmode = "USB";
 string noCATwidth = "";
 
 static void *rigCAT_loop(void *args);
 
-static const char hexsym[] = "0123456789ABCDEF";
-
-const string& printhex(const unsigned char* s, size_t len)
-{
-	static string hex;
-	if (unlikely(len == 0))
-		return hex.erase();
-
-	hex.resize(len * 3 - 1);
-	string::iterator i = hex.begin();
-	size_t j;
-	for (j = 0; j < len-1; j++) {
-		*i++ = hexsym[s[j] >> 4];
-		*i++ = hexsym[s[j] & 0xF];
-	        *i++ = ' ';
-	}
-        *i++ = hexsym[s[j] >> 4];
-	*i = hexsym[s[j] & 0xF];
-
-	return hex;
-}
-
-const string& printhex(const string& s)
-{
-	return printhex((const unsigned char*)s.data(), s.length());
-}
-
-bool readpending = false;
-int  readtimeout;
+int  readtimeout, readwait;
 
 bool hexout(const string& s, int retnbr)
 {
-// thread might call this function while a read from the rig is in process
-// wait here until that processing is finished or a timeout occurs
-// reset the readpending & return false if a timeout occurs
-
-	LOG_DEBUG("cmd = %s", printhex(s).c_str());
-
-	readtimeout = (rig.wait +rig.timeout) * rig.retries + 2000; // 2 second min timeout
-	while (readpending && readtimeout--)
-		MilliSleep(1);
-	if (readtimeout == 0) {
-		readpending = false;
-		LOG_ERROR("rigio timeout!");
-		return false;
-	}
-
-	readpending = true;
+	LOG_DEBUG("cmd = %s", str2hex(s.data(), s.length()));
 	
-	for (int n = 0; n < rig.retries; n++) {	
+	for (int n = 0; n < progdefaults.RigCatRetries; n++) {	
 		int num = 0;
 		memset(sendbuff,0, 200);
 		for (unsigned int i = 0; i < s.length(); i++)
 			sendbuff[i] = s[i];
-
+// write the command string
 		rigio.FlushBuffer();
 		rigio.WriteBuffer(sendbuff, s.size());
-		if (rig.echo == true) {
-//#ifdef __CYGWIN__
+		if (progdefaults.RigCatECHO == true) {
 			MilliSleep(10);
-//#endif
 			num = rigio.ReadBuffer (replybuff, s.size());
-			LOG_DEBUG("echoed = %s", printhex(replybuff, num).c_str());
+			LOG_DEBUG("echoed = %s", str2hex(replybuff, num));
 		}
-
-		memset (replybuff, 0, 200);
-	
-// wait interval before trying to read response
-		if ((readtimeout = rig.wait) > 0)
-			while (readtimeout--)
+// wait before trying to read response
+		if ((readwait = progdefaults.RigCatWait) > 0)
+			while (readwait--)
 				MilliSleep(1);
-
-		LOG_DEBUG("waiting for %d bytes", retnbr);
-
+// read the rig response if one is expected
+		memset (replybuff, 0, 200);
 		if (retnbr > 0) {
 			num = rigio.ReadBuffer (replybuff, retnbr > 200 ? 200 : retnbr);
-
-// debug code
 			if (num)
-				LOG_DEBUG("resp (%d) = %s", n, printhex(replybuff, num).c_str());
+				LOG_DEBUG("read %d bytes: %s", num, str2hex(replybuff, num));
 			else
-				LOG_ERROR("resp (%d) no reply", n);
-		}
-
-		if (retnbr == 0 || num == retnbr) {
-			readpending = false;
-			return true;
-//
-		if ((readtimeout = rig.timeout) > 0)
+				LOG_DEBUG("no reply ... retry # %d", n);
+            if (num == retnbr)
+                return true; // response OK
+		} else
+		    return true; // no response needed
+		if ((readtimeout = progdefaults.RigCatTimeout) > 0)
 			while (readtimeout--)
 				MilliSleep(1);
-
-		}
 	}
-
-	readpending = false;
-	return false;
+	return false; // timed out
 }
 
 string to_bcd_be(long long freq, int len)
@@ -356,7 +298,7 @@ long long rigCAT_getfreq()
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 
-	if (nonCATrig == true) {
+	if (nonCATrig == true || noXMLfile) {
 		return noCATfreq;
 	}
 	
@@ -371,7 +313,6 @@ long long rigCAT_getfreq()
 	
 	if (itrCmd == commands.end()) {
 		LOG_DEBUG("Cmd not defined");
-		nonCATrig = true;
 		return -2; // get_freq command is not defined!
 	}
 
@@ -441,7 +382,12 @@ void rigCAT_setfreq(long long f)
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 
-	LOG_DEBUG("set frequency %ld", f);
+	if (noXMLfile || nonCATrig) {
+		noCATfreq = f;
+		return;
+	}
+	
+	LOG_DEBUG("set frequency %lld", f);
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -470,17 +416,17 @@ void rigCAT_setfreq(long long f)
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				fl_lock(&rigCAT_mutex);
+				pthread_mutex_lock(&rigCAT_mutex);
 					hexout(strCmd, rTemp.size);
-				fl_unlock(&rigCAT_mutex);
+				pthread_mutex_unlock(&rigCAT_mutex);
 				return;
 			}
 			preply++;
 		}
 	} else {
-		fl_lock(&rigCAT_mutex);
+		pthread_mutex_lock(&rigCAT_mutex);
 			hexout(strCmd, 0);
-		fl_unlock(&rigCAT_mutex);
+		pthread_mutex_unlock(&rigCAT_mutex);
 	}
 }
 
@@ -490,7 +436,7 @@ string rigCAT_getmode()
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 	
-	if (nonCATrig == true) 
+	if (nonCATrig == true || noXMLfile) 
 		return noCATmode;
 
 	LOG_DEBUG("get mode");
@@ -511,10 +457,6 @@ string rigCAT_getmode()
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
 
-//	if (hexout(strCmd) == false) {
-//		return "";
-//	}
-
 	if (modeCmd.info.size()) {
 		list<XMLIOS>::iterator preply = reply.begin();
 		while (preply != reply.end()) {
@@ -589,7 +531,12 @@ void rigCAT_setmode(const string& md)
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 	
-	LOG_DEBUG("set mode");
+	if (nonCATrig == true || noXMLfile) {
+		noCATmode = md;
+		return;
+	}
+
+	LOG_DEBUG("set mode %s", md.c_str());
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -597,10 +544,6 @@ void rigCAT_setmode(const string& md)
 			break;
 		++itrCmd;
 	}
-	if (itrCmd == commands.end()) {
-		noCATmode = md;
-		return;
-	}
 	modeCmd = *itrCmd;
 	
 	if ( modeCmd.str1.empty() == false)
@@ -633,17 +576,17 @@ void rigCAT_setmode(const string& md)
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				fl_lock(&rigCAT_mutex);
+				pthread_mutex_lock(&rigCAT_mutex);
 					hexout(strCmd, rTemp.size);
-				fl_unlock(&rigCAT_mutex);
+				pthread_mutex_unlock(&rigCAT_mutex);
 				return;
 			}
 			preply++;
 		}
 	} else {
-		fl_lock(&rigCAT_mutex);
+		pthread_mutex_lock(&rigCAT_mutex);
 			hexout(strCmd, 0);
-		fl_unlock(&rigCAT_mutex);
+		pthread_mutex_unlock(&rigCAT_mutex);
 	}
 }
 
@@ -653,7 +596,7 @@ string rigCAT_getwidth()
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 
-	if (nonCATrig == true) 
+	if (nonCATrig == true || noXMLfile) 
 		return noCATwidth;
 	
 	LOG_DEBUG("get width");
@@ -674,10 +617,6 @@ string rigCAT_getwidth()
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
 
-//	if (hexout(strCmd) == false) {
-//		return "";
-//	}
-
 	if (modeCmd.info.size()) {
 		list<XMLIOS>::iterator preply = reply.begin();
 		while (preply != reply.end()) {
@@ -754,6 +693,11 @@ void rigCAT_setwidth(const string& w)
 	
 	LOG_DEBUG("set width");
 
+	if (noXMLfile) {
+		noCATwidth = w;
+		return;
+	}
+	
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
 		if ((*itrCmd).SYMBOL == "SETBW")
@@ -797,17 +741,17 @@ void rigCAT_setwidth(const string& w)
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				fl_lock(&rigCAT_mutex);
+				pthread_mutex_lock(&rigCAT_mutex);
 					hexout(strCmd, rTemp.size);
-				fl_unlock(&rigCAT_mutex);
+				pthread_mutex_unlock(&rigCAT_mutex);
 				return;
 			}
 			preply++;
 		}
 	} else {
-		fl_lock(&rigCAT_mutex);
+		pthread_mutex_lock(&rigCAT_mutex);
 			hexout(strCmd, 0);
-		fl_unlock(&rigCAT_mutex);
+		pthread_mutex_unlock(&rigCAT_mutex);
 	}
 }
 
@@ -817,7 +761,7 @@ void rigCAT_pttON()
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 	
-	LOG_DEBUG("ptt ON");
+	LOG_INFO("ptt ON");
 
 	rigio.SetPTT(1); // always execute the h/w ptt if enabled
 
@@ -859,7 +803,7 @@ void rigCAT_pttOFF()
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 	
-	LOG_DEBUG("ptt OFF");
+	LOG_INFO("ptt OFF");
 
 	rigio.SetPTT(0); // always execute the h/w ptt if enabled
 
@@ -900,6 +844,9 @@ void rigCAT_sendINIT()
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 	
+	if (noXMLfile)
+		return;
+		
 	LOG_DEBUG("INIT rig");
 
 	itrCmd = commands.begin();
@@ -923,65 +870,139 @@ void rigCAT_sendINIT()
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				fl_lock(&rigCAT_mutex);
+				pthread_mutex_lock(&rigCAT_mutex);
 					hexout(strCmd, rTemp.size);
-				fl_unlock(&rigCAT_mutex);
+				pthread_mutex_unlock(&rigCAT_mutex);
 				return;
 			}
 			preply++;
 		}
 	} else {
-		fl_lock(&rigCAT_mutex);
+		pthread_mutex_lock(&rigCAT_mutex);
 			hexout(strCmd, 0);
-		fl_unlock(&rigCAT_mutex);
+		pthread_mutex_unlock(&rigCAT_mutex);
 	}
-//	fl_lock(&rigCAT_mutex);
-//		hexout(strCmd, 0);
-//	fl_unlock(&rigCAT_mutex);
 }
 
-bool rigCAT_init()
+void rigCAT_defaults()
+{
+    progdefaults.XmlRigBaudrate = xmlrig.baud;
+    mnuXmlRigBaudrate->value(xmlrig.baud);
+    
+    progdefaults.RigCatRTSplus = xmlrig.rts;
+    btnRigCatRTSplus->value(xmlrig.rts);
+    
+	progdefaults.RigCatDTRplus = xmlrig.dtr;
+	btnRigCatDTRplus->value(xmlrig.dtr);
+	
+	progdefaults.RigCatRTSptt = xmlrig.rtsptt;
+	btnRigCatRTSptt->value(xmlrig.rtsptt);
+	
+	progdefaults.RigCatDTRptt = xmlrig.dtrptt;
+	btnRigCatDTRptt->value(xmlrig.dtrptt);
+	
+	progdefaults.RigCatRTSCTSflow = xmlrig.rtscts;
+	chkRigCatRTSCTSflow->value(xmlrig.rtscts);
+
+    progdefaults.RigCatRetries = xmlrig.retries;
+    cntRigCatRetries->value(xmlrig.retries);
+    
+    progdefaults.RigCatTimeout = xmlrig.timeout;
+    cntRigCatTimeout->value(xmlrig.timeout);
+    
+    progdefaults.RigCatWait = xmlrig.write_delay;
+    cntRigCatWait->value(xmlrig.write_delay);
+    
+    progdefaults.RigCatECHO = xmlrig.echo;
+    btnRigCatEcho->value(xmlrig.echo);
+    
+    progdefaults.RigCatCMDptt = xmlrig.cmdptt;
+    btnRigCatCMDptt->value(xmlrig.cmdptt);
+
+}
+
+bool rigCAT_init(bool useXML)
 {
 	
 	if (rigCAT_open == true) {
-		LOG_ERROR("RigCAT already open file present");
+		LOG_ERROR("RigCAT already open");
 		return false;
 	}
-
-	if (readRigXML() == false) {
-		LOG_ERROR("No rig.xml file present");
-		return false;
-	}
-
-	if (rigio.OpenPort() == false) {
-		LOG_ERROR("Cannot open serial port %s", rigio.Device().c_str());
-		return false;
-	}
-	llFreq = 0;
+	
+	noXMLfile = true;
 	sRigMode = "";
 	sRigWidth = "";
+	nonCATrig = true;
+
+	if (useXML == true) {
+		if (readRigXML() == false)
+			LOG_ERROR("No rig.xml file present");
+		else {	
+			noXMLfile = false;
+			rigio.Device(progdefaults.XmlRigDevice);
+			rigio.Baud(progdefaults.BaudRate(progdefaults.XmlRigBaudrate));
+			rigio.RTS(progdefaults.RigCatRTSplus);
+			rigio.DTR(progdefaults.RigCatDTRplus);
+			rigio.RTSptt(progdefaults.RigCatRTSptt);
+			rigio.DTRptt(progdefaults.RigCatDTRptt);
+			rigio.RTSCTS(progdefaults.RigCatRTSCTSflow);
+			
+			LOG_INFO("\n\
+Serial port parameters:\n\
+device     : %s\n\
+baudrate   : %d\n\
+retries    : %d\n\
+timeout    : %d\n\
+wait       : %d\n\
+initial rts: %+d\n\
+use rts ptt: %c\n\
+initial dtr: %+d\n\
+use dtr ptt: %c\n\
+flowcontrol: %c\n\
+echo       : %c\n",
+	        	rigio.Device().c_str(),
+				rigio.Baud(),
+				progdefaults.RigCatRetries, 
+				progdefaults.RigCatTimeout, 
+				progdefaults.RigCatWait,
+				(rigio.RTS() ? +12 : -12), (rigio.RTSptt() ? 'T' : 'F'), 
+				(rigio.DTR() ? +12 : -12), (rigio.DTRptt() ? 'T' : 'F'),
+				(rigio.RTSCTS() ? 'T' : 'F'),
+				progdefaults.RigCatECHO ? 'T' : 'F');
+
+			if (rigio.OpenPort() == false) {
+				LOG_ERROR("Cannot open serial port %s", rigio.Device().c_str());
+				return false;
+			}
+			sRigMode = "";
+			sRigWidth = "";
 	
-	nonCATrig = false;
+			nonCATrig = false;
 	
-	if (rigCAT_getfreq() == -1) {
-		LOG_ERROR("Xcvr Freq request not answered");
-		rigio.ClosePort();
-		return false;
-	}
-	
-	rigCAT_sendINIT();
+			if (rigCAT_getfreq() == -1) { // will set nonCATrig to true if getfreq not spec'd
+				LOG_ERROR("Xcvr Freq request not answered");
+				rigio.ClosePort();
+				return false;
+			} else {
+				rigCAT_sendINIT();
+				init_Xml_RigDialog();
+			}
+		}
+	} else
+		init_NoRig_RigDialog();
 	
+	llFreq = 0;
 	rigCAT_bypass = false;
 	rigCAT_exit = false;
+	
+	rigCAT_thread = new pthread_t;
 
-	if (fl_create_thread(rigCAT_thread, rigCAT_loop, &dummy) < 0) {
+	if (pthread_create(rigCAT_thread, NULL, rigCAT_loop, NULL) < 0) {
 		LOG_ERROR("pthread_create failed");
 		rigio.ClosePort();
 		return false;
 	} 
 
-	init_Xml_RigDialog();
-
 	rigCAT_open = true;
 	return true;
 }
@@ -993,20 +1014,20 @@ void rigCAT_close(void)
 		return;
 	rigCAT_exit = true;
 	
-//	std::cout << "Waiting for RigCAT to exit "; fflush(stdout);
 	while (rigCAT_open == true) {
-//		std::cout << "."; fflush(stdout);
 		MilliSleep(50);
 		count--;
 		if (!count) {
 			LOG_ERROR("RigCAT stuck");
-			fl_lock(&rigCAT_mutex);
+			pthread_mutex_lock(&rigCAT_mutex);
 			rigio.ClosePort();
-			fl_unlock(&rigCAT_mutex);
+			pthread_mutex_unlock(&rigCAT_mutex);
 			exit(0);
 		}
 	}
-	rigio.ClosePort();
+	delete rigCAT_thread;
+	rigCAT_thread = 0;
+	LOG_DEBUG("RigCAT closed, count = %d", count);
 }
 
 bool rigCAT_active(void)
@@ -1018,7 +1039,7 @@ void rigCAT_set_ptt(int ptt)
 {
 	if (rigCAT_open == false)
 		return;
-	fl_lock(&rigCAT_mutex);
+	pthread_mutex_lock(&rigCAT_mutex);
 		if (ptt) {
 			rigCAT_pttON();
 			rigCAT_bypass = true;
@@ -1026,7 +1047,7 @@ void rigCAT_set_ptt(int ptt)
 			rigCAT_pttOFF();
 			rigCAT_bypass = false;
 		}
-	fl_unlock(&rigCAT_mutex);
+	pthread_mutex_unlock(&rigCAT_mutex);
 }
 
 #ifndef RIGCATTEST
@@ -1051,6 +1072,11 @@ void rigCAT_set_qsy(long long f, long long fmid)
 
 bool ModeIsLSB(const string& s)
 {
+	if (noXMLfile) {
+		if (s == "LSB")
+			return true;
+		return false;
+	}
 	list<string>::iterator pM = LSBmodes.begin();
 	while (pM != LSBmodes.end() ) {
 		if (*pM == s)
@@ -1069,72 +1095,58 @@ static void *rigCAT_loop(void *args)
 
 	for (;;) {
 		MilliSleep(100);
-
+		
 		if (rigCAT_bypass == true)
 			continue;
 		if (rigCAT_exit == true)
 			break;
 
-		fl_lock(&rigCAT_mutex);
+		pthread_mutex_lock(&rigCAT_mutex);
 			freq = rigCAT_getfreq();
-		fl_unlock(&rigCAT_mutex);
+		pthread_mutex_unlock(&rigCAT_mutex);
+
+		pthread_mutex_lock(&rigCAT_mutex);
+			sWidth = rigCAT_getwidth();
+		pthread_mutex_unlock(&rigCAT_mutex);
+
+		pthread_mutex_lock(&rigCAT_mutex);
+			sMode = rigCAT_getmode();
+		pthread_mutex_unlock(&rigCAT_mutex);
 
 		if ((freq > 0) && (freq != llFreq)) {
 			llFreq = freq;
-			FreqDisp->value(freq); // REQ is built in to the widget
+			show_frequency(freq);
 			wf->rfcarrier(freq);
 		}
-
-		if (rigCAT_bypass == true)
-			continue;
-		if (rigCAT_exit == true)
-			break;
-
-		fl_lock(&rigCAT_mutex);
-			sWidth = rigCAT_getwidth();
-		fl_unlock(&rigCAT_mutex);
-		
+	
 		if (sWidth.size() && sWidth != sRigWidth) {
 			sRigWidth = sWidth;
-			REQ(&Fl_ComboBox::put_value, opBW, sWidth.c_str());
+			show_bw(sWidth);
 		}
 
-		if (rigCAT_bypass == true)
-			continue;
-		if (rigCAT_exit == true)
-			break;
-
-		fl_lock(&rigCAT_mutex);
-			sMode = rigCAT_getmode();
-		fl_unlock(&rigCAT_mutex);
-
 		if (sMode.size() && sMode != sRigMode) {
 			sRigMode = sMode;
 			if (ModeIsLSB(sMode))
 				wf->USB(false);
 			else
 				wf->USB(true);
-			REQ(&Fl_ComboBox::put_value, opMODE, sMode.c_str());
+			show_mode(sMode);
 		}
 	}
 
+	if (rigcontrol)
+		rigcontrol->hide();
+	wf->USB(true);
+	wf->setQSY(0);
+
+	pthread_mutex_lock(&rigCAT_mutex);
+		LOG_DEBUG("Exiting rigCAT loop, closing serial port");
+		rigio.ClosePort();
+	pthread_mutex_unlock(&rigCAT_mutex);
+
 	rigCAT_open = false;
 	rigCAT_exit = false;
 	rigCAT_bypass = false;
-	if (rigcontrol)
-		rigcontrol->hide();
-	wf->USB(true);
-	wf->rfcarrier(atoi(cboBand->value())*1000L);
-	wf->setQSY(0);
-	FL_LOCK();
-	cboBand->show();
-	btnSideband->show();
-	activate_rig_menu_item(false);
-	FL_UNLOCK();
-
-	fl_lock(&rigCAT_mutex);
-		rigio.ClosePort();
-	fl_unlock(&rigCAT_mutex);
 
 	return NULL;
 }
