@@ -22,12 +22,15 @@
 #endif
 #include "debug.h"
 
+#include "configuration.h"
+#include "fileselect.h"
+#include "confdialog.h"
+
 //#define DEBUGXML
 
 void parseRIGDEF(size_t &);
 void parseRIG(size_t &);
 void parseCOMMAND(size_t &);
-void parsePORT(size_t &);
 void parseREPLY(size_t &);
 void parseMODES(size_t &);
 void parseBANDWIDTHS(size_t &);
@@ -40,6 +43,15 @@ void parseLSBMODES(size_t &);
 void parseCOMMENTS(size_t &);
 void parseDISCARD(size_t &);
 
+void parseWRITE_DELAY(size_t &);
+void parsePOST_WRITE_DELAY(size_t &);
+void parseRETRIES(size_t &);
+void parseTIMEOUT(size_t &);
+void parseBAUDRATE(size_t &);
+void parseRTSCTS(size_t &);
+void parseCMDPTT(size_t &);
+void parseECHO(size_t &);
+
 void parseIOSsymbol(size_t &);
 void parseIOSsize(size_t &);
 void parseIOSbytes(size_t &);
@@ -52,18 +64,6 @@ void parseIOSstring(size_t &);
 void parseIOSint(size_t &);
 void parseIOSfill(size_t &);
 
-void parseBaud(size_t &);
-void parseDevice(size_t &);
-void parseEcho(size_t &);
-void parseRetries(size_t &);
-void parseTimeout(size_t &);
-void parseWait(size_t &);
-void parseDTRinit(size_t &);
-void parseDTRptt(size_t &);
-void parseRTSinit(size_t &);
-void parseRTSptt(size_t &);
-void parseRTSCTS(size_t &);
-
 void parseDTYPE(size_t &);
 void parseDSIZE(size_t &);
 void parseDMAX(size_t &);
@@ -85,7 +85,7 @@ list<MODE>		lmodeCMD;
 list<MODE>		lmodeREPLY;
 list<string> 	LSBmodes;
 
-XMLRIG rig;
+XMLRIG xmlrig;
 
 XMLIOS iosTemp;
 
@@ -95,7 +95,6 @@ TAGS rigdeftags[] = {
 	{"<RIGDEF",		parseRIGDEF},
 	{"<RIG", 		parseRIG},
 	{"<COMMAND",	parseCOMMAND},
-	{"<PORT",		parsePORT},
 	{"<REPLY",		parseREPLY},
 	{"<BANDWIDTHS", parseBANDWIDTHS},
 	{"<BW-CMD",		parseBWCMD},
@@ -108,6 +107,14 @@ TAGS rigdeftags[] = {
 	{"<!--",        parseCOMMENTS},
 	{"<PROGRAMMER", parseDISCARD},
 	{"<STATUS",		parseDISCARD},
+	{"<WRITE_DELAY", parseWRITE_DELAY},
+	{"<POST_WRITE_DELAY", parsePOST_WRITE_DELAY},
+	{"<RETRIES", parseRETRIES},
+	{"<TIMEOUT", parseTIMEOUT},
+	{"<BAUDRATE", parseBAUDRATE},
+	{"<RTSCTS", parseRTSCTS},
+	{"<ECHO", parseECHO},
+	{"<CMDPTT", parseCMDPTT},
 	{0, 0} 
 };
 
@@ -137,21 +144,6 @@ TAGS replytags[] = {
 	{0,0}
 };
 
-TAGS porttags[] = {
-	{"<BAUD",	parseBaud},
-	{"<DEVICE",	parseDevice},
-	{"<ECHO",	parseEcho},
-	{"<RETRIES",parseRetries},
-	{"<TIMEOUT",parseTimeout},
-	{"<WAIT",	parseWait},
-	{"<DTRINIT",parseDTRinit},
-	{"<DTRPTT",	parseDTRptt},
-	{"<RTSINIT",parseRTSinit},
-	{"<RTSPTT",	parseRTSptt},
-	{"<RTSCTS",	parseRTSCTS},
-	{0,0}
-};
-
 TAGS datatags[] = {
 	{"<DTYPE",	parseDTYPE},
 	{"<SIZE",	parseDSIZE},
@@ -172,7 +164,7 @@ void print(size_t &p0, int indent)
 #ifdef DEBUGXML
 	std::string istr(indent, '\t');
 	size_t tend = strXML.find(">", p0);
-	LOG_DEBUG("%s%s", istr.c_str(), strXML.substr(p0, tend - p0 + 1).c_str());
+	LOG_INFO("%s%s", istr.c_str(), strXML.substr(p0, tend - p0 + 1).c_str());
 #endif
 }
 
@@ -476,133 +468,69 @@ void parseTITLE(size_t &p0)
 void parseRIG(size_t &p0)
 {
 	size_t pend = tagEnd(p0);
-	rig.SYMBOL = getElement(p0);
 	p0 = pend;
 }
 
 //---------------------------------------------------------------------
-// Parse Port definition
+// Parse Baudrate, write_delay, post_write_delay, timeout, retries
+// RTSCTS handshake
 //---------------------------------------------------------------------
 
-void parseBaud(size_t &p1)
-{
-	int baud;
-	baud = atoi (getElement (p1).c_str());
-	rig.baud = baud;
-}
-
-void parseDevice(size_t &p1)
-{
-	string stemp = getElement(p1);
-	rig.port = stemp;
-
-}
-
-void parseEcho(size_t &p1)
-{
-	rig.echo = getBool(p1);
-}
-
-void parseRetries(size_t &p1)
-{
-	rig.retries = getInt(p1);
-}
-
-void parseTimeout(size_t &p1)
-{
-	rig.timeout = getInt(p1);
-}
-
-void parseWait(size_t &p1)
-{
-	rig.wait = getInt(p1);
-}
-
-void parseDTRinit(size_t &p1)
-{
-	int val = getInt(p1);
-	if (val > 0)
-		rig.dtr = true;
-	else
-		rig.dtr = false;
-}
-
-void parseDTRptt(size_t &p1)
-{
-	rig.dtrptt = getBool(p1);
-}
-
-void parseRTSinit(size_t &p1)
-{
-	int val = getInt(p1);
-	if (val > 0)
-		rig.rts = true;
-	else
-		rig.rts = false;
-}
-
-void parseRTSptt(size_t &p1)
-{
-	rig.rtsptt = getBool(p1);
-}
-
-void parseRTSCTS(size_t &p1)
-{
-	rig.rtscts = getBool(p1);
-}
-
-//===================================================
-void parsePORT(size_t &p0)
+void parseBAUDRATE(size_t &p0)
 {
+    string sVal = getElement(p0);
+    xmlrig.baud = progdefaults.nBaudRate(sVal.c_str());
 	size_t pend = tagEnd(p0);
-	size_t p1;
-	TAGS *pv;
-
-	print(p0,0);
-
-	rig.clear();
-	p1 = nextTag(p0);
-	while (p1 < pend) {
-		pv = porttags;
-		while (pv->tag) {
-			if (strXML.find(pv->tag, p1) == p1) {
-				print(p1, 1);
-				if (pv->fp) 
-					(pv->fp)(p1);
-				break;
-			}
-			pv++;
-		}
-		p1 = tagEnd(p1);
-		p1 = nextTag(p1);
-	}
-	if (rig.port.size()) {
-		rigio.Baud( rig.baud );
-		rigio.Device( rig.port );
-		rigio.Retries(rig.retries);
-		rigio.Timeout(rig.timeout);
-		rigio.RTS(rig.rts);
-		rigio.RTSptt(rig.rtsptt);
-		rigio.DTR(rig.dtr);
-		rigio.DTRptt(rig.dtrptt);
-		rigio.RTSCTS(rig.rtscts);
-	}
 	p0 = pend;
+}
 
-	LOG_DEBUG("\nSerial port parameters:\n\
-    port: %s\n\
-    baud: %d\n\
-    retries: %d\n\
-    timeout: %d\n\
-    initial rts: %+d\n\
-    use rts ptt: %c\n\
-    initial dts: %+d\n\
-    use dtr ptt: %c\n\
-    use flowcontrol: %c",
-		  rig.port.c_str(), rig.baud, rig.retries, rig.timeout, (rig.rts ? +12 : -12),
-		  (rig.rtsptt ? 'T' : 'F'), (rig.dtr ? +12 : -12), (rig.dtrptt ? 'T' : 'F'),
-		  (rig.rtscts ? 'T' : 'F'));
+void parseWRITE_DELAY(size_t &p0){
+    int val = getInt(p0);
+    xmlrig.write_delay = val;
+    size_t pend = tagEnd(p0);
+	p0 = pend;
+}
 
+void parsePOST_WRITE_DELAY(size_t &p0){
+    int val = getInt(p0);
+    xmlrig.post_write_delay = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parseRETRIES(size_t &p0){
+    int val = getInt(p0);
+    xmlrig.retries = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parseTIMEOUT(size_t &p0){
+    int val = getInt(p0);
+    xmlrig.timeout = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parseRTSCTS(size_t &p0){
+    bool val = getBool(p0);
+    xmlrig.rtscts = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parseCMDPTT(size_t &p0) {
+    bool val = getBool(p0);
+    xmlrig.cmdptt = val;
+    size_t pend = tagEnd(p0);
+    p0 = pend;
+}
+
+void parseECHO(size_t &p0) {
+    bool val = getBool(p0);
+    xmlrig.echo = val;
+    size_t pend = tagEnd(p0);
+    p0 = pend;
 }
 
 //---------------------------------------------------------------------
@@ -850,7 +778,7 @@ bool readRigXML()
 	LSBmodes.clear();
 	strXML = "";
 
-	ifstream xmlfile(xmlfname.c_str(), ios::in);
+	ifstream xmlfile(progdefaults.XmlRigFilename.c_str(), ios::in);
 	if (xmlfile) {
 		while (!xmlfile.eof()) {
 			lines++;
@@ -865,3 +793,15 @@ bool readRigXML()
 	return false;
 }
 
+void selectRigXmlFilename()
+{
+	string deffilename;
+	deffilename = progdefaults.XmlRigFilename;
+    const char *p = FSEL::select("Open rig xml file", "Fldigi rig xml definition file\t*.xml", deffilename.c_str());
+    if (p) {
+   		progdefaults.XmlRigFilename = p;
+   		txtXmlRigFilename->value(fl_filename_name(p));
+   		readRigXML();
+   		rigCAT_defaults();
+	}
+}
