@@ -7,24 +7,37 @@
 //-----------------------------------------------------------------------------
 
 #include <config.h>
+
+#include <cstdio>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <sys/time.h>
+
+#include <memory>
 #include <string>
-#include <sstream>
 
 #include "serial.h"
-#include "re.h"
 #include "debug.h"
 
+LOG_SET_SOURCE(debug::LOG_RIGCONTROL);
+
+using namespace std;
+
 #ifdef __CYGWIN__
+#include <sstream>
+#include "re.h"
 // convert COMx to /dev/ttySy with y = x - 1
-static void adjust_port(string& port)
+void adjust_port(string& port)
 {
 	re_t re("com([0-9]+)", REG_EXTENDED | REG_ICASE);
-	const char* s;
-	if (!(re.match(port.c_str()) && (s = re.submatch(1))))
+	if (!(re.match(port.c_str()) && re.nsub() == 2))
 		return;
 	stringstream ss;
 	int n;
-	ss << s;
+	ss << re.submatch(1);
 	ss >> n;
 	if (--n < 0)
 		n = 0;
@@ -59,6 +72,7 @@ Cserial::~Cserial() {
 // Argument         : c_string strPortName
 ///////////////////////////////////////////////////////
 bool Cserial::OpenPort()  {
+	
 #ifdef __CYGWIN__
 	adjust_port(device);
 #endif
@@ -155,38 +169,28 @@ void Cserial::SetPTT(bool b)
 		LOG_DEBUG("ptt fd < 0");
 		return;
 	}
-	if ( !(dtrptt || rtsptt) ) {
-		LOG_DEBUG("No h/w PTT");
-		return;
-	}
-
-	ioctl(fd, TIOCMGET, &status);
-	LOG_DEBUG("Status %X", status);
-	if (b == true) {				// ptt enabled
-		if (dtrptt && dtr)
-			status &= ~TIOCM_DTR;	// toggle low
-		if (dtrptt && !dtr)
-			status |= TIOCM_DTR;	// toggle high
-		if (rtscts == false) {
-			if (rtsptt && rts)
-				status &= ~TIOCM_RTS;	// toggle low
-			if (rtsptt && !rts)
-				status |= TIOCM_RTS;	// toggle high
-		}
-	} else {						// ptt disabled
-		if (dtrptt && dtr)
-			status |= TIOCM_DTR;	// toggle high
-		if (dtrptt && !dtr)
-			status &= ~TIOCM_DTR;	// toggle low
-		if (rtscts == false) {
-			if (rtsptt && rts)
-				status |= TIOCM_RTS;	// toggle high
-			if (rtsptt && !rts)
-				status &= ~TIOCM_RTS;	// toggle low
-		}
-	}
-	LOG_DEBUG("Status %X", status);
-	ioctl(fd, TIOCMSET, &status);
+	if (dtrptt || rtsptt) {
+        ioctl(fd, TIOCMGET, &status);
+	    LOG_INFO("h/w ptt %d, Status %X", b, status);
+	    if (b == true) {                                  // ptt enabled
+		    if (dtrptt && dtr)  status &= ~TIOCM_DTR;     // toggle low
+		    if (dtrptt && !dtr) status |= TIOCM_DTR;      // toggle high
+            if (rtscts == false) {
+			    if (rtsptt && rts)  status &= ~TIOCM_RTS; // toggle low
+			    if (rtsptt && !rts) status |= TIOCM_RTS;  // toggle high
+		    }
+	    } else {                                          // ptt disabled
+		    if (dtrptt && dtr)  status |= TIOCM_DTR;      // toggle high
+		    if (dtrptt && !dtr) status &= ~TIOCM_DTR;     // toggle low
+		    if (rtscts == false) {
+			    if (rtsptt && rts)  status |= TIOCM_RTS;  // toggle high
+			    if (rtsptt && !rts) status &= ~TIOCM_RTS; // toggle low
+		    }
+	    }
+	    LOG_INFO("Status %02X, %s", status & 0xFF, uint2bin(status, 8));
+	    ioctl(fd, TIOCMSET, &status);
+    }
+    LOG_DEBUG("No ptt specified");
 }
 
 ///////////////////////////////////////////////////////
@@ -197,10 +201,11 @@ void Cserial::SetPTT(bool b)
 void Cserial::ClosePort()
 {
 	if (fd < 0) return;
-	LOG_DEBUG("Serial port closed");
+	LOG_INFO("Serial port closed, fd = %d", fd);
 	ioctl(fd, TIOCMSET, &origstatus);
 	tcsetattr (fd, TCSANOW, &oldtio);
 	close(fd);
+	fd = -1;
 	return;
 }
 
@@ -229,6 +234,7 @@ bool  Cserial::IOselect ()
 ///////////////////////////////////////////////////////
 int  Cserial::ReadBuffer (unsigned char *buf, int nchars)
 {
+	if (fd < 0) return 0;
 	int retnum, nread = 0;
 	while (nchars > 0) {
 		if (!IOselect()) {
@@ -253,8 +259,7 @@ int  Cserial::ReadBuffer (unsigned char *buf, int nchars)
 ///////////////////////////////////////////////////////
 int Cserial::WriteBuffer(unsigned char *buff, int n)
 {
-	if (fd < 0) 
-		return 0;
+	if (fd < 0) return 0;
 	int ret = write (fd, buff, n);
 	return ret;
 }
