@@ -145,13 +145,8 @@ RSIDs cRsId::rsid_ids[] = {
 	{ RSID_MFSK8, MODE_MFSK8 },
 	{ RSID_MFSK16, MODE_MFSK16 },
 	{ RSID_MFSK32, MODE_MFSK32 },
-#ifdef EXPERIMENTAL
 	{ RSID_MFSK11, MODE_MFSK11 },
 	{ RSID_MFSK22, MODE_MFSK22 },
-#else
-	{ RSID_MFSK11, NUM_MODES },
-	{ RSID_MFSK22, NUM_MODES },
-#endif
 
 	{ RSID_RTTYM_8_250, NUM_MODES },
 	{ RSID_RTTYM_16_500, NUM_MODES },
@@ -313,10 +308,9 @@ void cRsId::CalculateBuckets(const double *pSpectrum, int iBegin, int iEnd)
 	double   Amp = 0.0, AmpMax = 0.0;
 	int   iBucketMax = iBegin - RSID_RESOL;
 	int   i, j;
-	bool  firstpass = true;
 
 	for (i = iBegin; i < iEnd; i += RSID_RESOL) {
-		if (firstpass) {
+		if (iBucketMax == i - RSID_RESOL) {
 			AmpMax		= pSpectrum[i];
 			iBucketMax	= i;
 			for (j = i + RSID_RESOL; j < i + RSID_NTIMES + RSID_RESOL; j += RSID_RESOL) {
@@ -326,7 +320,6 @@ void cRsId::CalculateBuckets(const double *pSpectrum, int iBegin, int iEnd)
 					iBucketMax = j;
 				}
 			}
-			firstpass = false;
 		} else {
 			j    = i + RSID_NTIMES;
 			Amp = pSpectrum[j];
@@ -372,10 +365,9 @@ void cRsId::search( const double *pSamples, int nSamples )
 	memcpy  (aInputSamples + ns, pSamples, ns * sizeof(double));
 	
 	memset  (aFFTReal, 0, RSID_ARRAY_SIZE * sizeof(double));
-	memcpy  (aFFTReal, aInputSamples, RSID_FFT_SIZE * sizeof(double));
-// or 
-//	for (int i = 0; i < RSID_FFT_SIZE; i++)
-//		aFFTReal[i] = aInputSamples[i] * fftwindow[i];
+
+	for (int i = 0; i < RSID_FFT_SIZE; i++)
+		aFFTReal[i] = aInputSamples[i] * fftwindow[i];
 
 	rsrfft( aFFTReal, 11);
 
@@ -503,10 +495,11 @@ void cRsId::apply(int iSymbol, int iBin)
 		break;
 	}
 
-	REQ(&configuration::loadDefaults, &progdefaults);
+//	REQ(&configuration::loadDefaults, &progdefaults);
 
-	active_modem->set_freq(freq);
+//	active_modem->set_freq(freq);
 	REQ(init_modem, mbin);
+	active_modem->set_freq(freq);
 	
 }
 
@@ -609,7 +602,7 @@ bool cRsId::search_amp( int &SymbolOut,	int &BinOut)
 // transmit rsid code for current mode
 //=============================================================================
 
-void cRsId::send()
+void cRsId::send(bool postidle)
 {
 	int iTone;
 	uchar rsid[RSID_NSYMBOLS];
@@ -710,11 +703,17 @@ void cRsId::send()
 	}
 	if (rmode == RSID_NONE)
 		return;
+		
 	Encode(rmode, rsid);
 
-
 	outbuf = new double[symlen];
 		
+// transmit 6 symbol periods of silence
+    if (!postidle) {
+    	for (int j = 0; j < symlen; j++) outbuf[j] = 0.0;
+	    for (int i = 0; i < 6; i++) active_modem->ModulateXmtr(outbuf, symlen);
+    }
+
 // transmit sequence of 15 symbols (tones)
 	phase = 0.0;
 	for (int i = 0; i < 15; i++) {
@@ -732,11 +731,11 @@ void cRsId::send()
 		active_modem->ModulateXmtr(outbuf, symlen);
 
 	}
-// transmit 3 symbol periods of silence
-	for (int j = 0; j < symlen; j++) outbuf[j] = 0.0;
-	active_modem->ModulateXmtr(outbuf, symlen);
-	active_modem->ModulateXmtr(outbuf, symlen);
-	active_modem->ModulateXmtr(outbuf, symlen);
+// transmit 6 symbol periods of silence
+    if (postidle) {
+    	for (int j = 0; j < symlen; j++) outbuf[j] = 0.0;
+	    for (int i = 0; i < 6; i++) active_modem->ModulateXmtr(outbuf, symlen);
+    }
 // clean up
 
 	delete [] outbuf;
