@@ -65,6 +65,8 @@
 #include "ringbuffer.h"
 #include "debug.h"
 
+LOG_SET_SOURCE(debug::LOG_AUDIO);
+
 // Define these constants here to avoid littering the code with magic numbers.
 // Audio input is mono in SoundPort and SoundPulse, stereo in SoundOSS
 #define INPUT_CHANNELS 1
@@ -998,7 +1000,7 @@ size_t SoundPort::Read(double *buf, size_t count)
 		sd[0].blocksize = SCBLOCKSIZE;
 		while (n < count) {
 			if  ((r = src_callback_read(rx_src_state, sd[0].src_ratio, count - n, rbuf + n)) == 0)
-				return n;
+				throw SndException(ETIMEDOUT);
 			n += r;
 		}
 	}
@@ -1248,12 +1250,11 @@ void SoundPort::init_stream(unsigned dir)
 	else if (sd[dir].idev == devs.end()) // if we only found a near-match point the idev iterator to it
 		sd[dir].idev = devs.begin() + idx;
 
-
-#ifdef __linux__
-	if ((dir == 0 && (*sd[dir].idev)->maxInputChannels == 0) ||
-	    (dir == 1 && (*sd[dir].idev)->maxOutputChannels == 0))
+	const PaHostApiInfo* host_api = Pa_GetHostApiInfo((*sd[dir].idev)->hostApi);
+	int max_channels = dir ? (*sd[dir].idev)->maxOutputChannels :
+		(*sd[dir].idev)->maxInputChannels;
+	if ((host_api->type == paALSA || host_api->type == paOSS) && max_channels == 0)
 		throw SndException(EBUSY);
-#endif
 
 	if (dir == 0) {
 		sd[0].params.device = idx;
@@ -1266,7 +1267,7 @@ void SoundPort::init_stream(unsigned dir)
 		sd[1].params.device = idx;
 		sd[1].params.channelCount = OUTPUT_CHANNELS;
 		sd[1].params.sampleFormat = paFloat32;
-                if (Pa_GetHostApiInfo((*sd[dir].idev)->hostApi)->type == paMME)
+		if (host_api->type == paMME)
                         sd[1].params.suggestedLatency = (*sd[dir].idev)->defaultLowOutputLatency;
                 else
                         sd[1].params.suggestedLatency = (*sd[dir].idev)->defaultHighOutputLatency;
@@ -1286,7 +1287,7 @@ void SoundPort::init_stream(unsigned dir)
         device_text[dir]
 		<< "index: " << idx
 		<< "\nname: " << (*sd[dir].idev)->name
-		<< "\nhost API: " << Pa_GetHostApiInfo((*sd[dir].idev)->hostApi)->name
+		<< "\nhost API: " << host_api->name
 		<< "\nmax input channels: " << (*sd[dir].idev)->maxInputChannels
 		<< "\nmax output channels: " << (*sd[dir].idev)->maxOutputChannels
 		<< "\ndefault sample rate: " << (*sd[dir].idev)->defaultSampleRate
@@ -1297,8 +1298,8 @@ void SoundPort::init_stream(unsigned dir)
 		<< "\nfull duplex: " << full_duplex_device(*sd[dir].idev)
 		<< "\nsystem default input: " << (idx == Pa_GetDefaultInputDevice())
 		<< "\nsystem default output: " << (idx == Pa_GetDefaultOutputDevice())
-		<< "\nhost API default input: " << (idx == Pa_GetHostApiInfo((*sd[dir].idev)->hostApi)->defaultInputDevice)
-		<< "\nhost API default output: " << (idx == Pa_GetHostApiInfo((*sd[dir].idev)->hostApi)->defaultOutputDevice)
+		<< "\nhost API default input: " << (idx == host_api->defaultInputDevice)
+		<< "\nhost API default output: " << (idx == host_api->defaultOutputDevice)
 		<< "\ndefault low input latency: " << (*sd[dir].idev)->defaultLowInputLatency
 		<< "\ndefault high input latency: " << (*sd[dir].idev)->defaultHighInputLatency
 		<< "\ndefault low output latency: " << (*sd[dir].idev)->defaultLowOutputLatency
@@ -1501,7 +1502,7 @@ void SoundPort::pa_perror(int err, const char* str)
 				LOG_ERROR("Possible OSS error %d: %s", errno, strerror(errno));
                 }
                 else
-			LOG_ERROR("%s error %d: %s", Pa_GetHostApiInfo(i)->name,
+			LOG_ERROR("%s error %ld: %s", Pa_GetHostApiInfo(i)->name,
 				  hosterr->errorCode, hosterr->errorText);
         }
 }
@@ -1828,17 +1829,19 @@ size_t SoundNull::Write_stereo(double* bufleft, double* bufright, size_t count)
 
 size_t SoundNull::Read(double *buf, size_t count)
 {
-	memset(buf, 0, count * sizeof(*buf));
-
 #if USE_SNDFILE
-	if (capture)
-		write_file(ofCapture, buf, count);
 	if (playback) {
 		read_file(ifPlayback, buf, count);
 		if (progdefaults.EnableMixer)
 			for (size_t i = 0; i < count; i++)
 				buf[i] *= progStatus.RcvMixer;
 	}
+	else
+#endif
+		memset(buf, 0, count * sizeof(*buf));
+#if USE_SNDFILE
+	if (capture)
+		write_file(ofCapture, buf, count);
 #endif
 
 	usleep((useconds_t)ceil((1e6 * count) / sample_frequency));
