@@ -93,6 +93,7 @@ modem::modem()
 	reverse = wfrev ^ !wfsb;
 	historyON = false;
 	cap = 0;
+	PTTphaseacc = 0.0;
 }
 
 void modem::init()
@@ -102,15 +103,19 @@ void modem::init()
 	reverse = wfrev ^ !wfsb;
 	
 	if (progdefaults.StartAtSweetSpot) {
-		if (active_modem == cw_modem)
+//		if (active_modem == cw_modem)
+		if (this == cw_modem)
 			set_freq(progdefaults.CWsweetspot);
-		else if (active_modem == rtty_modem)
+//		else if (active_modem == rtty_modem)
+		else if (this == rtty_modem)
 			set_freq(progdefaults.RTTYsweetspot);
 		else
 			set_freq(progdefaults.PSKsweetspot);
 	} else if (progStatus.carrier != 0) {
-			set_freq(progStatus.carrier);
-			progStatus.carrier = 0;
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
 	} else
 		set_freq(wf->Carrier());
 }
@@ -139,11 +144,15 @@ bool modem::freqlocked()
 
 double modem::get_txfreq(void)
 {
+	if (mailserver && progdefaults.PSKmailSweetSpot)
+		return progdefaults.PSKsweetspot;
 	return tx_frequency;
 }
 
 double modem::get_txfreq_woffset(void)
 {
+	if (mailserver && progdefaults.PSKmailSweetSpot)
+		return (progdefaults.PSKsweetspot - progdefaults.TxOffset);
 	return (tx_frequency - progdefaults.TxOffset);
 }
 
@@ -224,10 +233,24 @@ void modem::set_samplerate(int smprate)
 	samplerate = smprate;
 }
 
+double modem::PTTnco()
+{
+	PTTphaseacc += TWOPI * 1000 / samplerate;
+	if (PTTphaseacc > M_PI)
+		PTTphaseacc -= TWOPI;
+	return sin(PTTphaseacc);
+}
+
 mbuffer<double, 512 * 2, 2> _mdm_scdbl;
 
 void modem::ModulateXmtr(double *buffer, int len) 
 {
+    if (progdefaults.PTTrightchannel) {
+        for (int i = 0; i < len; i++)
+            PTTchannel[i] = PTTnco();
+            ModulateStereo( buffer, PTTchannel, len);
+        return;
+    }
 	try {
 		unsigned n = 4;
 		while (scard->Write(buffer, len) == 0 && --n);
@@ -242,7 +265,7 @@ void modem::ModulateXmtr(double *buffer, int len)
 	if (!progdefaults.viewXmtSignal)
 		return;
 	for (int i = 0; i < len; i++) {
-		_mdm_scdbl[scptr] = buffer[i] * 0.1;
+		_mdm_scdbl[scptr] = buffer[i] * progdefaults.TxMonitorLevel;
 		scptr++;
 		if (scptr == 512) {
 			REQ(&waterfall::sig_data, wf, _mdm_scdbl.c_array(), 512, samplerate );
@@ -252,6 +275,8 @@ void modem::ModulateXmtr(double *buffer, int len)
 	}
 }
 
+#include <iostream>
+using namespace std;
 void modem::ModulateStereo(double *left, double *right, int len)
 {
 	try {
@@ -268,7 +293,7 @@ void modem::ModulateStereo(double *left, double *right, int len)
 	if (!progdefaults.viewXmtSignal)
 		return;
 	for (int i = 0; i < len; i++) {
-		_mdm_scdbl[scptr] = left[i] * 0.1;
+		_mdm_scdbl[scptr] = left[i] * progdefaults.TxMonitorLevel;
 		scptr++;
 		if (scptr == 512) {
 			REQ(&waterfall::sig_data, wf, _mdm_scdbl.c_array(), 512, samplerate);
@@ -276,6 +301,8 @@ void modem::ModulateStereo(double *left, double *right, int len)
 			_mdm_scdbl.next(); // change buffers
 		}
 	}
+//	for (int i = 0; i < len; i++)
+//	    std::cout << (left[i] + 1.0) << "," << (right[i] - 1.0) << std::endl;
 }
 
 
