@@ -24,6 +24,8 @@
 
 #include <config.h>
 
+#include <fcntl.h>
+
 #include <string>
 
 #include "trx.h"
@@ -45,6 +47,12 @@
 #include "qrunner.h"
 #include "debug.h"
 
+#if BENCHMARK_MODE
+#  include "benchmark.h"
+#endif
+
+LOG_SET_SOURCE(debug::LOG_MODEM);
+
 using namespace std;
 
 void	trx_reset_loop();
@@ -52,15 +60,16 @@ void	trx_start_modem_loop();
 void	trx_receive_loop();
 void	trx_transmit_loop();
 void	trx_tune_loop();
+static void signal_trx_state(void);
 
 //#define DEBUG
 
 /* ---------------------------------------------------------------------- */
 
 
-Fl_Mutex	trx_cond_mutex = PTHREAD_MUTEX_INITIALIZER;
-Fl_Cond		trx_cond = PTHREAD_COND_INITIALIZER;
-Fl_Thread	trx_thread;
+pthread_mutex_t	trx_cond_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_cond_t	trx_cond = PTHREAD_COND_INITIALIZER;
+pthread_t	trx_thread;
 state_t 	trx_state;
 bool		restartOK = false;
 bool		trx_wait = false;
@@ -78,7 +87,6 @@ ringbuffer<double> trxrb(ceil2(NUMMEMBUFS * SCBLOCKSIZE));
 ringbuffer<double>::vector_type rbvec[2];
 bool    bHistory = false;
 
-static int dummy = 0;
 static bool trxrunning = false;
 
 static bool rsid_detecting = false;
@@ -92,118 +100,127 @@ void trx_trx_receive_loop()
 	int  current_samplerate;
 	assert(powerof2(SCBLOCKSIZE));
 
-	if (!scard) {
+	if (unlikely(!active_modem)) {
 		MilliSleep(10);
 		return;
 	}
-	if (active_modem) {
-		try {
-			current_samplerate = active_modem->get_samplerate();
-			if (scard->Open(O_RDONLY, current_samplerate))
-				REQ(sound_update, progdefaults.btnAudioIOis);
-		}
-		catch (const SndException& e) {
-			LOG_ERROR("%s", e.what());
-			put_status(e.what(), 5);
-			scard->Close();
-			if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
-				sound_close();
-				sound_init();
-			}
-			MilliSleep(1000);
-			return;
-		}
-		active_modem->rx_init();
 
-		while (1) {
-			if (progdefaults.rsid == true && rsid_detecting == false) {
-				rsid_detecting = true;
-				try {
-					current_samplerate = ReedSolomon->samplerate();
-					scard->Open(O_RDONLY, current_samplerate);
-				}
-				catch (const SndException& e) {
-					LOG_ERROR("%s", e.what());
-					put_status(e.what(), 5);
-					scard->Close();
-					if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
-						sound_close();
-						sound_init();
-					}
-					MilliSleep(1000);
-					return;
-				}
-			}
-			if (progdefaults.rsid == false && rsid_detecting == true) {
-				rsid_detecting = false;
-				active_modem->rx_init();
-				try {
-					current_samplerate = active_modem->get_samplerate();
-					scard->Open(O_RDONLY, current_samplerate);
-				}
-				catch (const SndException& e) {
-					LOG_ERROR("%s", e.what());
-					put_status(e.what(), 5);
-					scard->Close();
-					if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
-						sound_close();
-						sound_init();
-					}
-					MilliSleep(1000);
-					return;
-				}
-			}
-			// If we change to an 8000Hz modem while RSID is on we'll never detect anything.
-			// Toggle rsid_detecting so that the audio device is reopened with the ReedSolomon
-			// samplerate in the next loop iteration.
-			if (progdefaults.rsid && rsid_detecting && current_samplerate != ReedSolomon->samplerate())
-				rsid_detecting = false;
+#if BENCHMARK_MODE
+	do_benchmark();
+	trx_state = STATE_ENDED;
+	return;
+#endif
 
+	if (unlikely(!scard)) {
+		MilliSleep(10);
+		return;
+	}
+
+	try {
+		current_samplerate = active_modem->get_samplerate();
+		if (scard->Open(O_RDONLY, current_samplerate))
+			REQ(sound_update, progdefaults.btnAudioIOis);
+	}
+	catch (const SndException& e) {
+		LOG_ERROR("%s", e.what());
+		put_status(e.what(), 5);
+		scard->Close();
+		if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
+			sound_close();
+			sound_init();
+		}
+		MilliSleep(1000);
+		return;
+	}
+	active_modem->rx_init();
+
+	while (1) {
+		if (progdefaults.rsid == true && rsid_detecting == false) {
+			rsid_detecting = true;
 			try {
-				if (trxrb.write_space() == 0) // discard some old data
-					trxrb.read_advance(SCBLOCKSIZE);
-				trxrb.get_wv(rbvec);
-				numread = 0;
-				while (numread < SCBLOCKSIZE && trx_state == STATE_RX)
-					numread += scard->Read(rbvec[0].buf + numread, SCBLOCKSIZE - numread);
+				current_samplerate = ReedSolomon->samplerate();
+				scard->Open(O_RDONLY, current_samplerate);
 			}
 			catch (const SndException& e) {
-				scard->Close();
 				LOG_ERROR("%s", e.what());
 				put_status(e.what(), 5);
-				MilliSleep(10);
+				scard->Close();
+				if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
+					sound_close();
+					sound_init();
+				}
+				MilliSleep(1000);
 				return;
 			}
-			if (trx_state != STATE_RX)
-				break;
-
-			trxrb.write_advance(numread);
-			REQ(&waterfall::sig_data, wf, rbvec[0].buf, numread, current_samplerate);
-
-			if (!bHistory) {
-				if (rsid_detecting == true)
-					ReedSolomon->search(rbvec[0].buf, numread);
-				else
-					active_modem->rx_process(rbvec[0].buf, numread);
+		}
+		if (progdefaults.rsid == false && rsid_detecting == true) {
+			rsid_detecting = false;
+			active_modem->rx_init();
+			try {
+				current_samplerate = active_modem->get_samplerate();
+				scard->Open(O_RDONLY, current_samplerate);
 			}
-			else {
-				bool afc = progStatus.afconoff;
-				progStatus.afconoff = false;
-				QRUNNER_DROP(true);
-				active_modem->HistoryON(true);
-				trxrb.get_rv(rbvec);
-				if (rbvec[0].len)
-					active_modem->rx_process(rbvec[0].buf, rbvec[0].len);
-				if (rbvec[1].len)
-					active_modem->rx_process(rbvec[1].buf, rbvec[1].len);
-				QRUNNER_DROP(false);
-				progStatus.afconoff = afc;
-				bHistory = false;
-				active_modem->HistoryON(false);
+			catch (const SndException& e) {
+				LOG_ERROR("%s", e.what());
+				put_status(e.what(), 5);
+				scard->Close();
+				if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
+					sound_close();
+					sound_init();
+				}
+				MilliSleep(1000);
+				return;
 			}
 		}
-	} else
-		MilliSleep(10);
+		// If we change to an 8000Hz modem while RSID is on we'll never detect anything.
+		// Toggle rsid_detecting so that the audio device is reopened with the ReedSolomon
+		// samplerate in the next loop iteration.
+		if (progdefaults.rsid && rsid_detecting && current_samplerate != ReedSolomon->samplerate())
+			rsid_detecting = false;
+
+		try {
+			if (trxrb.write_space() == 0) // discard some old data
+				trxrb.read_advance(SCBLOCKSIZE);
+			trxrb.get_wv(rbvec);
+			numread = 0;
+			while (numread < SCBLOCKSIZE && trx_state == STATE_RX)
+				numread += scard->Read(rbvec[0].buf + numread, SCBLOCKSIZE - numread);
+		}
+		catch (const SndException& e) {
+			scard->Close();
+			LOG_ERROR("%s", e.what());
+			put_status(e.what(), 5);
+			MilliSleep(10);
+			return;
+		}
+		if (trx_state != STATE_RX)
+			break;
+
+		trxrb.write_advance(numread);
+		REQ(&waterfall::sig_data, wf, rbvec[0].buf, numread, current_samplerate);
+
+		if (!bHistory) {
+			if (rsid_detecting == true)
+				ReedSolomon->search(rbvec[0].buf, numread);
+			else
+				active_modem->rx_process(rbvec[0].buf, numread);
+		}
+		else {
+			bool afc = progStatus.afconoff;
+			progStatus.afconoff = false;
+			QRUNNER_DROP(true);
+			active_modem->HistoryON(true);
+			trxrb.get_rv(rbvec);
+			if (rbvec[0].len)
+				active_modem->rx_process(rbvec[0].buf, rbvec[0].len);
+			if (rbvec[1].len)
+				active_modem->rx_process(rbvec[1].buf, rbvec[1].len);
+			QRUNNER_DROP(false);
+			progStatus.afconoff = afc;
+			bHistory = false;
+			active_modem->HistoryON(false);
+		}
+	}
 }
 
 
@@ -232,7 +249,7 @@ void trx_trx_transmit_loop()
 		active_modem->tx_init(scard);
 
 		if (progdefaults.TransmitRSid == true)
-			ReedSolomon->send();
+			ReedSolomon->send(true);
 		
 		while (trx_state == STATE_TX) {
 			try {
@@ -249,7 +266,7 @@ void trx_trx_transmit_loop()
 		}
 
 		if (progdefaults.TransmitRSid == true)
-			ReedSolomon->send();
+			ReedSolomon->send(false);
 		
         scard->flush();
 		if (scard->must_close())
@@ -322,12 +339,20 @@ void *trx_loop(void *args)
 {
 	SET_THREAD_ID(TRX_TID);
 
+	state_t old_state = STATE_NOOP;
+
 	for (;;) {
+		if (unlikely(old_state != trx_state)) {
+			old_state = trx_state;
+			signal_trx_state();
+		}
 		switch (trx_state) {
 		case STATE_ABORT:
 			delete scard;
 			scard = 0;
 			trx_state = STATE_ENDED;
+			// fall through
+		case STATE_ENDED:
 			return 0;
 		case STATE_RESTART:
 			trx_reset_loop();
@@ -345,39 +370,28 @@ void *trx_loop(void *args)
 			trx_trx_receive_loop();
 			break;
 		default:
-			LOG_ERROR("trx in bad state %d\n", trx_state);
+			LOG(debug::ERROR_LEVEL, debug::LOG_MODEM, "trx in bad state %d\n", trx_state);
 			MilliSleep(100);
 		}
 	}
 }
 
 //=============================================================================
-modem *trx_m;
+modem* new_modem;
 
 void trx_start_modem_loop()
 {
-	if (trx_m == active_modem) {
-		trx_state = STATE_RX;
+	if (new_modem == active_modem) {
 		active_modem->restart();
-		signal_modem_ready();
+		trx_state = STATE_RX;
 		return;
 	}
 
 	modem* old_modem = active_modem;
 
-	if (old_modem == trx_m) {
-		trx_state = STATE_RX;
-		signal_modem_ready();
-		return;
-	}
-
-	if (old_modem)
-		old_modem->shutdown();
-
-	active_modem = trx_m;
-	active_modem->init();
+	new_modem->init();
+	active_modem = new_modem;
 	trx_state = STATE_RX;
-	signal_modem_ready();
 	REQ(&waterfall::opmode, wf);
 
 	if (old_modem) {
@@ -389,7 +403,7 @@ void trx_start_modem_loop()
 //=============================================================================
 void trx_start_modem(modem *m)
 {
-	trx_m = m;
+	new_modem = m;
 	trx_state = STATE_NEW_MODEM;
 }
 
@@ -449,7 +463,7 @@ void macro_timer(void *)
 		btnMacroDummy->show();
 		FL_UNLOCK();
 	} else {
-		snprintf(timermsg, sizeof(timermsg), "Timer: %d", countdown);
+		snprintf(timermsg, sizeof(timermsg), "%d", countdown);
 		FL_LOCK();
 		btnMacroTimer->label(timermsg);
 		btnMacroTimer->redraw_label();
@@ -463,7 +477,7 @@ void trx_start_macro_timer()
 {
 	countdown = progdefaults.timeout;
 	Fl::add_timeout(1.0, macro_timer);
-	snprintf(timermsg, sizeof(timermsg), "Timer: %d", countdown);
+	snprintf(timermsg, sizeof(timermsg), "%d", countdown);
 	FL_LOCK();
 	btnMacroTimer->label(timermsg);
 	btnMacroTimer->redraw_label();
@@ -475,8 +489,9 @@ void trx_start_macro_timer()
 //=============================================================================
 void trx_start(void)
 {
+#if !BENCHMARK_MODE
 	if (trxrunning) {
-		LOG_ERROR("trx already running!");
+		LOG(debug::ERROR_LEVEL, debug::LOG_MODEM, "trx already running!");
 		return;
 	}
 	
@@ -508,12 +523,13 @@ void trx_start(void)
 	}
 
 	ReedSolomon = new cRsId;
-	
+#endif // !BENCHMARK_MODE
+
 	trx_state = STATE_RX;
 	_trx_tune = 0;
 	active_modem = 0;
-	if (fl_create_thread(trx_thread, trx_loop, &dummy) < 0) {
-		LOG_ERROR("pthread_create failed");
+	if (pthread_create(&trx_thread, NULL, trx_loop, NULL) < 0) {
+		LOG(debug::ERROR_LEVEL, debug::LOG_MODEM, "pthread_create failed");
 		trxrunning = false;
 		exit(1);
 	} 
@@ -539,36 +555,24 @@ void trx_tune(void) { trx_state = STATE_TUNE; }
 void trx_receive(void) { trx_state = STATE_RX; }
 
 //=============================================================================
-void wait_modem_ready_prep(void)
-{
-#ifndef NDEBUG
-        if (GET_THREAD_ID() == TRX_TID)
-		LOG_ERROR("trx thread called wait_modem_ready_prep!");
-#endif
 
-        fl_lock(&trx_cond_mutex);
+void wait_trx_state_prep(void)
+{
+        pthread_mutex_lock(&trx_cond_mutex);
+}
+void wait_trx_state_wait(void)
+{
+        pthread_cond_wait(&trx_cond, &trx_cond_mutex);
+}
+void wait_trx_state_cmpl(void)
+{
+        pthread_mutex_unlock(&trx_cond_mutex);
 }
 
-void wait_modem_ready_cmpl(void)
+static void signal_trx_state(void)
 {
-#ifndef NDEBUG
-        if (GET_THREAD_ID() == TRX_TID)
-                LOG_ERROR("trx thread called wait_modem_ready_cmpl!");
-#endif
-
-        fl_cond_wait(&trx_cond, &trx_cond_mutex);
-        fl_unlock(&trx_cond_mutex);
+	ENSURE_THREAD(TRX_TID);
+        pthread_mutex_lock(&trx_cond_mutex);
+        pthread_cond_broadcast(&trx_cond);
+        pthread_mutex_unlock(&trx_cond_mutex);
 }
-
-void signal_modem_ready(void)
-{
-#ifndef NDEBUG
-        if (GET_THREAD_ID() != TRX_TID)
-		LOG_ERROR("thread %d called signal_modem_ready!", GET_THREAD_ID());
-#endif
-
-        fl_lock(&trx_cond_mutex);
-        fl_cond_bcast(&trx_cond);
-        fl_unlock(&trx_cond_mutex);
-}
-
