@@ -56,7 +56,7 @@ void loadPalette()
 	int r,g,b;
 	FILE *clrfile = NULL;
 	if (palfilename.size() == 0) {
-		palfilename = HomeDir;
+		palfilename = PalettesDir;
 		palfilename.append ("fldigi.pal");
 	}
     const char *p = FSEL::select("Open palette", "Fldigi palette\t*.pal", palfilename.c_str());
@@ -85,6 +85,7 @@ void loadPalette()
 		palLabelStr = "Palette: " + palLabelStr;
 		WF_Palette->label(palLabelStr.c_str());
 		WF_Palette->redraw();
+		progdefaults.PaletteName = palLabelStr;
 	}
 }
 
@@ -92,7 +93,7 @@ void savePalette()
 {
 	FILE *clrfile = NULL;
 	if (palfilename.size() == 0) {
-		palfilename = HomeDir;
+		palfilename = PalettesDir;
 		palfilename.append ("fldigi.pal");
 	}
 	const char *p = FSEL::saveas("Save palette", "Fldigi palette\t*.pal", palfilename.c_str());
@@ -108,5 +109,7 @@ void savePalette()
 		if (pos != string::npos) palLabelStr.erase(0, pos+1);
 		palLabelStr = "Palette: " + palLabelStr;
 		WF_Palette->label(palLabelStr.c_str());
+		WF_Palette->redraw();
+		progdefaults.PaletteName = palLabelStr;
    	}
 }
