@@ -31,6 +31,8 @@
 #include <sstream>
 #include <vector>
 #include <algorithm>
+#include <map>
+
 #include "waterfall.h"
 #include "threads.h"
 #include "main.h"
@@ -47,8 +49,27 @@
 #include "configuration.h"
 #include "Viewer.h"
 #include "macros.h"
+#include "arq_io.h"
+#include "confdialog.h"
+#include "gettext.h"
 
-Fl_Mutex	wf_mutex = PTHREAD_MUTEX_INITIALIZER;
+#define bwFFT		30
+#define cwRef		50
+#define bwX1		25
+#define bwMov		18
+#define bwRate		45
+#define cwCnt		92
+#define bwQsy		32
+#define bwXmtLock	32
+#define bwRev		32
+#define bwMem		40
+#define bwXmtRcv	40
+#define wSpace		1
+
+#define bwdths	(wSpace + bwFFT + wSpace + cwRef + wSpace + cwRef + wSpace + bwX1 + \
+				wSpace + 3*bwMov + wSpace + bwRate + wSpace + \
+				cwCnt + wSpace + bwQsy + wSpace + bwMem + wSpace + \
+				bwXmtLock + wSpace + bwRev + wSpace + bwXmtRcv + wSpace)
 
 extern modem *active_modem;
 
@@ -123,7 +144,7 @@ WFdisp::WFdisp (int x0, int y0, int w0, int h0, char *lbl) :
 	dispcnt = 4;
 	wantcursor = false;
 	cursormoved = false;
-	usebands = false;
+//	usebands = false;
 	for (int i = 0; i < IMAGE_WIDTH; i++)
 		pwr[i] = 0.0;
 
@@ -133,6 +154,9 @@ WFdisp::WFdisp (int x0, int y0, int w0, int h0, char *lbl) :
 	tmp_carrier = false;
 	ptrCB = 0;
 	ptrFFTbuff = 0;
+
+	for (int i = 0; i < 256; i++)
+		mag2RGBI[i].I = mag2RGBI[i].R = mag2RGBI[i].G = mag2RGBI[i].B = 0;
 }
 
 WFdisp::~WFdisp() {
@@ -170,6 +194,8 @@ void WFdisp::makeMarker() {
 				progdefaults.SearchRange;
 		if (mode >= MODE_FELDHELL && mode <= MODE_HELL80)
 			marker_width = (int)progdefaults.HELL_BW;
+        if (mode == MODE_RTTY)
+            marker_width = (int)progdefaults.RTTY_BW;
 	}
 	marker_width = (int)(marker_width / 2.0 + 1);
 
@@ -185,7 +211,7 @@ void WFdisp::makeMarker() {
 	if (bw_upper + static_cast<int>(carrierfreq+0.5) > IMAGE_WIDTH)
             bw_upper -= bw_upper + static_cast<int>(carrierfreq+0.5) - IMAGE_WIDTH;
 	for (int y = 0; y < WFMARKER - 2; y++) {
-		for (int i = bw_lower; i <= bw_upper ; i++) {
+		for (int i = bw_lower; i < bw_upper; i++) {
 			clrPos = clrM + i + y * IMAGE_WIDTH;
 			if (clrPos > clrMin && clrPos < clrMax)
 				*clrPos = RGBmarker;
@@ -280,7 +306,7 @@ void WFdisp::setcolors() {
 
 
 void WFdisp::initmaps() {
-	for (int i = 0; i < image_area; i++) fft_db[i] = log2disp(-1000);
+	for (int i = 0; i < image_area; i++) fft_db[i] = tmp_fft_db[i] = log2disp(-1000);
 
 	memset (fft_img, 0, image_area * sizeof(RGBI) );
 	memset (scaleimage, 0, scale_width * WFSCALE);
@@ -339,6 +365,9 @@ void WFdisp::processFFT() {
 	int		n;
 	double scale;
 	int    ptrSample;
+	if (prefilter != progdefaults.wfPreFilter)
+	    setPrefilter(progdefaults.wfPreFilter);
+	    
 	scale = (double)SC_SMPLRATE / srate;
     scale *= FFT_LEN / 2000.0;
 
@@ -359,7 +388,7 @@ FL_LOCK_D();
 			if (i <= progdefaults.LowFreqCutoff)
 				pw = 0.0;
 			else
-				pw = fftout[n]*fftout[n];
+				pw = fftout[n]*fftout[n] + fftout[n+1]*fftout[n+1];
 			pwr[i] = pw;
 			ffth = (int)(10.0 * log10(pw + 1e-10) );
 			fft_db[ptrFFTbuff * IMAGE_WIDTH + i] = log2disp(ffth);
@@ -413,6 +442,8 @@ FL_LOCK_D();
 	int cbc = ptrCB;
 	for (int c = 0; c < IMAGE_WIDTH; c++) {
 		ynext = (int)(h2 * sig[cbc]);
+		if (ynext < -h2) ynext = -h2;
+		if (ynext > h2) ynext = h2;
 		cbc = (cbc + 1) % (FFT_LEN *2);
 		for (; sigy < ynext; sigy++) sig_img[sigpixel -= IMAGE_WIDTH] = graylevel;
 		for (; sigy > ynext; sigy--) sig_img[sigpixel += IMAGE_WIDTH] = graylevel;
@@ -459,9 +490,9 @@ void WFdisp::sig_data( double *sig, int len, int sr ) {
 
 	static char szFrequency[14];
 	
-	if (usebands)
-		rfc = (long long)(atof(cboBand->value()) * 1000.0);
-	if (rfc != 0) {
+//	if (usebands)
+//		rfc = (long long)(atof(cboBand->value()) * 1000.0);
+	if (rfc != 0) { // use a boolean for the waterfall
 		if (usb)
 			dfreq = rfc + active_modem->get_txfreq();
 		else	
@@ -512,7 +543,7 @@ void WFdisp::movetocenter() {
 }
 
 void WFdisp::carrier(int cf) {
-	if (cf > bandwidth / 2 && cf < (IMAGE_WIDTH - bandwidth / 2)) {
+	if (cf >= bandwidth / 2 && cf < (IMAGE_WIDTH - bandwidth / 2)) {
 		carrierfreq = cf;
 		makeMarker();
 		redrawCursor();
@@ -578,7 +609,9 @@ void WFdisp::drawScale() {
 	double fr;
 	uchar *pixmap;
 	
-	if (usb || !rfc)
+	if (progdefaults.wf_audioscale)
+		pixmap = (scaleimage + (int)offset);
+	else if (usb || !rfc)
 		pixmap = (scaleimage +  (int)((rfc % 1000 + offset)) );
 	else
 		pixmap = (scaleimage + (int)((1000 - rfc % 1000 + offset)));
@@ -595,7 +628,7 @@ void WFdisp::drawScale() {
 	fl_color(fl_rgb_color(228));
 	fl_font(progdefaults.WaterfallFontnbr, progdefaults.WaterfallFontsize);
 	for (int i = 1; i < 10; i++) {
-		if (rfc == 0)
+		if (progdefaults.wf_audioscale)
 			fr = 500.0 * i;
 		else {
 			if (usb)
@@ -603,9 +636,14 @@ void WFdisp::drawScale() {
 			else
 				fr = (rfc - (rfc %500))/1000.0 + 0.5 - 0.5*i;
 		}
-		snprintf(szFreq, sizeof(szFreq), "%7.1f", fr);
+		if (progdefaults.wf_audioscale)
+			snprintf(szFreq, sizeof(szFreq), "%7.0f", fr);
+		else
+			snprintf(szFreq, sizeof(szFreq), "%7.1f", fr);
 		fw = (int)fl_width(szFreq);
-		if (usb)
+		if (progdefaults.wf_audioscale)
+			xchar = (int) (( (1000.0/step) * i - fw) / 2.0 - offset /step );
+		else if (usb)
 			xchar = (int) ( ( (1000.0/step) * i - fw) / 2.0 - 
 							(offset + rfc % 500) /step );
 		else
@@ -632,7 +670,10 @@ void WFdisp::update_waterfall() {
 	short int *p1, *p2;
 	RGBI *p3, *p4;
 	p1 = tmp_fft_db + offset;
+	p2 = p1;
 	p3 = fft_img;
+	p4 = p3;
+	
 	short* limit = tmp_fft_db + image_area - step + 1;
 
 	for (int row = 0; row < image_height; row++) {
@@ -880,6 +921,7 @@ void carrier_cb(Fl_Widget *w, void *v) {
 	Fl_Counter *cntr = (Fl_Counter *)w;
 	waterfall *wf = (waterfall *)w->parent();
 	int selfreq = (int) cntr->value();
+	stopMacroTimer();
 	active_modem->set_freq(selfreq);
 	wf->wfdisp->carrier(selfreq);
 	restoreFocus();
@@ -920,13 +962,8 @@ void qsy_cb(Fl_Widget *w, void *v)
                 }
         }
 
-        if (m.carrier > 0) {
-                rigCAT_set_qsy(m.rfcarrier, m.carrier);
-                rigMEM_set_qsy(m.rfcarrier, m.carrier);
-#if USE_HAMLIB
-                hamlib_set_qsy(m.rfcarrier, m.carrier);
-#endif
-        }
+        if (m.carrier > 0)
+		qsy(m.rfcarrier, m.carrier);
 	restoreFocus();
 }
 
@@ -972,6 +1009,8 @@ void xmtrcv_cb(Fl_Widget *w, void *vi)
 		else {
 			active_modem->set_stopflag(true);
 			TransmitText->clear();
+			if (arq_text_available)
+				AbortARQ();
 			if (progdefaults.useTimer)
 				progdefaults.useTimer = false;
 		}
@@ -996,33 +1035,22 @@ void waterfall::set_XmtRcvBtn(bool val)
 	FL_UNLOCK_D();
 }
 
-void mode_cb(Fl_Widget *w, void *v) {
-	FL_LOCK_D();
-	waterfall *wf = (waterfall *)w->parent();
-	if (Fl::event_shift()) {
-		wf->wfdisp->Mode(SCOPE);
-		w->label("sig");
-		wf->x1->deactivate();
-//		wf->bw_rsid->deactivate();
-		wf->wfcarrier->deactivate();
-		wf->wfRefLevel->deactivate();
-		wf->wfAmpSpan->deactivate();
-	} else if (wf->wfdisp->Mode() == WATERFALL) {
-		wf->wfdisp->Mode(SPECTRUM);
-		w->label("fft");
-	} else if (wf->wfdisp->Mode() == SPECTRUM) {
-		wf->wfdisp->Mode(WATERFALL);
-		w->label("Wtr");
-	} else {
-		wf->wfdisp->Mode(WATERFALL);
-		w->label("Wtr");
-		wf->x1->activate();
-//		wf->bw_rsid->activate();
-		wf->wfcarrier->activate();
-		wf->wfRefLevel->activate();
-		wf->wfAmpSpan->activate();
+void mode_cb(Fl_Widget* w, void*)
+{
+	static const char* names[NUM_WF_MODES] = { "WF", "FFT", "SIG" };
+	int m = wf->wfdisp->Mode() + (Fl::event_button() == FL_LEFT_MOUSE ? 1 : -1);
+	m = WCLAMP(m, WATERFALL, NUM_WF_MODES-1);
+
+	Fl_Widget* b[] = { wf->x1, wf->wfcarrier, wf->wfRefLevel, wf->wfAmpSpan };
+	for (size_t i = 0; i < sizeof(b)/sizeof(*b); i++) {
+		if (m == SCOPE)
+			b[i]->deactivate();
+		else
+			b[i]->activate();
 	}
-	FL_UNLOCK_D();
+
+	wf->wfdisp->Mode(static_cast<WFmode>(m));
+	w->label(names[m]);
 	restoreFocus();
 }
 
@@ -1095,13 +1123,8 @@ void btnMem_cb(Fl_Widget *, void *menu_event)
                 m = qrg_list[elem];
                 if (active_modem != *mode_info[m.mode].modem)
                         init_modem_sync(m.mode);
-                if (m.rfcarrier && m.rfcarrier != wf->rfcarrier()) {
-                        rigCAT_set_qsy(m.rfcarrier, m.carrier);
-                        rigMEM_set_qsy(m.rfcarrier, m.carrier);
-#if USE_HAMLIB
-                        hamlib_set_qsy(m.rfcarrier, m.carrier);
-#endif
-                }
+                if (m.rfcarrier && m.rfcarrier != wf->rfcarrier())
+			qsy(m.rfcarrier, m.carrier);
                 else
                         active_modem->set_freq(m.carrier);
                 break;
@@ -1164,7 +1187,7 @@ void waterfall::opmode() {
 		wfdisp->carrier( IMAGE_WIDTH - val/2);
 	wfdisp->Bandwidth( val );
 	FL_LOCK_D();	
-	wfcarrier->range(val/2-1, IMAGE_WIDTH - val/2-1);
+	wfcarrier->range(val/2, IMAGE_WIDTH - val/2-1);
 	FL_UNLOCK_D();
 }
 
@@ -1269,7 +1292,8 @@ bool waterfall::USB() {
 waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	Fl_Group(x0,y0,w0,h0,lbl) {
 	int xpos;
-	float ratio = w0 < 600 ? w0 / 600.0 : 1.0;
+	float ratio;// = w0 < 600 ? w0 / 600.0 : 1.0;
+	ratio = w0 * 1.0 / bwdths;
 
 	buttonrow = h() + y() - BTN_HEIGHT - BEZEL;
 	bezel = new Fl_Box(
@@ -1284,15 +1308,10 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 			h() - BTN_HEIGHT - 4 * BEZEL);
 	
 	xpos = x() + wSpace;
-//	bw_rsid = new Fl_Button(xpos, buttonrow, (int)(bwColor*ratio), BTN_HEIGHT, "Id?");
-//	bw_rsid->callback(bw_rsid_cb, 0);
-//	bw_rsid->tooltip("Auto detect RSID");
 
-//	xpos = xpos + (int)(bwColor*ratio) + wSpace;
-
-	mode = new Fl_Button(xpos, buttonrow, (int)(bwFFT*ratio), BTN_HEIGHT,"Wtr");
+	mode = new Fl_Button(xpos, buttonrow, (int)(bwFFT*ratio), BTN_HEIGHT, "WF");
 	mode->callback(mode_cb, 0);
-	mode->tooltip("Waterfall/FFT - Shift click for signal scope");
+	mode->tooltip(_("Waterfall / FFT / Scope"));
 
 	xpos = xpos + (int)(bwFFT*ratio) + wSpace;
 	wfRefLevel = new Fl_Counter(xpos, buttonrow, (int)(cwRef*ratio), BTN_HEIGHT );
@@ -1302,7 +1321,7 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	wfRefLevel->range(-40.0, 0.0);
 	wfRefLevel->value(-20.0);
 	wfdisp->Reflevel(-20.0);
-	wfRefLevel->tooltip("Upper signal level (dB)");
+	wfRefLevel->tooltip(_("Upper signal level (dB)"));
 	wfRefLevel->type(FL_SIMPLE_COUNTER);
 
 	xpos = xpos + (int)(cwRef*ratio) + wSpace;
@@ -1313,35 +1332,35 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	wfAmpSpan->range(6.0, 90.0);
 	wfAmpSpan->value(70.0);
 	wfdisp->Ampspan(70.0);
-	wfAmpSpan->tooltip("Signal range (dB)");
+	wfAmpSpan->tooltip(_("Signal range (dB)"));
 	wfAmpSpan->type(FL_SIMPLE_COUNTER);
 
 	xpos = xpos + (int)(cwRef*ratio) + wSpace;
 	x1 = new Fl_Button(xpos, buttonrow, (int)(bwX1*ratio), BTN_HEIGHT, "x1");
 	x1->callback(x1_cb, 0);
-	x1->tooltip("Change waterfall scale");
+	x1->tooltip(_("Change waterfall scale"));
 
 	xpos = xpos + (int)(bwX1*ratio) + wSpace;
 	left = new Fl_Repeat_Button(xpos, buttonrow, (int)(bwMov*ratio), BTN_HEIGHT, "@<");
 	left->callback(slew_left, 0);
-	left->tooltip("Slew display lower in freq");
+	left->tooltip(_("Slew display lower in frequency"));
 
-	xpos += (int)(bwMov*ratio);
+	xpos = xpos + (int)(bwMov*ratio);
 	center = new Fl_Button(xpos, buttonrow, (int)(bwMov*ratio), BTN_HEIGHT, "@||");
 	center->callback(center_cb, 0);
-	center->tooltip("Center display on signal");
+	center->tooltip(_("Center display on signal"));
 
-	xpos += (int)(bwMov*ratio);
+	xpos = xpos + (int)(bwMov*ratio);
 	right = new Fl_Repeat_Button(xpos, buttonrow, (int)(bwMov*ratio), BTN_HEIGHT, "@>");
 	right->callback(slew_right, 0);
-	right->tooltip("Slew display higher in freq");
+	right->tooltip(_("Slew display higher in frequency"));
 
 	xpos = xpos + (int)(bwMov*ratio) + wSpace;
 	wfrate = new Fl_Button(xpos, buttonrow, (int)(bwRate*ratio), BTN_HEIGHT, "Norm");
 	wfrate->callback(rate_cb, 0);
-	wfrate->tooltip("Waterfall drop speed");
+	wfrate->tooltip(_("Waterfall drop speed"));
 
-	xpos = xpos + (int)(bwRate*ratio) + 2*wSpace;
+	xpos = xpos + (int)(bwRate*ratio) + wSpace;
 	wfcarrier = new Fl_Counter(xpos, buttonrow, (int)(cwCnt*ratio), BTN_HEIGHT );
 	wfcarrier->callback(carrier_cb, 0);
 	wfcarrier->step(1.0);
@@ -1349,35 +1368,35 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	wfcarrier->precision(0);
 	wfcarrier->range(16.0, IMAGE_WIDTH - 16.0);
 	wfcarrier->value(wfdisp->carrier());
-	wfcarrier->tooltip("Adjust cursor frequency");
+	wfcarrier->tooltip(_("Adjust cursor frequency"));
 
-	xpos = xpos + (int)(cwCnt*ratio) + 2*wSpace;
+	xpos = xpos + (int)(cwCnt*ratio) + wSpace;
 	qsy = new Fl_Button(xpos, buttonrow, (int)(bwQsy*ratio), BTN_HEIGHT, "QSY");
 	qsy->callback(qsy_cb, 0);
-	qsy->tooltip("Cntr in Xcvr PB\nRight click to undo");
+	qsy->tooltip(_("Center in passband\nRight click to undo"));
 	qsy->deactivate();
 
 	xpos = xpos + (int)(bwQsy*ratio) + wSpace;
 	btnMem = new Fl_Button(xpos, buttonrow, (int)(bwMem*ratio), BTN_HEIGHT, "Store");
 	btnMem->callback(btnMem_cb, 0);
-	btnMem->tooltip("Store mode and frequency\nRight click for list");
+	btnMem->tooltip(_("Store mode and frequency\nRight click for list"));
 	mbtnMem = new Fl_Menu_Button(btnMem->x(), btnMem->y(), btnMem->w(), btnMem->h(), 0);
 	mbtnMem->callback(btnMem->callback(), mbtnMem);
 	mbtnMem->type(Fl_Menu_Button::POPUP3);
 
-	xpos = xpos + (int)(bwMem*ratio) + 2*wSpace;
+	xpos = xpos + (int)(bwMem*ratio) + wSpace;
 	xmtlock = new Fl_Light_Button(xpos, buttonrow, (int)(bwXmtLock*ratio), BTN_HEIGHT, "Lk");
 	xmtlock->callback(xmtlock_cb, 0);
 	xmtlock->value(0);
 	xmtlock->selection_color(FL_RED);
-	xmtlock->tooltip("Xmt freq locked");
+	xmtlock->tooltip(_("Lock transmit frequency"));
 
 	xpos = xpos + (int)(bwXmtLock*ratio) + wSpace;
 	btnRev = new Fl_Light_Button(xpos, buttonrow, (int)(bwRev*ratio), BTN_HEIGHT, "Rv");
 	btnRev->callback(btnRev_cb, 0);
 	btnRev->value(0);
 	btnRev->selection_color(FL_GREEN);
-	btnRev->tooltip("Reverse");
+	btnRev->tooltip(_("Reverse"));
 	reverse = false;
 
 	xpos = w() - (int)(bwXmtRcv*ratio) - wSpace;
@@ -1385,7 +1404,9 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	xmtrcv->callback(xmtrcv_cb, 0);
 	xmtrcv->selection_color(FL_RED);
 	xmtrcv->value(0);
-	xmtrcv->tooltip("Transmit/Receive");
+	xmtrcv->tooltip(_("Transmit/Receive"));
+	
+	end();
 }
 
 int waterfall::handle(int event)
@@ -1400,7 +1421,8 @@ int waterfall::handle(int event)
  	Fl_Valuator* v[] = { sldrSquelch, wfcarrier, wfRefLevel, wfAmpSpan, valRcvMixer, valXmtMixer };
   	for (size_t i = 0; i < sizeof(v)/sizeof(v[0]); i++) {
   		if (Fl::event_inside(v[i])) {
- 			if ((v[i] == sldrSquelch && !twoscopes) || v[i] == valRcvMixer || v[i] == valXmtMixer)
+ 			if ((v[i] == sldrSquelch && !progdefaults.docked_scope) ||
+			    v[i] == valRcvMixer || v[i] == valXmtMixer)
 				d = -d;
 			v[i]->value(v[i]->clamp(v[i]->increment(v[i]->value(), -d)));
 			v[i]->do_callback();
@@ -1439,18 +1461,116 @@ static void hide_cursor(void *w)
 		reinterpret_cast<Fl_Widget *>(w)->window()->cursor(cursor = FL_CURSOR_NONE);
 }
 
+map<string, qrg_mode_t> qsy_map;
+static qrg_mode_t last;
+
+void note_qrg(bool check, char prefix, char suffix)
+{
+	qrg_mode_t m;
+	m.rfcarrier = wf->rfcarrier();
+	m.carrier = active_modem->get_freq();
+	m.mode = active_modem->get_mode();
+	if (check && last == m)
+		return;
+	last = m;
+
+	char buf[64];
+
+	time_t t = time(NULL);
+	struct tm tm;
+	gmtime_r(&t, &tm);
+	size_t r1;
+	if ((r1 = strftime(buf, sizeof(buf), "<<%FT%H:%MZ ", &tm)) == 0)
+		return;
+
+	size_t r2;
+	if (m.rfcarrier)
+		r2 = snprintf(buf+r1, sizeof(buf)-r1, "%s @ %lld%c%04d>>",
+			     mode_info[m.mode].name, m.rfcarrier, (wf->USB() ? '+' : '-'), m.carrier);
+	else
+		r2 = snprintf(buf+r1, sizeof(buf)-r1, "%s @ %04d>>", mode_info[m.mode].name, m.carrier);
+	if (r2 >= sizeof(buf)-r1)
+		return;
+
+	qsy_map[buf] = m;
+	ReceiveText->add(prefix);
+	ReceiveText->add(buf, FTextBase::QSY);
+	ReceiveText->add(suffix);
+}
+
+static void insert_text(bool check = false)
+{
+	if (check) {
+		qrg_mode_t m;
+		m.rfcarrier = wf->rfcarrier();
+		m.carrier = active_modem->get_freq();
+		m.mode = active_modem->get_mode();
+		if (last.mode == m.mode && last.rfcarrier == m.rfcarrier &&
+		    abs(last.carrier - m.carrier) <= 16)
+			return;
+		last = m;
+	}
+
+	string::size_type i;
+	if ((i = progdefaults.WaterfallClickText.find("<FREQ>")) != string::npos) {
+		string s = progdefaults.WaterfallClickText;
+		s[i] = '\0';
+		ReceiveText->add(s.c_str());
+		note_qrg(false);
+		ReceiveText->add(s.c_str() + i + strlen("<FREQ>"));
+	}
+	else
+		ReceiveText->add(progdefaults.WaterfallClickText.c_str(), FTextView::SKIP);
+}
+
+static void find_signal_text(void)
+{
+	int freq = active_modem->get_freq();
+	trx_mode mode = active_modem->get_mode();
+
+	map<string, qrg_mode_t>::const_iterator i;
+	for (i = qsy_map.begin(); i != qsy_map.end(); ++i)
+		if (i->second.mode == mode && abs(i->second.carrier - freq) <= 20)
+			break;
+	if (i != qsy_map.end()) {
+		// Search backward from the current text cursor position, then
+		// try the other direction
+		int pos = ReceiveText->insert_position();
+		if (ReceiveText->buffer()->search_backward(pos, i->first.c_str(), &pos, 1) ||
+		    ReceiveText->buffer()->search_forward(pos, i->first.c_str(), &pos, 1)) {
+			ReceiveText->insert_position(pos);
+			ReceiveText->show_insert_position();
+		}
+	}
+}
+
 int WFdisp::handle(int event)
 {
-	if (!(event == FL_LEAVE || Fl::event_inside(this)))
+	static int pxpos, push;
+	if (!(event == FL_LEAVE || Fl::event_inside(this))) {
+		if (event == FL_RELEASE)
+			push = 0;
 		return 0;
+	}
 
 	if (trx_state != STATE_RX)
 		return 1;
 	int xpos = Fl::event_x() - x();
+	int ypos = Fl::event_y() - y();
 	int eb;
 
 	switch (event) {
 	case FL_MOVE:
+		if (progdefaults.WaterfallQSY && ypos < WFTEXT + WFSCALE) {
+			Fl::remove_timeout(hide_cursor, this);
+			if (cursor != FL_CURSOR_WE)
+				window()->cursor(cursor = FL_CURSOR_WE);
+			if (wantcursor) {
+				wantcursor = false;
+				makeMarker();
+			}
+			break;
+		}
 		if (cursor != FL_CURSOR_DEFAULT)
 			window()->cursor(cursor = FL_CURSOR_DEFAULT);
 		if (!Fl::has_timeout(hide_cursor, this))
@@ -1461,6 +1581,8 @@ int WFdisp::handle(int event)
 		redrawCursor();
 		break;
 	case FL_DRAG: case FL_PUSH:
+		stopMacroTimer();
+	
 		switch (eb = Fl::event_button()) {
 		case FL_RIGHT_MOUSE:
 			wantcursor = false;
@@ -1470,8 +1592,24 @@ int WFdisp::handle(int event)
 				if (progdefaults.WaterfallHistoryDefault)
 					bHistory = true;
 			}
-			// fall through
+			goto lrclick;
 		case FL_LEFT_MOUSE:
+			if (event == FL_PUSH) {
+				push = ypos;
+				pxpos = xpos;
+				if (Fl::event_clicks())
+					return 1;
+			}
+			if (progdefaults.WaterfallQSY && push < WFTEXT + WFSCALE) {
+				long long newrfc = (pxpos - xpos) * step;
+				if (!USB())
+					newrfc = -newrfc;
+				newrfc += rfcarrier();
+				qsy(newrfc, active_modem->get_freq());
+				pxpos = xpos;
+				return 1;
+			}
+		lrclick:
 			if (Fl::event_state() & FL_CTRL) {
 				if (event == FL_DRAG)
 					break;
@@ -1485,6 +1623,7 @@ int WFdisp::handle(int event)
 			if (progdefaults.WaterfallHistoryDefault)
 				bHistory = true;
 			newcarrier = cursorFreq(xpos);
+			newcarrier = (int)CLAMP(newcarrier, wf->wfcarrier->minimum(), wf->wfcarrier->maximum());
 			active_modem->set_freq(newcarrier);
 			if (!(Fl::event_state() & FL_SHIFT))
 				active_modem->set_sigsearch(SIGSEARCH);
@@ -1503,7 +1642,7 @@ int WFdisp::handle(int event)
 		}
 		break;
 	case FL_RELEASE:
-		switch (Fl::event_button()) {
+		switch (eb = Fl::event_button()) {
 		case FL_RIGHT_MOUSE:
 			tmp_carrier = false;
 			active_modem->set_freq(oldcarrier);
@@ -1511,35 +1650,62 @@ int WFdisp::handle(int event)
 			restoreFocus();
 			// fall through
 		case FL_LEFT_MOUSE:
+			push = 0;
 			oldcarrier = newcarrier;
+			if (eb != FL_LEFT_MOUSE || !ReceiveText->visible())
+				break;
+			if (!(Fl::event_state() & (FL_CTRL | FL_META | FL_ALT | FL_SHIFT))) {
+				if (Fl::event_clicks() == 1)
+					note_qrg(true, '\n', '\n');
+				else
+					if (progdefaults.WaterfallClickInsert)
+						insert_text(true);
+			}
+			else if (Fl::event_state() & (FL_META | FL_ALT))
+				find_signal_text();
 			break;
 		}
 		break;
 
 	case FL_MOUSEWHEEL:
-		change_modem_param(Fl::event_state());
+	{
+		stopMacroTimer();
+	
+		int d;
+		if ( !((d = Fl::event_dy()) || (d = Fl::event_dx())) )
+			break;
+		int state = Fl::event_state();
+		if (state & FL_CTRL)
+			wf->handle_mouse_wheel(waterfall::WF_AFC_BW, d);
+		else if (state & (FL_META | FL_ALT))
+			wf->handle_mouse_wheel(waterfall::WF_SIGNAL_SEARCH, d);
+		else if (state & FL_SHIFT)
+			wf->handle_mouse_wheel(waterfall::WF_SQUELCH, d);
+		else {
+			if (progdefaults.WaterfallQSY && Fl::event_inside(x(), y(), w(), WFTEXT+WFSCALE+WFMARKER))
+				qsy(wf->rfcarrier() - 500*d);
+			else
+				wf->handle_mouse_wheel(progdefaults.WaterfallWheelAction, d);
+		}
 		return handle(FL_MOVE);
-		break;
-
+	}
 	case FL_SHORTCUT:
 		if (Fl::event_inside(this))
 			take_focus();
 		break;
 	case FL_KEYBOARD:
 	{
+		stopMacroTimer();
+	
 		int d = (Fl::event_state() & FL_CTRL) ? 10 : 1;
-		switch (Fl::event_key()) {
-		case FL_Left:
-			if (xpos > 0) {
-				active_modem->set_freq(oldcarrier = newcarrier = carrier() - d);
-				redrawCursor();
-			}
-			break;
-		case FL_Right:
-			if (xpos < w()) {
-				active_modem->set_freq(oldcarrier = newcarrier = carrier() + d);
-				redrawCursor();
-			}
+		int k = Fl::event_key();
+		switch (k) {
+		case FL_Left: case FL_Right:
+			if (k == FL_Left)
+				d = -d;
+			oldcarrier = newcarrier = (int)CLAMP(carrier() + d, wf->wfcarrier->minimum(), wf->wfcarrier->maximum());
+			active_modem->set_freq(newcarrier);
+			redrawCursor();
 			break;
 		case FL_Tab:
 			restoreFocus();
@@ -1572,3 +1738,75 @@ int WFdisp::handle(int event)
 
 	return 1;
 }
+
+void waterfall::handle_mouse_wheel(int what, int d)
+{
+	if (d == 0)
+		return;
+
+	Fl_Valuator *val = 0;
+	const char* msg_fmt = 0, *msg_label = 0;
+
+	switch (what) {
+	case WF_NOP:
+		return;
+	case WF_AFC_BW:
+	{
+		trx_mode m = active_modem->get_mode();
+		if (m >= MODE_PSK_FIRST && m <= MODE_PSK_LAST)
+			val = mailserver ? cntServerOffset : cntSearchRange;
+		else if (m >= MODE_HELL_FIRST && m <= MODE_HELL_LAST)
+			val = sldrHellBW;
+		else if (m == MODE_CW)
+			val = sldrCWbandwidth;
+        else if (m == MODE_RTTY)
+            val = sldrRTTYbandwidth;
+		else
+			return;
+		msg_fmt = "%s = %2.0f Hz";
+		msg_label = val->label();
+		break;
+	}
+	case WF_SIGNAL_SEARCH:
+		if (d > 0)
+			active_modem->searchDown();
+		else
+			active_modem->searchUp();
+		return;
+	case WF_SQUELCH:
+		val = sldrSquelch;
+		if (!progdefaults.docked_scope)
+			d = -d;
+		msg_fmt = "%s = %2.0f %%";
+		msg_label = "Squelch";
+		break;
+	case WF_CARRIER:
+		val = wfcarrier;
+		break;
+	case WF_MODEM:
+		init_modem(d > 0 ? MODE_NEXT : MODE_PREV);
+		return;
+	case WF_SCROLL:
+		(d > 0 ? right : left)->do_callback();
+		return;
+	}
+
+	val->value(val->clamp(val->increment(val->value(), -d)));
+	bool changed_save = progdefaults.changed;
+	val->do_callback();
+	progdefaults.changed = changed_save;
+	if (val == cntServerOffset || val == cntSearchRange)
+		active_modem->set_sigsearch(SIGSEARCH);
+	else if (val == sldrSquelch) // sldrSquelch gives focus to TransmitText
+		take_focus();
+
+	if (msg_fmt) {
+		char msg[60];
+		snprintf(msg, sizeof(msg), msg_fmt, msg_label, val->value());
+		put_status(msg, 2.0);
+	}
+}
+
+const char waterfall::wf_wheel_action[] = "None|AFC range or BW|"
+	                                  "Signal search|Squelch level|"
+	                                  "Modem carrier|Modem|Scroll";
