@@ -1,9 +1,12 @@
 // ----------------------------------------------------------------------------
 //      FTextView.cxx
 //
-// Copyright (C) 2007
+// Copyright (C) 2007-2008
 //              Stelios Bounanos, M0GLD
 //
+// Copyright (C) 2008
+//              Dave Freese, W1HKJ
+//
 // This file is part of fldigi.
 //
 // fldigi is free software; you can redistribute it and/or modify
@@ -25,6 +28,11 @@
 #include <cstring>
 #include <cstdlib>
 #include <cstdio>
+#include <map>
+#include <sstream>
+#include <algorithm>
+
+#include <FL/Fl_Tooltip.H>
 
 #include "FTextView.h"
 #include "main.h"
@@ -35,12 +43,24 @@
 #include "cw.h"
 
 #include "fileselect.h"
+#include "font_browser.h"
 
 #include "ascii.h"
 #include "configuration.h"
 #include "qrunner.h"
 
 #include "mfsk.h"
+#include "icons.h"
+#include "globals.h"
+#include "re.h"
+#include "dxcc.h"
+#include "locator.h"
+#include "logsupport.h"
+#include "status.h"
+#include "gettext.h"
+
+#include "debug.h"
+
 
 
 using namespace std;
@@ -57,6 +77,12 @@ FTextBase::FTextBase(int x, int y, int w, int h, const char *l)
 	: Fl_Text_Editor_mod(x, y, w, h, l),
           wrap(true), wrap_col(80), max_lines(0), scroll_hint(false)
 {
+	oldw = olds = -1;
+	oldf = (Fl_Font)-1;
+	textfont(FL_COURIER);
+	textsize(FL_NORMAL_SIZE);
+	textcolor(FL_FOREGROUND_COLOR);
+
 	tbuf = new Fl_Text_Buffer;
 	sbuf = new Fl_Text_Buffer;
 
@@ -90,6 +116,30 @@ int FTextBase::handle(int event)
 void FTextBase::setFont(Fl_Font f, int attr)
 {
 	set_style(attr, f, textsize(), textcolor(), SET_FONT);
+
+	if (Font_Browser::fixed_width(f))
+		return;
+
+	// Check if we should generate a warning message
+	bool* warn = 0;
+	const char* fn = Fl::get_font_name(f);
+	if (this == ReceiveText) {
+		warn = &progdefaults.RxFontWarn;
+		if (progdefaults.RxFontName != fn)
+			*warn = true;
+	}
+	else if (this == TransmitText) {
+		warn = &progdefaults.TxFontWarn;
+		if (progdefaults.TxFontName != fn)
+			*warn = true;
+	}
+	if (warn && *warn) {
+		add(Fl::get_font_name(f), XMIT);
+		add(" is a variable width font.\n", XMIT);
+		add("Line wrapping with a variable width font may be\n"
+		    "too slow. Consider using a fixed width font.\n\n", XMIT);
+		*warn = false;
+	}
 }
 
 void FTextBase::setFontSize(int s, int attr)
@@ -114,8 +164,11 @@ void FTextBase::setFontColor(Fl_Color c, int attr)
 ///
 void FTextBase::resize(int X, int Y, int W, int H)
 {
-	reset_wrap_col();
-
+	// do we need to recalculate the wrap column?
+	if (unlikely(text_area.w != oldw || textfont() != oldf || textsize() != olds)) {
+		oldw = text_area.w; oldf = textfont(); olds = textsize();
+		reset_wrap_col();
+	}
         if (scroll_hint) {
                 mTopLineNumHint = mNBufferLines;
                 mHorizOffsetHint = 0;
@@ -160,7 +213,7 @@ void FTextBase::set_style(int attr, Fl_Font f, int s, Fl_Color c, int set)
 		start = 0;
 		end = NATTR;
 		if (set & SET_FONT)
-			textfont(f);
+			Fl_Text_Display_mod::textfont(f);
 		if (set & SET_SIZE)
 			textsize(s);
 		if (set & SET_COLOR)
@@ -186,6 +239,9 @@ void FTextBase::set_style(int attr, Fl_Font f, int s, Fl_Color c, int set)
 			else
 				styles[i].color = c;
 		}
+		if (i == SKIP) // clickable styles always same as SKIP for now
+			for (int j = CLICK_START; j < NATTR; j++)
+				memcpy(&styles[j], &styles[i], sizeof(styles[j]));
 	}
 
 	resize(x(), y(), w(), h()); // to redraw and recalculate the wrap column
@@ -196,7 +252,7 @@ void FTextBase::set_style(int attr, Fl_Font f, int s, Fl_Color c, int set)
 /// @return 0 on success, -1 on error
 int FTextBase::readFile(const char* fn)
 {
-	if ( !(fn || (fn = FSEL::select("Insert text", "Text\t*.txt"))) )
+	if ( !(fn || (fn = FSEL::select(_("Insert text"), "Text\t*.txt"))) )
 		return -1;
 
 	int ret = 0, pos = insert_position();
@@ -249,7 +305,7 @@ int FTextBase::readFile(const char* fn)
 ///
 void FTextBase::saveFile(void)
 {
- 	const char *fn = FSEL::saveas("Save text as", "Text\t*.txt");
+ 	const char *fn = FSEL::saveas(_("Save text as"), "Text\t*.txt");
 	if (fn) {
 #ifdef __CYGWIN__
 		ofstream tfile(fn);
@@ -276,27 +332,42 @@ void FTextBase::saveFile(void)
 	}
 }
 
-/// Returns a character string containing the selected text, if any,
+/// Returns a character string containing the selected word, if any,
 /// or the word at (\a x, \a y) relative to the widget's \c x() and \c y().
+/// If \a ontext is true, this function will return text only if the
+/// mouse cursor position is inside the text range.
 ///
 /// @param x 
 /// @param y 
 ///
 /// @return The selection, or the word text at (x,y). <b>Must be freed by the caller</b>.
 ///
-char *FTextBase::get_word(int x, int y)
+char* FTextBase::get_word(int x, int y, bool ontext)
 {
-	if (!tbuf->selected()) {
-		int p = xy_to_position(x + this->x(), y + this->y(),
-				       Fl_Text_Display_mod::CURSOR_POS);
-		tbuf->select(word_start(p), word_end(p));
-	}
-	char *s = tbuf->selection_text();
-	tbuf->unselect();
+	int p = xy_to_position(x + this->x(), y + this->y(), Fl_Text_Display_mod::CURSOR_POS);
+	int start, end;
 
-	return s;
+	if (tbuf->selected()) {
+		if (ontext && tbuf->selection_position(&start, &end) && (p < start || p >= end))
+			return 0;
+		else
+			return tbuf->selection_text();
+	}
+
+	if (!tbuf->findchars_backward(p, " \t\n", &start))
+		start = 0;
+	else
+		start++;
+	if (!tbuf->findchars_forward(p, " .,;|\t\n", &end))
+		end = tbuf->length();
+
+	if (ontext && (p < start || p >= end))
+		return 0;
+	else
+		return tbuf->text_range(start, end);
 }
 
+
 /// Displays the menu pointed to by \c context_menu and calls the menu function;
 /// @see call_cb.
 ///
@@ -310,29 +381,31 @@ void FTextBase::show_context_menu(void)
 	popy = ypos - y();
 	window()->cursor(FL_CURSOR_DEFAULT);
 	m = context_menu->popup(xpos, ypos, 0, 0, 0);
-	if (m)
+	if (m) {
 		for (int i = 0; i < context_menu->size(); ++i) {
-			if (m->text == context_menu[i].text) {
+			if (m == &context_menu[i]) {
 				menu_cb(i);
 				break;
-			}	
+			}
 		}
-	restoreFocus();
+	}
 }
 
 /// Recalculates the wrap margin when the font is changed or the widget resized.
-/// At the moment we only handle constant width fonts. Using proportional fonts
-/// will result in a small amount of unused space at the end of each line.
+/// Line wrapping works with proportional fonts but may be very slow.
 ///
 int FTextBase::reset_wrap_col(void)
 {
-	if (!wrap || wrap_col == 0 || text_area.w == 0)
+	if (!wrap || text_area.w == 0)
 		return wrap_col;
 
 	int old_wrap_col = wrap_col;
-
-	fl_font(textfont(), textsize());
-	wrap_col = (int)floor(text_area.w / fl_width('X'));
+	if (Font_Browser::fixed_width(textfont())) {
+		fl_font(textfont(), textsize());
+		wrap_col = (int)floorf(text_area.w / fl_width('X'));
+	}
+	else // use slower (but accurate) wrapping for variable width fonts
+		wrap_col = 0;
 	// wrap_mode triggers a resize; don't call it if wrap_col hasn't changed
 	if (old_wrap_col != wrap_col)
 		wrap_mode(wrap, wrap_col);
@@ -342,33 +415,48 @@ int FTextBase::reset_wrap_col(void)
 
 void FTextBase::reset_styles(int set)
 {
-	set_style(NATTR, FL_SCREEN, FL_NORMAL_SIZE, FL_FOREGROUND_COLOR, set);
-	set_style(XMIT, FL_SCREEN, FL_NORMAL_SIZE, FL_RED, set);
-	set_style(CTRL, FL_SCREEN, FL_NORMAL_SIZE, FL_DARK_GREEN, set);
-	set_style(SKIP, FL_SCREEN, FL_NORMAL_SIZE, FL_BLUE, set);
-	set_style(ALTR, FL_SCREEN, FL_NORMAL_SIZE, FL_DARK_MAGENTA, set);
+	set_style(NATTR, FL_COURIER, FL_NORMAL_SIZE, FL_FOREGROUND_COLOR, set);
+	set_style(XMIT, FL_COURIER, FL_NORMAL_SIZE, FL_RED, set);
+	set_style(CTRL, FL_COURIER, FL_NORMAL_SIZE, FL_DARK_GREEN, set);
+	set_style(SKIP, FL_COURIER, FL_NORMAL_SIZE, FL_BLUE, set);
+	set_style(ALTR, FL_COURIER, FL_NORMAL_SIZE, FL_DARK_MAGENTA, set);
 }
 
 // ----------------------------------------------------------------------------
 
+#if !USE_IMAGE_LABELS
+#  define LOOKUP_SYMBOL "@-4>> "
+#  define ENTER_SYMBOL  "@-9-> "
+#else
+#  define LOOKUP_SYMBOL
+#  define ENTER_SYMBOL
+#endif
 
 Fl_Menu_Item FTextView::view_menu[] = {
-	{ "@-4>> &Look up call",		0, 0 },
-	{ "@-9-> &Call",			0, 0 },
-	{ "@-9-> &Name",			0, 0 },
-	{ "@-9-> QT&H",				0, 0 },
-	{ "@-9-> &Locator",			0, 0 },
-	{ "@-9-> &RST(r)",			0, 0, 0, FL_MENU_DIVIDER },
-	{ "Insert divider",			0, 0 },
-	{ "C&lear",				0, 0 },
-	{ "&Copy",				0, 0, 0, FL_MENU_DIVIDER },
+	{ make_icon_label(LOOKUP_SYMBOL _("&Look up call"), net_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("&Call"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("&Name"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("QT&H"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("&State"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("&Province"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("Countr&y"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("&Locator"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("&RST(r)"), enter_key_icon), 0, 0, 0,  FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("Serial number"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(ENTER_SYMBOL _("Exchange In"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Insert divider"), insert_link_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("&Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("C&lear"), edit_clear_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Select All"), edit_select_all_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 #if 0 //#ifndef NDEBUG
-        { "(debug) &Append file...",		0, 0, 0, FL_MENU_DIVIDER },
+        { "(debug) &Append file...", 0, 0, 0, FL_MENU_DIVIDER, FL_NORMAL_LABEL },
 #endif
-	{ "Save to &file...",			0, 0, 0, FL_MENU_DIVIDER },
-	{ "Word &wrap",				0, 0, 0, FL_MENU_TOGGLE	 },
+	{ make_icon_label(_("Save &as..."), save_as_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ _("Quick entry"),      0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL },
+	{ _("Word &wrap"),       0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL },
 	{ 0 }
 };
+static bool view_init = false;
 
 /// FTextView constructor.
 /// We remove \c Fl_Text_Display_mod::buffer_modified_cb from the list of callbacks
@@ -380,16 +468,25 @@ Fl_Menu_Item FTextView::view_menu[] = {
 /// @param h 
 /// @param l 
 FTextView::FTextView(int x, int y, int w, int h, const char *l)
-        : FTextBase(x, y, w, h, l)
+        : FTextBase(x, y, w, h, l), quick_entry(false)
 {
 	tbuf->remove_modify_callback(buffer_modified_cb, this);
 	tbuf->add_modify_callback(changed_cb, this);
+	tbuf->canUndo(0);
 
 	cursor_style(Fl_Text_Display_mod::NORMAL_CURSOR);
 
 	context_menu = view_menu;
 	// disable some keybindings that are not allowed in FTextView buffers
 	change_keybindings();
+
+	if (!view_init) {
+		for (int i = 0; i < view_menu->size() - 1; i++)
+			if (view_menu[i].labeltype() == _FL_MULTI_LABEL)
+				set_icon_label(&view_menu[i]);
+		view_menu[RX_MENU_QUICK_ENTRY].clear();
+	}
+	view_init = true;
 }
 
 FTextView::~FTextView()
@@ -409,48 +506,55 @@ FTextView::~FTextView()
 ///
 int FTextView::handle(int event)
 {
+	static Fl_Cursor cursor;
+
 	switch (event) {
 	case FL_PUSH:
 		if (!Fl::event_inside(this))
 			break;
  		switch (Fl::event_button()) {
- 		case FL_LEFT_MOUSE: {
- 			if (!Fl::event_shift())
- 				goto out;
- 			char* s = get_word(Fl::event_x() - x(), Fl::event_y() - y());
-  			inpCall->value(s);
-  			free(s);
-  			stopMacroTimer();
-  		}
-// fall through
- 		case FL_MIDDLE_MOUSE:
-// stop mouse2 text paste events from reaching Fl_Text_Editor_mod
- 			return 1;
+		case FL_LEFT_MOUSE:
+			if (Fl::event_shift() || (unlikely(Fl::event_clicks()) && progdefaults.rxtext_clicks_qso_data)) {
+				handle_qso_data(Fl::event_x() - x(), Fl::event_y() - y());
+				return 1;
+			}
+			goto out;
+		case FL_MIDDLE_MOUSE:
+			if (cursor != FL_CURSOR_HAND)
+				handle_qso_data(Fl::event_x() - x(), Fl::event_y() - y());
+			return 1;
  		case FL_RIGHT_MOUSE:
-  			break;
+			handle_context_menu();
+			return 1;
  		default:
  			goto out;
  		}
-// enable/disable menu items
-		if (tbuf->length())
-			view_menu[RX_MENU_CLEAR].flags &= ~FL_MENU_INACTIVE;
-		else
-			view_menu[RX_MENU_CLEAR].flags |= FL_MENU_INACTIVE;
-
-		if (tbuf->selected())
-			view_menu[RX_MENU_COPY].flags &= ~FL_MENU_INACTIVE;
-		else
-			view_menu[RX_MENU_COPY].flags |= FL_MENU_INACTIVE;
-		if (wrap)
-			view_menu[RX_MENU_WRAP].flags |= FL_MENU_VALUE;
-		else
-			view_menu[RX_MENU_WRAP].flags &= ~FL_MENU_VALUE;
-		context_menu = progdefaults.QRZ ? view_menu : view_menu + 1;
-
-		show_context_menu();
-		return 1;
 		break;
-		// catch some text-modifying events that are not handled by kf_* functions
+	case FL_DRAG:
+		if (Fl::event_button() != FL_LEFT_MOUSE)
+			return 1;
+		break;
+	case FL_RELEASE:
+		{	int eb = Fl::event_button();
+			if (cursor == FL_CURSOR_HAND && eb == FL_LEFT_MOUSE &&
+			    Fl::event_is_click() && !Fl::event_clicks()) {
+				handle_clickable(Fl::event_x() - x(), Fl::event_y() - y());
+				return 1;
+			}
+			break;
+		}
+	case FL_MOVE: {
+		int p = xy_to_position(Fl::event_x(), Fl::event_y(), Fl_Text_Display_mod::CURSOR_POS);
+		if (sbuf->character(p) >= CLICK_START + FTEXT_DEF) {
+			if (cursor != FL_CURSOR_HAND)
+				window()->cursor(cursor = FL_CURSOR_HAND);
+			return 1;
+		}
+		else
+			cursor = FL_CURSOR_INSERT;
+		break;
+	}
+	// catch some text-modifying events that are not handled by kf_* functions
 	case FL_KEYBOARD:
 		int k;
 		if (Fl::compose(k))
@@ -460,6 +564,22 @@ int FTextView::handle(int event)
 			return 1;
 		else if (k == FL_Tab)
 		    return Fl_Widget::handle(event);
+	case FL_ENTER:
+		if (!progdefaults.rxtext_tooltips || Fl_Tooltip::delay() == 0.0f)
+			break;
+		tooltips.enabled = Fl_Tooltip::enabled();
+		tooltips.delay = Fl_Tooltip::delay();
+		Fl_Tooltip::enable(1);
+		Fl_Tooltip::delay(0.0f);
+		Fl::add_timeout(tooltips.delay / 2.0, dxcc_tooltip, this);
+		break;
+	case FL_LEAVE:
+		if (!progdefaults.rxtext_tooltips || Fl_Tooltip::delay() != 0.0f)
+			break;
+		Fl_Tooltip::enable(tooltips.enabled);
+		Fl_Tooltip::delay(tooltips.delay);
+		Fl::remove_timeout(dxcc_tooltip, this);
+		break;
 	}
 
 out:
@@ -518,65 +638,186 @@ void FTextView::add(unsigned char c, int attr)
 	FL_AWAKE_D();
 }
 
+void FTextView::clear(void)
+{
+	FTextBase::clear();
+	extern map<string, qrg_mode_t> qsy_map;
+	qsy_map.clear();
+}
+
+void FTextView::handle_clickable(int x, int y)
+{
+	int pos, style;
+
+	pos = xy_to_position(x + this->x(), y + this->y(), Fl_Text_Display_mod::CURSOR_POS);
+	// return unless clickable style
+	if ((style = sbuf->character(pos)) < CLICK_START + FTEXT_DEF)
+		return;
+
+	int start, end;
+	for (start = pos-1; start >= 0; start--)
+		if (sbuf->character(start) != style)
+			break;
+	start++;
+	int len = sbuf->length();
+	for (end = pos+1; end < len; end++)
+		if (sbuf->character(end) != style)
+			break;
+
+	switch (style - FTEXT_DEF) {
+	case QSY:
+		handle_qsy(start, end);
+		break;
+	// ...
+	default:
+		break;
+	}
+}
+
+void FTextView::handle_qsy(int start, int end)
+{
+	char* text = tbuf->text_range(start, end);
+
+	extern map<string, qrg_mode_t> qsy_map;
+	map<string, qrg_mode_t>::const_iterator i;
+	if ((i = qsy_map.find(text)) != qsy_map.end()) {
+		const qrg_mode_t& m = i->second;
+		if (active_modem->get_mode() != m.mode)
+			init_modem_sync(m.mode);
+		qsy(m.rfcarrier, m.carrier);
+	}
+
+	free(text);
+}
+
+static fre_t rst("^[1-5][1-9]{2}$", REG_EXTENDED | REG_NOSUB);
+static fre_t loc("[a-r]{2}[[:digit:]]{2}([a-x]{2})?", REG_EXTENDED | REG_ICASE);
+static fre_t call("([[:alnum:]]?[[:alpha:]/]+[[:digit:]]+[[:alnum:]/]+)", REG_EXTENDED);
+
+void FTextView::handle_qso_data(int start, int end)
+{
+	char* s = get_word(start, end);
+	if (!s)
+		return;
+	char* p = s;
+
+	Fl_Input* target = 0;
+
+	if (rst.match(s))
+		target = inpRstIn;
+	else if (loc.match(s))
+		target = inpLoc;
+	else if (call.match(s)) { // point p to substring
+		const regmatch_t& offsets = call.suboff()[1];
+		p = s + offsets.rm_so;
+		*(s + offsets.rm_eo) = '\0';
+		target = inpCall;
+	}
+	else if (count_if(s, s + strlen(s), static_cast<int(*)(int)>(isdigit)))
+		target = inpQth;
+	else
+		target = *inpName->value() ? inpQth : inpName;
+
+	target->value(p);
+	target->do_callback();
+	free(s);
+}
+
+void FTextView::handle_context_menu(void)
+{
+	// toggle visibility of quick entry items
+	if (view_menu[RX_MENU_QUICK_ENTRY].value()) {
+		view_menu[RX_MENU_CALL].flags &= ~FL_MENU_DIVIDER;
+		for (size_t i = RX_MENU_NAME; i <= RX_MENU_RST_IN; i++)
+			view_menu[i].show();
+	}
+	else {
+		view_menu[RX_MENU_CALL].flags |= FL_MENU_DIVIDER;
+		for (size_t i = RX_MENU_NAME; i <= (progStatus.contest ? RX_MENU_LOC : RX_MENU_RST_IN); i++)
+			view_menu[i].hide();
+	}
+
+	set_active(&view_menu[RX_MENU_COPY], tbuf->selected());
+	set_active(&view_menu[RX_MENU_CLEAR], tbuf->length());
+	set_active(&view_menu[RX_MENU_SELECT_ALL], tbuf->length());
+	set_active(&view_menu[RX_MENU_SAVE], tbuf->length());
+	if (wrap)
+		view_menu[RX_MENU_WRAP].set();
+	else
+		view_menu[RX_MENU_WRAP].clear();
+
+	if (progStatus.contest) {
+			view_menu[RX_MENU_SERIAL].show();
+			view_menu[RX_MENU_XCHG].show();
+	} else {
+			view_menu[RX_MENU_SERIAL].hide();
+			view_menu[RX_MENU_XCHG].hide();
+	}
+
+	if (progdefaults.QRZ != QRZNONE)
+		view_menu[RX_MENU_QRZ_THIS].show();
+	else
+		view_menu[RX_MENU_QRZ_THIS].hide();
+	show_context_menu();
+}
+
 /// The context menu handler
 ///
 /// @param val 
 ///
 void FTextView::menu_cb(int val)
 {
-	if (progdefaults.QRZ == 0)
-		++val;
-
+	Fl_Input* input = 0;
 	switch (val) {
-		char *s;
 	case RX_MENU_QRZ_THIS:
 		menu_cb(RX_MENU_CALL);
 		extern void CALLSIGNquery();
 		CALLSIGNquery();
 		break;
 	case RX_MENU_CALL:
-		stopMacroTimer();
-		s = get_word(popx, popy);
-		inpCall->value(s);
-		free(s);
+		input = inpCall;
 		break;
 	case RX_MENU_NAME:
-		s = get_word(popx, popy);
-		inpName->value(s);
-		free(s);
+		input = inpName;
 		break;
 	case RX_MENU_QTH:
-		s = get_word(popx, popy);
-		inpQth->value(s);
-		free(s);
+		input = inpQth;
+		break;
+	case RX_MENU_STATE:
+		input = inpState;
+		break;
+	case RX_MENU_PROVINCE:
+		input = inpVEprov;
+		break;
+	case RX_MENU_COUNTRY:
+		input = inpCountry;
 		break;
 	case RX_MENU_LOC:
-		s = get_word(popx, popy);
-		inpLoc->value(s);
-		free(s);
+		input = inpLoc;
 		break;
 	case RX_MENU_RST_IN:
-		s = get_word(popx, popy);
-		inpRstIn->value(s);
-		free(s);
+		input = inpRstIn;
+		break;
+
+	case RX_MENU_SERIAL:
+		input = inpSerNo;
+		break;
+	case RX_MENU_XCHG:
+		input = inpXchgIn;
 		break;
 
 	case RX_MENU_DIV:
-        {
-                time_t t = time(NULL);
-                struct tm st;
-                localtime_r(&t, &st);
-                char s[64];
-                strftime(s, sizeof(s), "\n<<======= %Y-%m-%d %H:%M:%S %z ========>>\n", &st);
-                add(s, CTRL);
-		break;
-        }
-	case RX_MENU_CLEAR:
-		clear();
+		note_qrg(false, '\n', '\n');
 		break;
 	case RX_MENU_COPY:
 		kf_copy(Fl::event_key(), this);
 		break;
+	case RX_MENU_CLEAR:
+		clear();
+		break;
+	case RX_MENU_SELECT_ALL:
+		tbuf->select(0, tbuf->length());
+		break;
 
 #if 0 //#ifndef NDEBUG
         case RX_MENU_READ:
@@ -595,12 +836,35 @@ void FTextView::menu_cb(int val)
 		saveFile();
 		break;
 
+	case RX_MENU_QUICK_ENTRY:
+		view_menu[val].flags ^= FL_MENU_VALUE;
+		quick_entry = view_menu[val].value();
+		break;
+
 	case RX_MENU_WRAP:
-		view_menu[RX_MENU_WRAP].flags ^= FL_MENU_VALUE;
+		view_menu[val].flags ^= FL_MENU_VALUE;
 		wrap_mode((wrap = !wrap), wrap_col);
 		show_insert_position();
 		break;
 	}
+
+	if (!input)
+		return;
+	char* s = get_word(popx, popy);
+	if (!s)
+		return;
+
+	if (val == RX_MENU_XCHG) { // append
+		input->position(input->size());
+		input->insert(" ", 1);
+		input->insert(s);
+	}
+	else
+		input->value(s);
+	input->do_callback();
+	free(s);
+	if (input == inpCall)
+		stopMacroTimer();
 }
 
 /// Scrolls down if the buffer has been modified and the last line is
@@ -652,21 +916,131 @@ loop:
 	}
 }
 
+const char* FTextView::dxcc_lookup_call(int x, int y)
+{
+	char* s = get_word(x - this->x(), y - this->y());
+	char* mem = s;
+	if (!(s && *s && call.match(s))) {
+		free(s);
+		return 0;
+	}
+
+	const char *name = 0, *date = 0, *qth = 0, *locator = 0, *mode;
+	double lon1, lat1, lon2 = 360.0, lat2 = 360.0, distance, azimuth;
+	static string tip;
+	ostringstream stip;
+	const dxcc* e = 0;
+	cQsoRec* qso = 0;
+	unsigned char qsl;
+
+	// prevent locator-only lookup if Ctrl is held
+	if (!(Fl::event_state() & FL_CTRL) && loc.match(s)) {
+		const vector<regmatch_t>& v = loc.suboff();
+		s += v[0].rm_so;
+		*(s + v[0].rm_eo) = '\0';
+		if (locator2longlat(&lon2, &lat2, s) != RIG_OK)
+			goto ret;
+		e = 0; qsl = 0; qso = 0;
+	}
+	else {
+		e = dxcc_lookup(s);
+		qsl = qsl_lookup(s);
+		qso = SearchLog(s);
+	}
+
+	if (qso) {
+		locator = qso->getField(GRIDSQUARE);
+		if (!(locator && *locator && locator2longlat(&lon2, &lat2, locator) == RIG_OK))
+			lon2 = lat2 = 360.0;
+		name = qso->getField(NAME);
+		date = qso->getField(QSO_DATE);
+		qth  = qso->getField(QTH);
+	}
+	if (e) {
+		if (lon2 == 360.0)
+			lon2 = -e->longitude;
+		if (lat2 == 360.0)
+			lat2 = e->latitude;
+		stip << e->country << " (" << e->continent
+		     << " GMT" << fixed << showpos << setprecision(1) << -e->gmt_offset << noshowpos
+		     << ") CQ-" << e->cq_zone << " ITU-" << e->itu_zone << '\n';
+	}
+
+	if (locator2longlat(&lon1, &lat1, progdefaults.myLocator.c_str()) == RIG_OK &&
+	    qrb(lon1, lat1, lon2, lat2, &distance, &azimuth) == RIG_OK) {
+		stip << "QTE " << fixed << setprecision(0) << azimuth << '\260' << " ("
+		     << azimuth_long_path(azimuth) << '\260' << ")  QRB " << distance << "km ("
+		     << distance_long_path(distance) << "km)\n";
+	}
+	if (name && *name) {
+		stip << "* " << name;
+		if (qth && *qth)
+			stip << ' ' << _("in") << ' ' << qth;
+		stip << '\n';
+	}
+	if (date && *date) {
+		stip << "* " << _("Last QSO") << ": " << date;
+		mode = qso->getField(MODE);
+		if (mode)
+			stip << ' ' << _("in") << ' ' << mode;
+		stip << '\n';
+	}
+	if (qsl) {
+		stip << "* ";
+		for (unsigned char i = 0; i < QSL_END; i++)
+			if (qsl & (1 << i))
+				stip << qsl_names[i] << ' ';
+		stip << '\n';
+	}
+
+ret:
+	free(mem);
+	tip = stip.str();
+	return tip.empty() ? 0 : tip.c_str();
+}
+
+void FTextView::dxcc_tooltip(void* obj)
+{
+	struct point {
+		int x, y;
+		bool operator==(const point& p) { return x == p.x && y == p.y; }
+		bool operator!=(const point& p) { return !(*this == p); }
+	};
+	static point p[3] = { {0, 0}, {0, 0}, {0, 0} };
+
+	memmove(p, p+1, 2 * sizeof(point));
+	p[2].x = Fl::event_x(); p[2].y = Fl::event_y();
+
+	static const char* tip = 0;
+	FTextView* v = reinterpret_cast<FTextView*>(obj);
+	// look up word under cursor if we have been called twice with the cursor
+	// at the same position, and if the cursor was previously somewhere else
+	if (p[2] == p[1] && p[2] != p[0]  &&  ((tip = v->dxcc_lookup_call(p[2].x, p[2].y))))
+		Fl_Tooltip::enter_area(v, p[2].x, p[2].y, 100, 100, tip);
+	else if (p[2] != p[1])
+		Fl_Tooltip::exit(v);
+
+	Fl::repeat_timeout(tip ? Fl_Tooltip::hoverdelay() : v->tooltips.delay / 2.0, dxcc_tooltip, obj);
+}
+
+
 // ----------------------------------------------------------------------------
 
 
 Fl_Menu_Item FTextEdit::edit_menu[] = {
-	{"&Transmit",		0, 0  },
-	{"&Receive",		0, 0  },
-	{"Send &image...",	0, 0, 0, FL_MENU_DIVIDER },
-	{"C&lear",		0, 0, },
-	{"Cu&t",		0, 0, },
-	{"&Copy",		0, 0, },
-	{"&Paste",		0, 0, 0, FL_MENU_DIVIDER },
-	{"Insert &file...",	0, 0, 0, FL_MENU_DIVIDER },
-	{"Word &wrap",		0, 0, 0, FL_MENU_TOGGLE	 } ,
+	{ make_icon_label(_("&Transmit"), tx_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("&Receive"), rx_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("&Abort"), process_stop_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Send &image..."), image_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Cu&t"), edit_cut_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("&Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("&Paste"), edit_paste_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("C&lear"), edit_clear_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Insert &file..."), file_open_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ _("Word &wrap"), 0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL } ,
 	{ 0 }
 };
+static bool edit_init = false;
 
 // needed by our static kf functions, which may restrict editing depending on
 // the transmit cursor position
@@ -686,6 +1060,13 @@ FTextEdit::FTextEdit(int x, int y, int w, int h, const char *l)
 	change_keybindings();
 	ascii_cnt = 0;
 	ascii_chr = 0;
+
+	if (!edit_init) {
+		for (int i = 0; i < edit_menu->size() - 1; i++)
+			if (edit_menu[i].labeltype() == _FL_MULTI_LABEL)
+				set_icon_label(&edit_menu[i]);
+	}
+	edit_init = true;
 }
 
 /// Handles fltk events for this widget.
@@ -723,56 +1104,18 @@ int FTextEdit::handle(int event)
 	case FL_PUSH:
 	{
 		int eb = Fl::event_button();
-		if (eb == FL_RIGHT_MOUSE)
-			break;
+		if (eb == FL_RIGHT_MOUSE) {
+			handle_context_menu();
+			return 1;
+		}
 		else if (eb == FL_MIDDLE_MOUSE && xy_to_position(Fl::event_x(), Fl::event_y(), CHARACTER_POS) < txpos)
 			return 1; // ignore mouse2 text pastes inside the transmitted text
 	}
 	default:
-		return FTextBase::handle(event);
+		break;
 	}
 
-	// handle a right click
-	if (trx_state == STATE_RX)
-		edit_menu[TX_MENU_TX].label("Transmit");
-	else
-		edit_menu[TX_MENU_TX].label("Abort");
-	if (wf->xmtrcv->active())
-		edit_menu[TX_MENU_TX].flags &= ~FL_MENU_INACTIVE;
-	else
-		edit_menu[TX_MENU_TX].flags |= FL_MENU_INACTIVE;
-	if (trx_state == STATE_RX)
-		edit_menu[TX_MENU_RX].flags |= FL_MENU_INACTIVE;
-	else
-		edit_menu[TX_MENU_RX].flags &= ~FL_MENU_INACTIVE;
-	if (active_modem->get_cap() & modem::CAP_IMG)
-		edit_menu[TX_MENU_MFSK16_IMG].flags &= ~FL_MENU_INACTIVE;
-	else
-		edit_menu[TX_MENU_MFSK16_IMG].flags |= FL_MENU_INACTIVE;
-
-	if (tbuf->length())
-		edit_menu[TX_MENU_CLEAR].flags &= ~FL_MENU_INACTIVE;
-	else
-		edit_menu[TX_MENU_CLEAR].flags |= FL_MENU_INACTIVE;
-	if (tbuf->selected()) {
-		edit_menu[TX_MENU_CUT].flags &= ~FL_MENU_INACTIVE;
-		edit_menu[TX_MENU_COPY].flags &= ~FL_MENU_INACTIVE;
-	}
-	else {
-		edit_menu[TX_MENU_CUT].flags |= FL_MENU_INACTIVE;
-		edit_menu[TX_MENU_COPY].flags |= FL_MENU_INACTIVE;
-	}
-	if (insert_position() < txpos)
-		edit_menu[TX_MENU_READ].flags |= FL_MENU_INACTIVE;
-	else
-		edit_menu[TX_MENU_READ].flags &= ~FL_MENU_INACTIVE;
-	if (wrap)
-		edit_menu[TX_MENU_WRAP].flags |= FL_MENU_VALUE;
-	else
-		edit_menu[TX_MENU_WRAP].flags &= ~FL_MENU_VALUE;
-
-	show_context_menu();
-	return 1;
+	return FTextBase::handle(event);
 }
 
 /// @see FTextView::add
@@ -872,8 +1215,8 @@ int FTextEdit::handle_key(int key)
 	{
 		static time_t t[2] = { 0, 0 };
 		static unsigned char i = 0;
-		if (t[i] == time(&t[!i])) { // two presses in a second: reset modem
-			abort_tx();
+		if (t[i] == time(&t[!i])) { // two presses in a second: abort transmission
+			menu_cb(TX_MENU_ABORT);
 			t[i = !i] = 0;
 			return 1;
 		}
@@ -1098,6 +1441,48 @@ int FTextEdit::handle_dnd_drop(void)
 #endif
 }
 
+/// Handles mouse-3 clicks by displaying the context menu
+///
+/// @param val 
+///
+void FTextEdit::handle_context_menu(void)
+{
+	// adjust Abort/Transmit/Receive menu items
+	switch (trx_state) {
+	case STATE_TX:
+		edit_menu[TX_MENU_TX].hide();
+		edit_menu[TX_MENU_RX].show();
+		edit_menu[TX_MENU_ABORT].show();
+		break;
+	case STATE_TUNE:
+		edit_menu[TX_MENU_TX].hide();
+		edit_menu[TX_MENU_RX].show();
+		edit_menu[TX_MENU_ABORT].hide();
+		break;
+	default:
+		edit_menu[TX_MENU_TX].show();
+		edit_menu[TX_MENU_RX].hide();
+		edit_menu[TX_MENU_ABORT].hide();
+		break;
+	}
+
+	bool modify_text_ok = insert_position() >= txpos;
+	bool selected = tbuf->selected();
+	set_active(&edit_menu[TX_MENU_MFSK16_IMG], active_modem->get_cap() & modem::CAP_IMG);
+	set_active(&edit_menu[TX_MENU_CLEAR], tbuf->length());
+	set_active(&edit_menu[TX_MENU_CUT], selected && modify_text_ok);
+	set_active(&edit_menu[TX_MENU_COPY], selected);
+	set_active(&edit_menu[TX_MENU_PASTE], modify_text_ok);
+	set_active(&edit_menu[TX_MENU_READ], modify_text_ok);
+
+	if (wrap)
+		edit_menu[TX_MENU_WRAP].set();
+	else
+		edit_menu[TX_MENU_WRAP].clear();
+
+	show_context_menu();
+}
+
 /// The context menu handler
 ///
 /// @param val 
@@ -1106,12 +1491,14 @@ void FTextEdit::menu_cb(int val)
 {
   	switch (val) {
   	case TX_MENU_TX:
- 		if (trx_state == STATE_RX) {
- 			active_modem->set_stopflag(false);
- 			start_tx();
- 		}
- 		else
- 			abort_tx();
+		active_modem->set_stopflag(false);
+		start_tx();
+		break;
+	case TX_MENU_ABORT:
+#ifndef NDEBUG
+		put_status("Don't panic!", 1.0);
+#endif
+		abort_tx();
   		break;
   	case TX_MENU_RX:
  		if (trx_state == STATE_TX) {
@@ -1287,21 +1674,23 @@ int FTextEdit::kf_paste(int c, Fl_Text_Editor_mod* e)
 // ----------------------------------------------------------------------------
 
 Fl_Menu_Item FTextLog::log_menu[] = {
-	{ "C&lear",				0, 0 },
-	{ "&Copy",				0, 0 },
-	{ "Save to &file...",			0, 0, 0, FL_MENU_DIVIDER },
-	{ "Word &wrap",				0, 0, 0, FL_MENU_TOGGLE	 },
+	{ make_icon_label(_("C&lear"), edit_clear_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("&Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Save to &file..."), save_as_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ _("Word &wrap"),	0, 0, 0, FL_MENU_TOGGLE },
 	{ 0 }
 };
-
+static bool log_init = false;
 
 FTextLog::FTextLog(int x, int y, int w, int h, const char* l)
 	: FTextView(x, y, w, h, l)
 {
 	context_menu = log_menu;
-//	highlight_data(0, 0, 0, 0, 0, 0);
-//	delete sbuf;
-//	sbuf = 0;
+	if (!log_init)
+		for (int i = 0; i < log_menu->size() - 1; i++)
+			if (log_menu[i].labeltype() == _FL_MULTI_LABEL)
+				set_icon_label(&log_menu[i]);
+	log_init = true;
 }
 
 FTextLog::~FTextLog() { }
@@ -1322,19 +1711,12 @@ int FTextLog::handle(int event)
 		}
 
 		// handle a right mouse click
-		if (tbuf->length())
-			log_menu[LOG_MENU_CLEAR].flags &= ~FL_MENU_INACTIVE;
-		else
-			log_menu[LOG_MENU_CLEAR].flags |= FL_MENU_INACTIVE;
-
-		if (tbuf->selected())
-			log_menu[LOG_MENU_COPY].flags &= ~FL_MENU_INACTIVE;
-		else
-			log_menu[LOG_MENU_COPY].flags |= FL_MENU_INACTIVE;
+		set_active(&log_menu[LOG_MENU_CLEAR], tbuf->length());
+		set_active(&log_menu[LOG_MENU_COPY], tbuf->selected());
 		if (wrap)
-			log_menu[LOG_MENU_WRAP].flags |= FL_MENU_VALUE;
+			log_menu[LOG_MENU_WRAP].set();
 		else
-			log_menu[LOG_MENU_WRAP].flags &= ~FL_MENU_VALUE;
+			log_menu[LOG_MENU_WRAP].clear();
 
 		show_context_menu();
 		return 1;
